// D import file generated from 'tango/core/BitArray.d'
module tango.core.BitArray;
private import tango.core.BitManip;

struct BitArray
{
    size_t len;
    size_t* ptr;
    this(size_t _len, size_t* _ptr)
{
len = _len;
ptr = _ptr;
}
    this(bool[] bits);
    const @property size_t length()
{
return len;
}

    @property void length(const size_t newlen)
{
if (newlen != len)
{
auto olddim = dim();
auto newdim = (newlen + 31) / 32;
if (newdim != olddim)
{
size_t[] buf = ptr[0..olddim];
buf.length = newdim;
ptr = buf.ptr;
if (newdim & 31)
{
ptr[newdim - 1] &= ~(~0 << (newdim & 31));
}
}
len = newlen;
}
}

    @property size_t dim()
{
return (len + 31) / 32;
}

    @property BitArray dup()
{
BitArray ba;
size_t[] buf = ptr[0..dim].dup;
ba.len = len;
ba.ptr = buf.ptr;
return ba;
}

    debug (UnitTest)
{
    }
    this(this)
{
ptr = ptr;
len = len;
}
    void opAssign(bool[] bits);
    BitArray opSliceAssign(BitArray rhs)
in
{
assert(rhs.len == len);
}
body
{
size_t mDim = len / 32;
ptr[0..mDim] = rhs.ptr[0..mDim];
int rest = cast(int)(len & cast(size_t)31u);
if (rest)
{
uint mask = ~0u << rest;
ptr[mDim] = rhs.ptr[mDim] & ~mask | ptr[mDim] & mask;
}
return this;
}
    void init(void[] target, size_t numbits)
in
{
assert(numbits <= target.length * 8);
assert((target.length & 3) == 0);
}
body
{
ptr = cast(size_t*)target.ptr;
len = numbits;
}
    debug (UnitTest)
{
    }
    @property BitArray reverse()
out(result)
{
assert(result == this);
}
body
{
if (len >= 2)
{
bool t;
size_t lo,hi;
lo = 0;
hi = len - 1;
for (; lo < hi; ++lo , --hi)
{
{
t = this[lo];
this[lo] = this[hi];
this[hi] = t;
}
}
}
return this;
}

    debug (UnitTest)
{
    }
    @property BitArray sort();

    debug (UnitTest)
{
    }
    int opApply(scope int delegate(ref bool) dg);
    int opApply(scope int delegate(ref size_t, ref bool) dg);
    debug (UnitTest)
{
    }
    const bool opEquals(ref const(BitArray) rhs)
{
if (this.length() != rhs.length())
return 0;
const size_t* p1 = this.ptr;
const size_t* p2 = rhs.ptr;
size_t n = this.length / 32;
size_t i;
{
for (i = 0; i < n; ++i)
{
{
if (p1[i] != p2[i])
return 0;
}
}
}
int rest = cast(int)(this.length & cast(size_t)31u);
uint mask = ~(~0u << rest);
return rest == 0 || (p1[i] & mask) == (p2[i] & mask);
}

    debug (UnitTest)
{
    }
    int opCmp(BitArray rhs)
{
auto len = this.length;
if (rhs.length < len)
len = rhs.length;
size_t* p1 = this.ptr;
size_t* p2 = rhs.ptr;
size_t n = len / 32;
size_t i;
{
for (i = 0; i < n; ++i)
{
{
if (p1[i] != p2[i])
{
return p1[i] < p2[i] ? -1 : 1;
}
}
}
}
int rest = cast(int)(len & cast(size_t)31u);
if (rest > 0)
{
uint mask = ~(~0u << rest);
uint v1 = p1[i] & mask;
uint v2 = p2[i] & mask;
if (v1 != v2)
return v1 < v2 ? -1 : 1;
}
return this.length < rhs.length ? -1 : this.length == rhs.length ? 0 : 1;
}
    debug (UnitTest)
{
    }
    void[] opCast()
{
return cast(void[])ptr[0..dim];
}
    debug (UnitTest)
{
    }
    bool opIndex(size_t pos)
in
{
assert(pos < len);
}
body
{
return cast(bool)bt(ptr,pos);
}
    BitArray opCom()
{
auto dim = this.dim();
BitArray result;
result.length = len;
{
for (size_t i = 0;
 i < dim; ++i)
{
result.ptr[i] = ~this.ptr[i];
}
}
if (len & 31)
result.ptr[dim - 1] &= ~(~0 << (len & 31));
return result;
}
    debug (UnitTest)
{
    }
    BitArray opAnd(BitArray rhs)
in
{
assert(len == rhs.length);
}
body
{
auto dim = this.dim();
BitArray result;
result.length = len;
{
for (size_t i = 0;
 i < dim; ++i)
{
result.ptr[i] = this.ptr[i] & rhs.ptr[i];
}
}
return result;
}
    debug (UnitTest)
{
    }
    BitArray opOr(BitArray rhs)
in
{
assert(len == rhs.length);
}
body
{
auto dim = this.dim();
BitArray result;
result.length = len;
{
for (size_t i = 0;
 i < dim; ++i)
{
result.ptr[i] = this.ptr[i] | rhs.ptr[i];
}
}
return result;
}
    debug (UnitTest)
{
    }
    BitArray opXor(BitArray rhs)
in
{
assert(len == rhs.length);
}
body
{
auto dim = this.dim();
BitArray result;
result.length = len;
{
for (size_t i = 0;
 i < dim; ++i)
{
result.ptr[i] = this.ptr[i] ^ rhs.ptr[i];
}
}
return result;
}
    debug (UnitTest)
{
    }
    BitArray opSub(BitArray rhs)
in
{
assert(len == rhs.length);
}
body
{
auto dim = this.dim();
BitArray result;
result.length = len;
{
for (size_t i = 0;
 i < dim; ++i)
{
result.ptr[i] = this.ptr[i] & ~rhs.ptr[i];
}
}
return result;
}
    debug (UnitTest)
{
    }
    BitArray opCat(bool rhs)
{
BitArray result;
result = this.dup;
result.length = len + 1;
result[len] = rhs;
return result;
}
    BitArray opCat_r(bool lhs)
{
BitArray result;
result.length = len + 1;
result[0] = lhs;
{
for (size_t i = 0;
 i < len; ++i)
{
result[1 + i] = this[i];
}
}
return result;
}
    BitArray opCat(BitArray rhs)
{
BitArray result;
result = this.dup();
result ~= rhs;
return result;
}
    debug (UnitTest)
{
    }
    bool opIndexAssign(bool b, size_t pos)
in
{
assert(pos < len);
}
body
{
if (b)
bts(ptr,pos);
else
btr(ptr,pos);
return b;
}
    BitArray opAndAssign(BitArray rhs)
in
{
assert(len == rhs.length);
}
body
{
auto dim = this.dim();
{
for (size_t i = 0;
 i < dim; ++i)
{
ptr[i] &= rhs.ptr[i];
}
}
return this;
}
    debug (UnitTest)
{
    }
    BitArray opOrAssign(BitArray rhs)
in
{
assert(len == rhs.length);
}
body
{
auto dim = this.dim();
{
for (size_t i = 0;
 i < dim; ++i)
{
ptr[i] |= rhs.ptr[i];
}
}
return this;
}
    debug (UnitTest)
{
    }
    BitArray opXorAssign(BitArray rhs)
in
{
assert(len == rhs.length);
}
body
{
auto dim = this.dim();
{
for (size_t i = 0;
 i < dim; ++i)
{
ptr[i] ^= rhs.ptr[i];
}
}
return this;
}
    debug (UnitTest)
{
    }
    BitArray opSubAssign(BitArray rhs)
in
{
assert(len == rhs.length);
}
body
{
auto dim = this.dim();
{
for (size_t i = 0;
 i < dim; ++i)
{
ptr[i] &= ~rhs.ptr[i];
}
}
return this;
}
    debug (UnitTest)
{
    }
    BitArray opCatAssign(bool b)
{
length = len + 1;
this[len - 1] = b;
return this;
}
    debug (UnitTest)
{
    }
    BitArray opCatAssign(BitArray rhs)
{
auto istart = len;
length = len + rhs.length;
{
for (auto i = istart;
 i < len; ++i)
{
this[i] = rhs[i - istart];
}
}
return this;
}
    debug (UnitTest)
{
    }
}
