// D import file generated from 'tango/core/ThreadPool.d'
module tango.core.ThreadPool;
private 
{
    import tango.core.Thread;
    import tango.core.sync.Atomic;
    import tango.core.sync.Mutex;
    import tango.core.sync.Condition;
    import tango.core.Exception;
}
private import tango.stdc.string;

private version = Queued;

template ThreadPool(Args...)
{
class ThreadPool
{
    alias void delegate(Args) JobD;
    this(size_t workers, size_t q_size = 0)
{
q.length = q_size;
q.length = 0;
m = new Mutex;
poolActivity = new Condition(m);
workerActivity = new Condition(m);
flagSet(priority_job,cast(Job*)null);
flagSet(active_jobs,cast(size_t)0);
flagSet(done,false);
{
for (size_t i = 0;
 i < workers; i++)
{
{
auto thread = new Thread(&doJob);
thread.isDaemon = true;
thread.start();
pool ~= thread;
}
}
}
}
    void assign(JobD job, Args args)
{
if (this.pool.length == 0)
{
throw new ThreadPoolException("No workers available!");
}
m.lock();
scope(exit) m.unlock();
auto j = Job(job,args);
flagSet(priority_job,&j);
poolActivity.notify();
while (flagGet(priority_job) !is null)
workerActivity.wait();
}
    bool tryAssign(JobD job, Args args)
{
if (flagGet(active_jobs) >= pool.length)
return false;
assign(job,args);
return true;
}
    void append(JobD job, Args args)
{
if (this.pool.length == 0)
{
throw new ThreadPoolException("No workers available!");
}
m.lock();
q ~= Job(job,args);
m.unlock();
poolActivity.notify();
}
    size_t pendingJobs()
{
m.lock();
scope(exit) m.unlock();
return q.length;
}
    size_t activeJobs()
{
return flagGet(active_jobs);
}
    void wait()
{
m.lock();
while (q.length > 0 || flagGet(active_jobs) > 0)
workerActivity.wait();
m.unlock();
}
    void shutdown()
{
flagSet(done,true);
m.lock();
q.length = 0;
m.unlock();
poolActivity.notifyAll();
foreach (thread; pool)
{
thread.join();
}
pool.length = 0;
m.lock();
m.unlock();
}
    void finish()
{
wait();
shutdown();
}
    private 
{
    Thread[] pool;
    struct Job
{
    JobD dg;
    Args args;
}
    Job[] q;
    Job* priority_job;
    Mutex m;
    Condition poolActivity;
    Condition workerActivity;
    bool done;
    size_t active_jobs;
    void doJob()
{
while (!flagGet(done))
{
m.lock();
while (q.length == 0 && flagGet(priority_job) is null && !flagGet(done))
poolActivity.wait();
if (flagGet(done))
{
m.unlock();
break;
}
Job job;
Job* jobPtr = flagGet(priority_job);
if (jobPtr !is null)
{
job = *jobPtr;
flagSet(priority_job,cast(Job*)null);
workerActivity.notify();
}
else
{
version (Queued)
{
job = q[0];
memmove(q.ptr,q.ptr + 1,(q.length - 1) * (typeof(*q.ptr)).sizeof);
q.length = q.length - 1;
}
else
{
job = q[$ - 1];
q.length = q.length - 1;
}

}
m.unlock();
flagAdd!(size_t)(active_jobs,1);
try
{
job.dg(job.args);
}
catch(Exception ex)
{
}
flagAdd!(size_t)(active_jobs,-1);
m.lock();
workerActivity.notify();
m.unlock();
}
m.lock();
workerActivity.notify();
m.unlock();
}
}
}
}
debug (ThreadPool)
{
    import tango.util.log.Trace;
    import Integer = tango.text.convert.Integer;
    void main(char[][] args);
}
