// D import file generated from 'tango/core/tools/StackTrace.d'
module tango.core.tools.StackTrace;
import tango.core.tools.Demangler;
import tango.core.Runtime;
import tango.core.Thread;
import tango.core.Traits;
import tango.stdc.string;
import tango.stdc.stringz;
import tango.stdc.stdlib;
import tango.core.tools.FrameInfo;
version (Windows)
{
    import tango.core.tools.WinStackTrace;
}
else
{
    import tango.stdc.posix.ucontext;
    import tango.stdc.posix.sys.types;
    import tango.stdc.signal;
    import tango.stdc.stdlib;
}
version (linux)
{
    import tango.core.tools.LinuxStackTrace;
}
version (CatchRecursiveTracing)
{
    __gshared ThreadLocal!(int) recursiveStackTraces;

    shared static this();
}
version (Windows)
{
}
else
{
    struct TraceContext
{
    bool hasContext;
    ucontext_t context;
    pid_t hProcess;
    pthread_t hThread;
}
}
alias size_t function(TraceContext* context, TraceContext* contextOut, size_t* traceBuf, size_t bufLength, int* flags) AddrBacktraceFunc;
__gshared AddrBacktraceFunc addrBacktraceFnc;

alias bool function(ref FrameInfo fInfo, TraceContext* context, char[] buf) SymbolizeFrameInfoFnc;
__gshared SymbolizeFrameInfoFnc symbolizeFrameInfoFnc;

shared static this();
extern (C) void rt_setAddrBacktraceFnc(AddrBacktraceFunc f)
{
addrBacktraceFnc = f;
}

extern (C) void rt_setSymbolizeFrameInfoFnc(SymbolizeFrameInfoFnc f)
{
symbolizeFrameInfoFnc = f;
}

extern (C) Exception.TraceInfo rt_createTraceContext(void* ptr);

alias Exception.TraceInfo function(void* ptr = null) TraceHandler;
extern (C) size_t rt_addrBacktrace(TraceContext* context, TraceContext* contextOut, size_t* traceBuf, size_t bufLength, int* flags)
{
if (addrBacktraceFnc !is null)
{
return addrBacktraceFnc(context,contextOut,traceBuf,bufLength,flags);
}
else
{
return 0;
}
}

extern (C) bool rt_symbolizeFrameInfo(ref FrameInfo fInfo, TraceContext* context, char[] buf)
{
if (symbolizeFrameInfoFnc !is null)
{
return symbolizeFrameInfoFnc(fInfo,context,buf);
}
else
{
return false;
}
}

__gshared int[const(char)[]] internalFuncs;

shared static this();
const(char)[] nameOfFunctionAt(void* addr, char[] buf)
{
FrameInfo fInfo;
fInfo.clear();
fInfo.address = cast(size_t)addr;
if (rt_symbolizeFrameInfo(fInfo,null,buf) && fInfo.func.length)
{
return fInfo.func;
}
else
{
return "function@" ~ ctfe_i2a(cast(size_t)addr);
}
}
const(char)[] nameOfFunctionAt(void* addr)
{
char[1024] buf;
return nameOfFunctionAt(addr,buf).dup;
}
enum AddrPrecision 
{
AllReturn = 0,
TopExact = 1,
AllExact = 3,
}
class BasicTraceInfo : Throwable.TraceInfo
{
    size_t[] traceAddresses;
    size_t[128] traceBuf;
    AddrPrecision addrPrecision;
    TraceContext context;
    this()
{
}
    this(size_t[] traceAddresses, AddrPrecision addrPrecision)
{
this.traceAddresses[] = traceAddresses;
if (traceAddresses.length <= traceBuf.length)
{
traceBuf[0..traceAddresses.length] = traceAddresses;
this.traceAddresses = traceBuf[0..traceAddresses.length];
}
this.addrPrecision = addrPrecision;
}
    void trace(TraceContext* contextIn = null, int skipFrames = 0)
{
int flags;
size_t nFrames = rt_addrBacktrace(contextIn,&context,traceBuf.ptr,traceBuf.length,&flags);
traceAddresses = traceBuf[skipFrames..nFrames];
addrPrecision = cast(AddrPrecision)flags;
if (flags == AddrPrecision.TopExact && skipFrames != 0)
addrPrecision = AddrPrecision.AllReturn;
}
    override int opApply(scope int delegate(ref char[]) dg);

    override int opApply(scope int delegate(ref size_t line, ref char[] func) loopBody)
{
FrameInfo fInfo;
{
for (size_t iframe = 0;
 iframe < traceAddresses.length; ++iframe)
{
{
char[2048] buf;
char[1024] buf2;
fInfo.clear();
fInfo.address = cast(size_t)traceAddresses[iframe];
fInfo.iframe = cast(ptrdiff_t)iframe;
fInfo.exactAddress = addrPrecision & 2 || iframe == 0 && addrPrecision & 1;
rt_symbolizeFrameInfo(fInfo,&context,buf);
auto r = fInfo.func in internalFuncs;
fInfo.internalFunction |= r !is null;
fInfo.func = demangler.demangle(fInfo.func.dup,buf2);
int res = loopBody(fInfo.iframe,fInfo.func);
if (res)
return res;
}
}
}
return 0;
}

    int opApply(scope int delegate(ref FrameInfo fInfo) loopBody)
{
FrameInfo fInfo;
{
for (size_t iframe = 0;
 iframe < traceAddresses.length; ++iframe)
{
{
char[2048] buf;
char[1024] buf2;
fInfo.clear();
fInfo.address = cast(size_t)traceAddresses[iframe];
fInfo.iframe = cast(ptrdiff_t)iframe;
fInfo.exactAddress = addrPrecision & 2 || iframe == 0 && addrPrecision & 1;
rt_symbolizeFrameInfo(fInfo,&context,buf);
auto r = fInfo.func in internalFuncs;
fInfo.internalFunction |= r !is null;
fInfo.func = demangler.demangle(fInfo.func,buf2);
int res = loopBody(fInfo);
if (res)
return res;
}
}
}
return 0;
}
    override string toString();

}
version (linux)
{
    version = LibCBacktrace;
    version = DladdrSymbolification;
    version = ElfSymbolification;
}
version (darwin)
{
    version = LibCBacktrace;
    version = DladdrSymbolification;
}
version (LibCBacktrace)
{
    extern (C) int backtrace(void**, int);

}
size_t defaultAddrBacktrace(TraceContext* context, TraceContext* contextOut, size_t* traceBuf, size_t length, int* flags);
version (DladdrSymbolification)
{
    extern (C) struct Dl_info
{
    char* dli_fname;
    void* dli_fbase;
    char* dli_sname;
    void* dli_saddr;
}

    extern (C) int dladdr(void* addr, Dl_info* info);

    bool dladdrSymbolizeFrameInfo(ref FrameInfo fInfo, TraceContext* context, char[] buf)
{
Dl_info dli;
void* ip = cast(void*)fInfo.address;
if (!fInfo.exactAddress)
--ip;
if (dladdr(ip,&dli))
{
if (dli.dli_fname && dli.dli_fbase)
{
fInfo.offsetImg = cast(ptrdiff_t)ip - cast(ptrdiff_t)dli.dli_fbase;
fInfo.baseImg = cast(size_t)dli.dli_fbase;
fInfo.file = dli.dli_fname[0..strlen(dli.dli_fname)];
}
if (dli.dli_sname && dli.dli_saddr)
{
fInfo.offsetSymb = cast(ptrdiff_t)ip - cast(ptrdiff_t)dli.dli_saddr;
fInfo.baseSymb = cast(size_t)dli.dli_saddr;
fInfo.func = dli.dli_sname[0..strlen(dli.dli_sname)];
}
}
return true;
}
}
version (ElfSymbolification)
{
    version (TangoDoc)
{
    bool elfSymbolizeFrameInfo(ref FrameInfo fInfo, TraceContext* context, char[] buf);
}
else
{
    bool elfSymbolizeFrameInfo(ref FrameInfo fInfo, TraceContext* context, char[] buf);
}
}
bool defaultSymbolizeFrameInfo(ref FrameInfo fInfo, TraceContext* context, char[] buf);
Exception.TraceInfo basicTracer(void* ptr = null);
version (Posix)
{
    version (linux)
{
    version (X86)
{
    version = haveSegfaultTrace;
}
else
{
    version (X86_64)
{
    version = haveSegfaultTrace;
}
}
}
    extern (C) void tango_stacktrace_fault_handler(int sn, siginfo_t* si, void* ctx);

    __gshared sigaction_t fault_action;

    void setupSegfaultTracer();
    version (noSegfaultTrace)
{
}
else
{
    shared static this();
}
}
else
{
    version (Windows)
{
}
else
{
    pragma (msg, "[INFO] SEGFAULT trace not yet implemented for this OS");
}
}
