// D import file generated from 'tango/io/Console.d'
module tango.io.Console;
private import tango.sys.Common;

private 
{
    import tango.io.device.Device;
    import tango.io.stream.Buffered;
}
version (Posix)
{
    private import tango.stdc.posix.unistd;

}
struct Console
{
    version (Win32)
{
    enum immutable(char)[] Eol = "\x0d\x0a";
}
else
{
    enum immutable(char)[] Eol = "\x0a";
}
    class Input
{
    private Bin buffer;

    private bool redirect;

    public alias copyln get;

    private this(Conduit conduit, bool redirected)
{
redirect = redirected;
buffer = new Bin(conduit);
}

    final char[] copyln(bool raw = false)
{
char[] line;
return readln(line,raw) ? line.dup : null;
}

    final bool readln(ref char[] content, bool raw = false);

    final @property InputStream stream()
{
return buffer;
}

    final @property bool redirected()
{
return redirect;
}

    final @property Input redirected(bool yes)
{
redirect = yes;
return this;
}

    final @property InputStream input()
{
return buffer.input;
}

    final @property Input input(InputStream source)
{
buffer.input = source;
return this;
}

}
    class Output
{
    private Bout buffer;

    private bool redirect;

    public alias append opCall;

    public alias flush opCall;

    private this(Conduit conduit, bool redirected)
{
redirect = redirected;
buffer = new Bout(conduit);
}

    final Output append(const(char[]) x)
{
buffer.append(x.ptr,x.length);
return this;
}

    final Output append(Object other)
{
return append(other.toString());
}

    final @property Output newline()
{
buffer.append(Eol);
if (redirect is false)
buffer.flush();
return this;
}

    final Output flush()
{
buffer.flush();
return this;
}

    final @property OutputStream stream()
{
return buffer;
}

    final @property bool redirected()
{
return redirect;
}

    final @property Output redirected(bool yes)
{
redirect = yes;
return this;
}

    final @property OutputStream output()
{
return buffer.output;
}

    final @property Output output(OutputStream sink)
{
buffer.output = sink;
return this;
}

}
    class Conduit : Device
{
    private bool redirected = false;

    override immutable(char)[] toString()
{
return "<console>";
}

    version (Win32)
{
    private wchar[] input;

    private wchar[] output;

    this(size_t handle)
{
input = new wchar[](1024 * 1);
output = new wchar[](1024 * 1);
reopen(handle);
}
    private void reopen(size_t handle_)
{
static const DWORD[] id = [cast(DWORD)-10,cast(DWORD)-11,cast(DWORD)-12];
static const char[][] f = ["CONIN$\x00","CONOUT$\x00","CONOUT$\x00"];
assert(handle_ < 3);
io.handle = GetStdHandle(id[handle_]);
if (io.handle is null || io.handle is INVALID_HANDLE_VALUE)
io.handle = CreateFileA(cast(PCHAR)f[handle_].ptr,GENERIC_READ | GENERIC_WRITE,FILE_SHARE_READ | FILE_SHARE_WRITE,null,OPEN_EXISTING,0,cast(HANDLE)0);
if (io.handle != INVALID_HANDLE_VALUE)
{
DWORD mode;
if (!GetConsoleMode(io.handle,&mode))
redirected = (io.track = true);
}
}

    version (Win32SansUnicode)
{
}
else
{
    override size_t write(const(void)[] src)
{
if (redirected)
return super.write(src);
else
{
DWORD i = src.length;
if (i is 0)
return 0;
if (output.length < i)
output.length = i;
i = MultiByteToWideChar(CP_UTF8,0,cast(PCHAR)src.ptr,i,output.ptr,output.length);
{
for (wchar* p = output.ptr,end = output.ptr + i; p < end; p += i)
{
{
const int MAX = 16 * 1024;
DWORD len = end - p;
if (len > MAX)
{
len = MAX;
if ((p[len - 1] & 64512) is 56320)
--len;
}
if (!WriteConsoleW(io.handle,p,len,&i,null))
error();
}
}
}
return src.length;
}
}

}
    version (Win32SansUnicode)
{
}
else
{
    protected override size_t read(void[] dst)
{
if (redirected)
return super.read(dst);
else
{
DWORD i = dst.length / 4;
assert(i);
if (i > input.length)
i = input.length;
if (!ReadConsoleW(io.handle,input.ptr,i,&i,null))
error();
if (i is 0)
return Eof;
i = WideCharToMultiByte(CP_UTF8,0,input.ptr,i,cast(PCHAR)dst.ptr,dst.length,null,null);
if (i is 0)
error();
return i;
}
}


}
}
else
{
    private this(size_t handle)
{
this.handle = cast(Handle)handle;
redirected = isatty(cast(int)handle) is 0;
}

}
}
}
static __gshared Console.Input Cin;

static __gshared Console.Output Cout;

static __gshared Console.Output Cerr;

shared static this();
debug (Console)
{
    void main()
{
Cout("hello world").newline;
}
}
