// D import file generated from 'tango/io/FileSystem.d'
module tango.io.FileSystem;
private import tango.sys.Common;

private import tango.io.FilePath;

private import tango.core.Exception;

private import tango.io.Path;

version (Win32)
{
    private import Text = tango.text.Util;

    private extern (Windows) DWORD GetLogicalDriveStringsA(DWORD, LPSTR);


    private import tango.stdc.stringz;

    enum 
{
FILE_DEVICE_DISK = 7,
IOCTL_DISK_BASE = FILE_DEVICE_DISK,
METHOD_BUFFERED = 0,
FILE_READ_ACCESS = 1,
}
    uint CTL_CODE(uint t, uint f, uint m, uint a)
{
return t << 16 | a << 14 | f << 2 | m;
}
    const IOCTL_DISK_GET_LENGTH_INFO = CTL_CODE(IOCTL_DISK_BASE,23,METHOD_BUFFERED,FILE_READ_ACCESS);
}
version (Posix)
{
    private import tango.stdc.string;

    private 
{
    import tango.stdc.posix.unistd;
    import tango.stdc.posix.sys.statvfs;
}
    private import tango.io.device.File;

    private import Integer = tango.text.convert.Integer;

}
struct FileSystem
{
    private static void exception(string msg);


    version (Windows)
{
    version (Win32SansUnicode)
{
    private static void windowsPath(const(char)[] path, ref char[] result)
{
result[0..path.length] = path;
result[path.length] = 0;
}


}
else
{
    private static void windowsPath(const(char)[] path, ref wchar[] result)
{
assert(path.length < result.length);
auto i = MultiByteToWideChar(CP_UTF8,0,cast(PCHAR)path.ptr,path.length,result.ptr,result.length);
result[i] = 0;
}


}
    deprecated static void setDirectory(const(char)[] path);


    deprecated static char[] getDirectory();


    @property static char[][] roots()
{
int len;
char[] str;
char[][] roots;
len = GetLogicalDriveStringsA(0,null);
if (len)
{
str = new char[](len);
GetLogicalDriveStringsA(len,cast(PCHAR)str.ptr);
roots = Text.delimit(str[0..$ - 1],"\x00");
}
return roots;
}


    private enum 
{
volumePathBufferLen = MAX_PATH + 6,
}

    private static TCHAR[] getVolumePath(const(char)[] folder, TCHAR[] volPath_, bool trailingBackslash);


    static long freeSpace(const(char)[] folder, bool superuser = false);

    static ulong totalSpace(const(char)[] folder, bool superuser = false);

}
    version (Posix)
{
    deprecated static void setDirectory(const(char)[] path)
{
char[512] tmp = void;
tmp[path.length] = 0;
tmp[0..path.length] = path;
if (tango.stdc.posix.unistd.chdir(tmp.ptr))
exception("Failed to set current directory");
}


    deprecated static char[] getDirectory()
{
char[512] tmp = void;
char* s = tango.stdc.posix.unistd.getcwd(tmp.ptr,tmp.length);
if (s is null)
exception("Failed to get current directory");
auto path = s[0..strlen(s) + 1];
path[$ - 1] = '/';
return path;
}


    @property static char[][] roots();


    static long freeSpace(const(char)[] folder, bool superuser = false)
{
scope fp = new FilePath(folder.dup);
statvfs_t info;
int res = statvfs(fp.native.cString().ptr,&info);
if (res == -1)
exception("freeSpace->statvfs failed:" ~ SysError.lastMsg.idup);
if (superuser)
return cast(long)info.f_bfree * cast(long)info.f_bsize;
else
return cast(long)info.f_bavail * cast(long)info.f_bsize;
}

    static long totalSpace(const(char)[] folder, bool superuser = false)
{
scope fp = new FilePath(folder.dup);
statvfs_t info;
int res = statvfs(fp.native.cString().ptr,&info);
if (res == -1)
exception("totalSpace->statvfs failed:" ~ SysError.lastMsg.idup);
return cast(long)info.f_blocks * cast(long)info.f_frsize;
}

}
}
debug (FileSystem)
{
    import tango.io.Stdout;
    static void foo(FilePath path)
{
Stdout("all: ")(path).newline;
Stdout("path: ")(path.path).newline;
Stdout("file: ")(path.file).newline;
Stdout("folder: ")(path.folder).newline;
Stdout("name: ")(path.name).newline;
Stdout("ext: ")(path.ext).newline;
Stdout("suffix: ")(path.suffix).newline.newline;
}

    void main()
{
Stdout.formatln("dir: {}",FileSystem.getDirectory);
auto path = new FilePath(".");
foo(path);
path.set("..");
foo(path);
path.set("...");
foo(path);
path.set("/x/y/.file");
foo(path);
path.suffix = ".foo";
foo(path);
path.set("file.bar");
path.absolute("c:/prefix");
foo(path);
path.set("arf/test");
foo(path);
path.absolute("c:/prefix");
foo(path);
path.name = "foo";
foo(path);
path.suffix = ".d";
path.name = path.suffix;
foo(path);
}
}
