// D import file generated from 'tango/io/Path.d'
module tango.io.Path;
private import tango.sys.Common;

public import tango.time.Time;

private import tango.io.model.IFile;

public import tango.core.Exception;

private import tango.stdc.string;

private import tango.core.Octal;

version (Win32)
{
    version (Win32SansUnicode)
{
    private extern (C) int strlen(const char* s);


    private alias WIN32_FIND_DATA FIND_DATA;

}
else
{
    private extern (C) int wcslen(const wchar* s);


    private alias WIN32_FIND_DATAW FIND_DATA;

}
}
version (Posix)
{
    private import tango.stdc.stdio;

    private import tango.stdc.string;

    private import tango.stdc.posix.utime;

    private import tango.stdc.posix.dirent;

}
package struct FS
{
    struct Stamps
{
    Time created;
    Time accessed;
    Time modified;
}
    struct Listing
{
    const(char)[] folder;
    bool allFiles;
    int opApply(scope int delegate(ref FileInfo) dg);
}
    static void exception(const(char)[] filename)
{
exception(filename[0..$ - 1] ~ ": ",SysError.lastMsg);
}

    static void exception(const(char)[] prefix, const(char)[] error);

    static inout(char)[] padded(inout(char)[] path, char c = '/')
{
if (path.length && path[$ - 1] != c)
path = path ~ c;
return path;
}

    static inout(char)[] paddedLeading(inout(char)[] path, char c = '/')
{
if (path.length && path[0] != c)
path = c ~ path;
return path;
}

    static inout(char)[] stripped(inout(char)[] path, char c = '/')
{
if (path.length && path[$ - 1] is c)
path = path[0..$ - 1];
return path;
}

    static char[] join(const(char)[][] paths...);

    static char[] strz(const(char)[] src, char[] dst)
{
auto i = src.length + 1;
if (dst.length < i)
dst.length = i;
dst[0..i - 1] = src;
dst[i - 1] = 0;
return dst[0..i];
}

    version (Win32)
{
    private static wchar[] toString16(wchar[] tmp, const(char)[] path)
{
auto i = MultiByteToWideChar(CP_UTF8,0,cast(PCHAR)path.ptr,path.length,tmp.ptr,tmp.length);
return tmp[0..i];
}


    private static char[] toString(char[] tmp, const(wchar[]) path)
{
auto i = WideCharToMultiByte(CP_UTF8,0,path.ptr,path.length,cast(PCHAR)tmp.ptr,tmp.length,null,null);
return tmp[0..i];
}


    private static bool fileInfo(const(char)[] name, ref WIN32_FILE_ATTRIBUTE_DATA info);


    private static DWORD getInfo(const(char)[] name, ref WIN32_FILE_ATTRIBUTE_DATA info)
{
if (!fileInfo(name,info))
exception(name);
return info.dwFileAttributes;
}


    private static DWORD getFlags(const(char)[] name)
{
WIN32_FILE_ATTRIBUTE_DATA info = void;
return getInfo(name,info);
}


    static bool exists(const(char)[] name)
{
WIN32_FILE_ATTRIBUTE_DATA info = void;
return fileInfo(name,info);
}

    static ulong fileSize(const(char)[] name)
{
WIN32_FILE_ATTRIBUTE_DATA info = void;
getInfo(name,info);
return (cast(ulong)info.nFileSizeHigh << 32) + info.nFileSizeLow;
}

    static bool isWritable(const(char)[] name)
{
return (getFlags(name) & FILE_ATTRIBUTE_READONLY) is 0;
}

    static bool isFolder(const(char)[] name)
{
return (getFlags(name) & FILE_ATTRIBUTE_DIRECTORY) != 0;
}

    static bool isFile(const(char)[] name)
{
return (getFlags(name) & FILE_ATTRIBUTE_DIRECTORY) == 0;
}

    static Stamps timeStamps(const(char)[] name);

    static void timeStamps(const(char)[] name, Time accessed, Time modified);

    static void copy(const(char)[] src, const(char)[] dst);

    static bool remove(const(char)[] name);

    static void rename(const(char)[] src, const(char)[] dst);

    static void createFile(const(char)[] name)
{
createFile(name,null);
}

    static void createFolder(const(char)[] name);

    static int list(const(char)[] folder, scope int delegate(ref FileInfo) dg, bool all = false);

    private static void createFile(const(char)[] name, scope void delegate(HANDLE) dg);


}
    version (Posix)
{
    private static uint getInfo(const(char)[] name, ref stat_t stats)
{
if (posix.stat(name.ptr,&stats))
exception(name);
return stats.st_mode;
}


    static bool exists(const(char)[] name)
{
stat_t stats = void;
return posix.stat(name.ptr,&stats) is 0;
}

    static ulong fileSize(const(char)[] name)
{
stat_t stats = void;
getInfo(name,stats);
return cast(ulong)stats.st_size;
}

    static bool isWritable(const(char)[] name)
{
stat_t stats = void;
return (getInfo(name,stats) & O_RDONLY) is 0;
}

    static bool isFolder(const(char)[] name)
{
stat_t stats = void;
return (getInfo(name,stats) & S_IFMT) is S_IFDIR;
}

    static bool isFile(const(char)[] name)
{
stat_t stats = void;
return (getInfo(name,stats) & S_IFMT) is S_IFREG;
}

    static Stamps timeStamps(const(char)[] name);

    static void timeStamps(const(char)[] name, Time accessed, Time modified)
{
utimbuf time = void;
time.actime = cast(time_t)(accessed - Time.epoch1970).seconds;
time.modtime = cast(time_t)(modified - Time.epoch1970).seconds;
if (utime(name.ptr,&time) is -1)
exception(name);
}

    static void copy(const(char)[] source, const(char)[] dest);

    static bool remove(const(char)[] name)
{
return tango.stdc.stdio.remove(name.ptr) != -1;
}

    static void rename(const(char)[] src, const(char)[] dst)
{
if (tango.stdc.stdio.rename(src.ptr,dst.ptr) is -1)
exception(src);
}

    static void createFile(const(char)[] name)
{
int fd;
fd = posix.open(name.ptr,O_CREAT | O_WRONLY | O_TRUNC,octal!(660));
if (fd is -1)
exception(name);
if (posix.close(fd) is -1)
exception(name);
}

    static void createFolder(const(char)[] name)
{
if (posix.mkdir(name.ptr,octal!(777)))
exception(name);
}

    static int list(const(char)[] folder, scope int delegate(ref FileInfo) dg, bool all = false);

}
}

template PathParser(char_t = char)
{
struct PathParser
{
    package char_t[] fp;

    package 
{
    int end_;
    int ext_;
    int name_;
    int folder_;
    int suffix_;
}
    PathParser parse(char_t[] path)
{
return parse(path,path.length);
}
    @property const PathParser dup()
{
PathParser ret;
ret.fp = fp.dup;
ret.end_ = end_;
ret.ext_ = ext_;
ret.name_ = name_;
ret.folder_ = folder_;
ret.suffix_ = suffix_;
return ret;
}

    inout inout(char_t)[] toString()
{
return fp[0..end_];
}
    @property inout inout(char_t)[] root()
{
return fp[0..folder_];
}

    @property inout inout(char_t)[] folder()
{
return fp[folder_..name_];
}

    @property inout inout(char_t)[] parent()
{
auto p = path;
if (name.length is 0)
{
for (int i = cast(int)p.length - 1;
 --i > 0;)
{
if (p[i] is FileConst.PathSeparatorChar)
{
p = p[0..i];
break;
}
}
}
return FS.stripped(p);
}

    inout inout(char_t)[] pop()
{
return FS.stripped(path);
}
    @property inout inout(char_t)[] name()
{
return fp[name_..suffix_];
}

    @property char_t[] ext()
{
auto x = suffix;
if (x.length)
{
if (ext_ is 0)
foreach (c; x)
{
if (c is '.')
++ext_;
else
break;
}
x = x[ext_..$];
}
return x;
}

    @property inout inout(char_t)[] suffix()
{
return fp[suffix_..end_];
}

    @property inout inout(char_t)[] path()
{
return fp[0..name_];
}

    @property inout inout(char_t)[] file()
{
return fp[name_..end_];
}

    const @property bool isAbsolute()
{
return folder_ > 0 || folder_ < end_ && fp[folder_] is FileConst.PathSeparatorChar;
}

    const @property bool isEmpty()
{
return end_ is 0;
}

    const @property bool isChild()
{
return folder().length > 0;
}

    const bool equals(const(char)[] s)
{
return FS.stripped(s) == FS.stripped(toString());
}

    package PathParser parse(char_t[] path, size_t end)
{
end_ = cast(int)end;
fp = path;
folder_ = 0;
name_ = (suffix_ = -1);
{
for (int i = end_;
 --i >= 0;)
{
switch (fp[i])
{
case FileConst.FileSeparatorChar:
{
if (name_ < 0)
if (suffix_ < 0 && i && fp[i - 1] != '.')
suffix_ = i;
break;
}
case FileConst.PathSeparatorChar:
{
if (name_ < 0)
name_ = i + 1;
break;
}
case '\x5c':
{
FS.exception("unexpected '\\' character in path: ",path[0..end]);
break;
version (Win32)
{
case ':':
{
folder_ = i + 1;
break;
}
}

}
default:
{
break;
}
}
}
}
if (name_ < 0)
name_ = folder_;
if (suffix_ < 0 || suffix_ is name_)
suffix_ = end_;
return this;
}

}
}
bool exists(const(char)[] name)
{
char[512] tmp = void;
return FS.exists(FS.strz(name,tmp));
}
Time modified(const(char)[] name)
{
return timeStamps(name).modified;
}
Time accessed(const(char)[] name)
{
return timeStamps(name).accessed;
}
Time created(const(char)[] name)
{
return timeStamps(name).created;
}
ulong fileSize(const(char)[] name)
{
char[512] tmp = void;
return FS.fileSize(FS.strz(name,tmp));
}
bool isWritable(const(char)[] name)
{
char[512] tmp = void;
return FS.isWritable(FS.strz(name,tmp));
}
bool isFolder(const(char)[] name)
{
char[512] tmp = void;
return FS.isFolder(FS.strz(name,tmp));
}
bool isFile(const(char)[] name)
{
char[512] tmp = void;
return FS.isFile(FS.strz(name,tmp));
}
FS.Stamps timeStamps(const(char)[] name)
{
char[512] tmp = void;
return FS.timeStamps(FS.strz(name,tmp));
}
void timeStamps(const(char)[] name, Time accessed, Time modified)
{
char[512] tmp = void;
FS.timeStamps(FS.strz(name,tmp),accessed,modified);
}
bool remove(const(char)[] name)
{
char[512] tmp = void;
return FS.remove(FS.strz(name,tmp));
}
char[][] remove(char[][] paths);
void createFile(const(char)[] name)
{
char[512] tmp = void;
FS.createFile(FS.strz(name,tmp));
}
void createFolder(const(char)[] name)
{
char[512] tmp = void;
FS.createFolder(FS.strz(name,tmp));
}
void createPath(const(char)[] path);
void rename(const(char)[] src, const(char)[] dst)
{
char[512] tmp1 = void;
char[512] tmp2 = void;
FS.rename(FS.strz(src,tmp1),FS.strz(dst,tmp2));
}
void copy(const(char)[] src, const(char)[] dst)
{
char[512] tmp1 = void;
char[512] tmp2 = void;
FS.copy(FS.strz(src,tmp1),FS.strz(dst,tmp2));
}
FS.Listing children(const(char)[] path, bool all = false)
{
return FS.Listing(path,all);
}
char[][] collate(const(char)[] path, const(char)[] pattern, bool recurse = false);
char[] join(const(char)[][] paths...)
{
return FS.join(paths);
}
char[] standard(char[] path)
{
return replace(path,'\x5c','/');
}
char[] native(char[] path);
inout(char)[] parent(inout(char)[] path)
{
return pop(FS.stripped(path));
}
inout(char)[] pop(inout(char)[] path);
inout(char)[] split(inout(char)[] path, out inout(char)[] head, out inout(char)[] tail);
char[] replace(char[] path, char from, char to);
template parse(char_t)
{
PathParser!(char_t) parse(char_t[] path)
{
PathParser!(char_t) p;
p.parse(path);
return p;
}
}
debug (UnitTest)
{
    }
bool patternMatch(const(char)[] filename, const(char)[] pattern);
debug (UnitTest)
{
    }
char[] normalize(const(char)[] in_path, char[] buf = null);
debug (UnitTest)
{
    }
debug (Path)
{
    import tango.io.Stdout;
    void main();
}
