// D import file generated from 'tango/io/device/Array.d'
module tango.io.device.Array;
private import tango.core.Exception;

private import tango.io.device.Conduit;

extern (C) protected void* memcpy(void* dst, const(void)* src, size_t);


class Array : Conduit,InputBuffer,OutputBuffer,Conduit.Seek
{
    private void[] data;

    private size_t index;

    private size_t extent;

    private size_t dimension;

    private size_t expansion;

    private enum immutable(char)[] overflow = "output buffer is full";

    private enum immutable(char)[] underflow = "input buffer is empty";

    private enum immutable(char)[] eofRead = "end-of-flow while reading";

    private enum immutable(char)[] eofWrite = "end-of-flow while writing";

        this(size_t capacity, size_t growing = 0)
{
assign(new ubyte[](capacity),0);
expansion = growing;
}
    this(void[] data)
{
assign(data,data.length);
}
    this(void[] data, size_t readable)
{
assign(data,readable);
}
    final override immutable(char)[] toString()
{
return "<array>";
}

    final override size_t read(void[] dst)
{
auto content = readable;
if (content)
{
if (content >= dst.length)
content = dst.length;
dst[0..content] = data[index..index + content];
index += content;
}
else
content = IConduit.Eof;
return content;
}

    final override size_t write(const(void)[] src)
{
auto len = src.length;
if (len)
{
if (len > writable)
if (expand(len) < len)
return Eof;
memcpy(&data[extent],src.ptr,len);
extent += len;
}
return len;
}

    const final override size_t bufferSize()
{
return data.length;
}

    override void detach()
{
}

    override long seek(long offset, Anchor anchor = Anchor.Begin);

    Array assign(void[] data)
{
return assign(data,data.length);
}
    Array assign(void[] data, size_t readable)
{
this.data = data;
this.extent = readable;
this.dimension = data.length;
this.expansion = 0;
this.index = 0;
return this;
}
    final void[] assign()
{
return data;
}

    final void[] opSlice(size_t start, size_t end)
{
assert(start <= extent && end <= extent && start <= end);
return data[start..end];
}

    final void[] slice()
{
return data[index..extent];
}

    final void[] slice(size_t size, bool eat = true)
{
if (size > readable)
error(underflow);
auto i = index;
if (eat)
index += size;
return data[i..i + size];
}

    final Array append(const(void)[] src)
{
if (write(src) is Eof)
error(overflow);
return this;
}

    final bool next(scope size_t delegate(const(void)[]) scan)
{
return reader(scan) != IConduit.Eof;
}

    const final @property size_t readable()
{
return extent - index;
}

    const final @property size_t writable()
{
return dimension - extent;
}

    const final @property size_t limit()
{
return extent;
}

    const final size_t capacity()
{
return dimension;
}

    const final size_t position()
{
return index;
}

    final Array clear()
{
index = (extent = 0);
return this;
}

    final override Array flush()
{
return this;
}

    final size_t writer(scope size_t delegate(void[]) dg)
{
auto count = dg(data[extent..dimension]);
if (count != IConduit.Eof)
{
extent += count;
assert(extent <= dimension);
}
return count;
}

    final size_t reader(scope size_t delegate(const(void)[]) dg)
{
auto count = dg(data[index..extent]);
if (count != IConduit.Eof)
{
index += count;
assert(index <= extent);
}
return count;
}

    private final size_t expand(size_t size)
{
if (expansion)
{
if (size < expansion)
size = expansion;
dimension += size;
data.length = dimension;
}
return writable;
}


    private static template convert(T)
{
T[] convert(void[] x)
{
return (cast(T*)x.ptr)[0..x.length / T.sizeof];
}
}


}
debug (Array)
{
    import tango.io.Stdout;
    void main()
{
auto b = new Array(6,10);
b.seek(0);
b.write("fubar");
Stdout.formatln("extent {}, pos {}, read {}, bufsize {}",b.limit,b.position,cast(char[])b.slice,b.bufferSize);
b.write("fubar");
Stdout.formatln("extent {}, pos {}, read {}, bufsize {}",b.limit,b.position,cast(char[])b.slice,b.bufferSize);
}
}
