// D import file generated from 'tango/io/device/File.d'
module tango.io.device.File;
private import tango.sys.Common;

private import tango.io.device.Device;

private import stdc = tango.stdc.stringz;

private import tango.core.Octal;

version (Win32)
{
    private import Utf = tango.text.convert.Utf;

}
else
{
    private import tango.stdc.posix.unistd;

}
class File : Device,Device.Seek,Device.Truncate
{
    public alias Device.read read;

    public alias Device.write write;

    align (1)struct Style
{
    Access access;
    Open open;
    Share share;
    Cache cache;
}

    enum Access : ubyte
{
Read = 1,
Write = 2,
ReadWrite = 3,
}
    enum Open : ubyte
{
Exists = 0,
Create,
Sedate,
Append,
New,
}
    enum Share : ubyte
{
None = 0,
Read,
ReadWrite,
}
    enum Cache : ubyte
{
None = 0,
Random = 1,
Stream = 2,
WriteThru = 4,
}
    enum Style ReadExisting = {Access.Read,Open.Exists};
    enum Style ReadShared = {Access.Read,Open.Exists,Share.Read};
    enum Style WriteExisting = {Access.Write,Open.Exists};
    enum Style WriteCreate = {Access.Write,Open.Create};
    enum Style WriteAppending = {Access.Write,Open.Append};
    enum Style ReadWriteExisting = {Access.ReadWrite,Open.Exists};
    enum Style ReadWriteCreate = {Access.ReadWrite,Open.Create};
    enum Style ReadWriteOpen = {Access.ReadWrite,Open.Sedate};
    private const(char)[] path_;

    private Style style_;

    this()
{
}
    this(const(char[]) path, Style style = ReadExisting)
{
open(path,style);
}
    const @property Style style()
{
return style_;
}

    override immutable(char)[] toString()
{
return path_.idup;
}

    static void[] get(const(char)[] path, void[] dst = null)
{
scope file = new File(path);
auto len = cast(size_t)file.length;
if (dst.length < len)
{
if (dst is null)
{
dst = new ubyte[](len);
}
else
{
dst.length = len;
}
}
len = file.read(dst);
if (len is file.Eof)
file.error("File.read :: unexpected eof".dup);
return dst[0..len];
}

    static void set(const(char)[] path, void[] content)
{
scope file = new File(path,ReadWriteCreate);
file.write(content);
}

    static void append(const(char)[] path, void[] content)
{
scope file = new File(path,WriteAppending);
file.write(content);
}

    version (Win32)
{
    protected bool open(const(char)[] path, Style style, DWORD addattr);

    void open(const(char[]) path, Style style = ReadExisting)
{
if (!open(path,style,0))
error();
}
    void truncate()
{
truncate(position);
}
    override void truncate(long size)
{
auto s = seek(size);
assert(s is size);
if (!SetEndOfFile(io.handle))
error();
}

    override long seek(long offset, Anchor anchor = Anchor.Begin)
{
long newOffset;
if (anchor is Anchor.Current)
SetFilePointerEx(io.handle,*cast(LARGE_INTEGER*)&io.asynch.Offset,cast(PLARGE_INTEGER)&newOffset,0);
if (!SetFilePointerEx(io.handle,*cast(LARGE_INTEGER*)&offset,cast(PLARGE_INTEGER)&newOffset,anchor))
error();
return *cast(long*)&io.asynch.Offset = newOffset;
}

    @property long position()
{
return *cast(long*)&io.asynch.Offset;
}

    @property long length()
{
long len;
if (!GetFileSizeEx(io.handle,cast(PLARGE_INTEGER)&len))
error();
return len;
}

    void sync()
{
if (!FlushFileBuffers(io.handle))
error();
}
}
    version (Posix)
{
    protected bool open(const(char[]) path, Style style, int addflags, int access = octal!(666))
{
alias int[] Flags;
enum O_LARGEFILE = 32768;
enum Flags Access = [0,O_RDONLY,O_WRONLY,O_RDWR];
enum Flags Create = [0,O_CREAT | O_TRUNC,O_CREAT,O_APPEND | O_CREAT,O_CREAT | O_EXCL];
enum short[] Locks = [F_WRLCK,F_RDLCK];
assert(path);
path_ = path;
style_ = style;
char[512] zero = void;
auto name = stdc.toStringz(path,zero);
auto mode = Access[style.access] | Create[style.open];
handle = posix.open(name,mode | O_LARGEFILE | addflags,access);
if (handle is -1)
return false;
return true;
}

    void open(const(char[]) path, Style style = ReadExisting)
{
if (!open(path,style,0))
error();
}
    void truncate()
{
truncate(position);
}
    override void truncate(long size)
{
if (posix.ftruncate(handle,cast(off_t)size) is -1)
error();
}

    override long seek(long offset, Anchor anchor = Anchor.Begin)
{
long result = posix.lseek(handle,cast(off_t)offset,anchor);
if (result is -1)
error();
return result;
}

    @property long position()
{
return seek(0,Anchor.Current);
}

    @property long length()
{
stat_t stats = void;
if (posix.fstat(handle,&stats))
error();
return cast(long)stats.st_size;
}

    void sync()
{
if (fsync(handle))
error();
}
}
}
debug (File)
{
    import tango.io.Stdout;
    void main()
{
char[10] ff;
auto file = new File("file.d");
auto content = cast(char[])file.load(file);
assert(content.length is file.length);
assert(file.read(ff) is file.Eof);
assert(file.position is content.length);
file.seek(0);
assert(file.position is 0);
assert(file.read(ff) is 10);
assert(file.position is 10);
assert(file.seek(0,file.Anchor.Current) is 10);
assert(file.seek(0,file.Anchor.Current) is 10);
}
}
