// D import file generated from 'tango/io/device/FileMap.d'
module tango.io.device.FileMap;
private import tango.sys.Common;

private 
{
    import tango.io.device.File;
    import tango.io.device.Array;
}
version (Win32)
{
    private extern (Windows) 
{
    BOOL UnmapViewOfFile(LPCVOID);
    BOOL FlushViewOfFile(LPCVOID, DWORD);
    LPVOID MapViewOfFile(HANDLE, DWORD, DWORD, DWORD, DWORD);
    HANDLE CreateFileMappingA(HANDLE, LPSECURITY_ATTRIBUTES, DWORD, DWORD, DWORD, LPCTSTR);
}

}
version (Posix)
{
    private import tango.stdc.posix.sys.mman;

}
class FileMap : Array
{
    private MappedFile file;

    this(const(char[]) path, File.Style style = File.ReadWriteOpen)
{
file = new MappedFile(path,style);
super(file.map);
}
    final ubyte[] resize(long size)
{
auto ret = file.resize(size);
super.assign(ret);
return ret;
}

    override void close()
{
super.close();
if (file)
file.close();
file = null;
}

}
class MappedFile
{
    private File host;

    this(const(char[]) path, File.Style style = File.ReadWriteOpen)
{
host = new File(path,style);
}
    final @property long length()
{
return host.length;
}

    final @property const(char)[] path()
{
return host.toString();
}

    final ubyte[] resize(long size)
{
host.truncate(size);
return map;
}

    version (Win32)
{
    private void* base;

    private HANDLE mmFile;

    final @property ubyte[] map()
{
DWORD flags;
if (base)
reset();
auto size = cast(size_t)host.length;
auto access = host.style.access;
flags = PAGE_READONLY;
if (access & host.Access.Write)
flags = PAGE_READWRITE;
auto handle = cast(HANDLE)host.fileHandle;
mmFile = CreateFileMappingA(handle,null,flags,0,0,null);
if (mmFile is null)
host.error();
flags = FILE_MAP_READ;
if (access & host.Access.Write)
flags |= FILE_MAP_WRITE;
base = MapViewOfFile(mmFile,flags,0,0,0);
if (base is null)
host.error();
return (cast(ubyte*)base)[0..size];
}

    final void close()
{
reset();
if (host)
host.close();
host = null;
}

    private void reset()
{
if (base)
UnmapViewOfFile(base);
if (mmFile)
CloseHandle(mmFile);
mmFile = null;
base = null;
}

    MappedFile flush()
{
if (!FlushViewOfFile(base,0))
host.error();
return this;
}
}
    version (Posix)
{
    private void* base;

    private size_t size;

    final @property ubyte[] map()
{
if (base)
reset();
size = cast(size_t)host.length;
int flags = MAP_SHARED;
int protection = PROT_READ;
auto access = host.style.access;
if (access & host.Access.Write)
protection |= PROT_WRITE;
base = mmap(null,size,protection,flags,host.fileHandle,0);
if (base is MAP_FAILED)
{
base = null;
host.error();
}
return (cast(ubyte*)base)[0..size];
}

    final void close()
{
reset();
if (host)
host.close();
host = null;
}

    private void reset()
{
if (base)
if (munmap(base,size))
host.error();
base = null;
}

    final MappedFile flush()
{
if (msync(base,size,MS_SYNC | MS_INVALIDATE))
host.error();
return this;
}

}
}
debug (FileMap)
{
    import tango.io.Path;
    void main()
{
auto file = new MappedFile("foo.map");
auto heap = file.resize(1000000);
auto file1 = new MappedFile("foo1.map");
auto heap1 = file1.resize(1000000);
file.close();
remove("foo.map");
file1.close();
remove("foo1.map");
}
}
