// D import file generated from 'tango/io/device/TempFile.d'
module tango.io.device.TempFile;
import Path = tango.io.Path;
import tango.math.random.Kiss;
import tango.io.device.Device;
import tango.io.device.File;
import tango.stdc.stringz;
import tango.core.Octal;
version (Win32)
{
    import tango.sys.Common;
    enum : DWORD
{
FILE_FLAG_OPEN_REPARSE_POINT = 2097152,
}
    version (Win32SansUnicode)
{
    import tango.sys.Common;
    char[] GetTempPath();
}
else
{
    import tango.sys.Common;
    char[] GetTempPath();
}
    @property bool reparseSupported();

}
else
{
    version (Posix)
{
    import tango.stdc.posix.pwd;
    import tango.stdc.posix.unistd;
    import tango.stdc.posix.sys.types;
    import tango.stdc.posix.sys.stat;
    import tango.sys.consts.fcntl;
    import tango.stdc.posix.stdlib;
    import tango.stdc.string;
}
}
class TempFile : File
{
    enum Transience : ubyte
{
Transient,
Permanent,
}
    align (1)struct TempStyle
{
    Transience transience;
    ubyte attempts = 10;
}

    static const __gshared TempStyle Transient = {Transience.Transient};

    static const __gshared TempStyle Permanent = {Transience.Permanent};

    private char[] _path;

    private TempStyle _style;

    this(TempStyle style = TempStyle.init)
{
open(style);
}
    this(const(char)[] prefix, TempStyle style = TempStyle.init)
{
open(prefix,style);
}
    TempStyle tempStyle()
{
return _style;
}
    private void open(TempStyle style)
{
open(tempPath(),style);
}

    private void open(const(char)[] prefix, TempStyle style)
{
{
for (ubyte i = style.attempts;
 i--;)
{
{
if (openTempFile(Path.join(prefix,randomName()),style))
return ;
}
}
}
error("could not create temporary file");
}

    version (Win32)
{
    private static enum DEFAULT_LENGTH = 6;


    private static enum DEFAULT_PREFIX = "~t";


    private static enum DEFAULT_SUFFIX = ".tmp";


    private static enum JUNK_CHARS = "abcdefghijklmnopqrstuvwxyz0123456789";


    public static char[] tempPath()
{
return GetTempPath();
}


    private bool openTempFile(const(char)[] path, TempStyle style)
{
Style filestyle = {Access.ReadWrite,Open.New,Share.None,Cache.None};
DWORD attr;
attr = reparseSupported ? FILE_FLAG_OPEN_REPARSE_POINT : 0;
if (style.transience == Transience.Transient)
attr |= FILE_FLAG_DELETE_ON_CLOSE;
if (!super.open(path,filestyle,attr))
return false;
_style = style;
return true;
}

}
else
{
    version (Posix)
{
    private static enum DEFAULT_LENGTH = 6;


    private static enum DEFAULT_PREFIX = ".tmp";


    private static enum DEFAULT_SUFFIX = "~";


    private static enum JUNK_CHARS = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";


    public static const(char)[] tempPath()
{
char* ptr = getenv("TMPDIR");
if (ptr is null)
return "/tmp/";
else
return ptr[0..strlen(ptr)].dup;
}


    private bool openTempFile(const(char)[] path, TempStyle style)
{
{
auto parentz = toStringz(Path.parse(path).path);
if (access(parentz,W_OK) == -1)
error("do not have write access to temporary directory");
stat_t sb;
if (stat(parentz,&sb) == -1)
error("could not stat temporary directory");
auto pwe = getpwnam("root");
if (pwe is null)
error("could not get root's uid");
auto root_uid = pwe.pw_uid;
if (!(sb.st_uid == root_uid || sb.st_uid == getuid()))
error("temporary directory owned by neither root nor user");
if ((sb.st_mode & octal!(22)) != 0 && (sb.st_mode & octal!(1000)) == 0)
error("sticky bit not set on world-writable directory");
}
{
Style filestyle = {Access.ReadWrite,Open.New,Share.None,Cache.None};
auto addflags = O_NOFOLLOW;
if (!super.open(path,filestyle,addflags,octal!(600)))
return false;
if (style.transience == Transience.Transient)
{
if (unlink(toStringz(path)) == -1)
error("could not remove transient file");
}
_style = style;
return true;
}
}

}
else
{
    static assert(false,"Unsupported platform");
}
}
    private char[] randomName(size_t length = DEFAULT_LENGTH, const(char)[] prefix = DEFAULT_PREFIX, const(char)[] suffix = DEFAULT_SUFFIX);

    override void detach();

}
version (TempFile_SelfTest)
{
    import tango.io.Console;
    import tango.io.Stdout;
    void main()
{
Stdout("\x0aPlease ensure that the transient file no longer exists once the TempFile\x0aobject is destroyed, and that the permanent file does.  You should also check\x0athe following on both:\x0a\x0a * the file should be owned by you,\x0a * the owner should have read and write permissions,\x0a * no other permissions should be set on the file.\x0a\x0aFor POSIX systems:\x0a\x0a * the temp directory should be owned by either root or you,\x0a * if anyone other than root or you can write to it, the sticky bit should be\x0a   set,\x0a * if the directory is writable by anyone other than root or the user, and the\x0a   sticky bit is *not* set, then creating the temporary file should fail.\x0a\x0aYou might want to delete the permanent one afterwards, too. :)").newline;
Stdout.formatln("Creating a transient file:");
{
scope tempFile = new TempFile;
Stdout.formatln(" .. path: {}",tempFile);
tempFile.write("Transient temp file.");
char[] buffer = new char[](1023);
tempFile.seek(0);
buffer = buffer[0..tempFile.read(buffer)];
Stdout.formatln(" .. contents: \"{}\"",buffer);
Stdout(" .. press Enter to destroy TempFile object.").newline;
Cin.copyln();
}
Stdout.newline;
Stdout.formatln("Creating a permanent file:");
{
scope tempFile = new TempFile(TempFile.Permanent);
Stdout.formatln(" .. path: {}",tempFile);
tempFile.write("Permanent temp file.");
char[] buffer = new char[](1023);
tempFile.seek(0);
buffer = buffer[0..tempFile.read(buffer)];
Stdout.formatln(" .. contents: \"{}\"",buffer);
Stdout(" .. press Enter to destroy TempFile object.").flush;
Cin.copyln();
}
Stdout("\x0aDone.").newline;
}
}
