// D import file generated from 'tango/io/selector/EpollSelector.d'
module tango.io.selector.EpollSelector;
version (linux)
{
    public import tango.io.model.IConduit;

    public import tango.io.selector.model.ISelector;

    private import tango.io.selector.AbstractSelector;

    private import tango.sys.Common;

    private import tango.sys.linux.linux;

    private import tango.stdc.errno;

    debug (selector)
{
    private import tango.io.Stdout;

}
    public class EpollSelector : AbstractSelector
{
    alias AbstractSelector.select select;
    public const uint DefaultSize = 64;


    public const uint DefaultMaxEvents = 16;


    private SelectionKey[ISelectable.Handle] _keys;

    private int _epfd = -1;

    private epoll_event[] _events;

    private ISelectionSet _selectionSetIface;

    private int _eventCount = 0;

    ~this()
{
close();
}
    override public void open(uint size = DefaultSize, uint maxEvents = DefaultMaxEvents)
in
{
assert(size > 0);
assert(maxEvents > 0);
}
body
{
_events = new epoll_event[](maxEvents);
_selectionSetIface = new EpollSelectionSet;
_epfd = epoll_create(cast(int)size);
if (_epfd < 0)
{
checkErrno("tango/io/selector/EpollSelector.d",179);
}
}


    override public void close()
{
if (_epfd >= 0)
{
.close(_epfd);
_epfd = -1;
}
_events = null;
_eventCount = 0;
}


    override public size_t count()
{
return _keys.length;
}


    override public void register(ISelectable conduit, Event events, Object attachment = null)
in
{
assert(conduit !is null && conduit.fileHandle() >= 0);
}
body
{
auto key = conduit.fileHandle() in _keys;
if (key !is null)
{
epoll_event event;
key.events = events;
key.attachment = attachment;
event.events = events;
event.data.ptr = cast(void*)key;
if (epoll_ctl(_epfd,EPOLL_CTL_MOD,conduit.fileHandle(),&event) != 0)
{
checkErrno("tango/io/selector/EpollSelector.d",255);
}
}
else
{
epoll_event event;
SelectionKey newkey = SelectionKey(conduit,events,attachment);
event.events = events;
_keys[conduit.fileHandle()] = newkey;
auto x = conduit.fileHandle in _keys;
event.data.ptr = cast(void*)x;
if (epoll_ctl(_epfd,EPOLL_CTL_ADD,conduit.fileHandle(),&event) != 0)
{
_keys.remove(conduit.fileHandle);
checkErrno("tango/io/selector/EpollSelector.d",281);
}
}
}


    override public void unregister(ISelectable conduit);


    override public int select(TimeSpan timeout);


    protected class EpollSelectionSet : ISelectionSet
{
    public size_t length()
{
return _events.length;
}

    public int opApply(scope int delegate(ref SelectionKey) dg);

}

    override public ISelectionSet selectedSet()
{
return _eventCount > 0 ? _selectionSetIface : null;
}


    override public SelectionKey key(ISelectable conduit)
{
if (conduit !is null)
{
if (auto k = conduit.fileHandle in _keys)
{
return *k;
}
}
return SelectionKey.init;
}


    int opApply(scope int delegate(ref SelectionKey) dg);
    }

}
