// D import file generated from 'tango/io/selector/SelectSelector.d'
module tango.io.selector.SelectSelector;
public import tango.io.model.IConduit;

private import core.time;

public import tango.io.selector.model.ISelector;

private import tango.io.selector.AbstractSelector;

private import tango.io.selector.SelectorException;

private import tango.sys.Common;

private import tango.stdc.errno;

debug (selector)
{
    private import tango.io.Stdout;

    private import tango.text.convert.Integer;

}
version (Windows)
{
    private import tango.core.Thread;

    private 
{
    struct fd_set
{
}
    extern (Windows) int select(int nfds, fd_set* readfds, fd_set* writefds, fd_set* errorfds, timeval* timeout);

}
}
version (Posix)
{
    private import tango.core.BitArray;

}
public class SelectSelector : AbstractSelector
{
    alias AbstractSelector.select select;
    uint _size;
    private SelectionKey[ISelectable.Handle] _keys;

    private HandleSet _readSet;

    private HandleSet _writeSet;

    private HandleSet _exceptionSet;

    private HandleSet _selectedReadSet;

    private HandleSet _selectedWriteSet;

    private HandleSet _selectedExceptionSet;

    int _eventCount;
    version (Posix)
{
    private ISelectable.Handle _maxfd = cast(ISelectable.Handle)-1;

    public const uint DefaultSize = 1024;


}
else
{
    public const uint DefaultSize = 63;


}
    override public void open(uint size = DefaultSize, uint maxEvents = DefaultSize)
in
{
assert(size > 0);
}
body
{
_size = size;
}


    override public void close()
{
_size = 0;
_keys = null;
_readSet = HandleSet.init;
_writeSet = HandleSet.init;
_exceptionSet = HandleSet.init;
_selectedReadSet = HandleSet.init;
_selectedWriteSet = HandleSet.init;
_selectedExceptionSet = HandleSet.init;
}


    private HandleSet* allocateSet(ref HandleSet set, ref HandleSet selectedSet)
{
if (!set.initialized)
{
set.setup(_size);
selectedSet.setup(_size);
}
return &set;
}

    override public void register(ISelectable conduit, Event events, Object attachment = null);


    override public void unregister(ISelectable conduit);


    override public int select(TimeSpan timeout);


    override public ISelectionSet selectedSet()
{
return _eventCount > 0 ? new SelectSelectionSet(_keys,cast(uint)_eventCount,_selectedReadSet,_selectedWriteSet,_selectedExceptionSet) : null;
}


    override public SelectionKey key(ISelectable conduit)
{
if (conduit !is null)
{
if (auto k = conduit.fileHandle in _keys)
{
return *k;
}
}
return SelectionKey.init;
}


    override public size_t count()
{
return _keys.length;
}


    int opApply(scope int delegate(ref SelectionKey) dg);
}

private class SelectSelectionSet : ISelectionSet
{
    SelectionKey[ISelectable.Handle] _keys;
    uint _eventCount;
    HandleSet _readSet;
    HandleSet _writeSet;
    HandleSet _exceptionSet;
    this(SelectionKey[ISelectable.Handle] keys, uint eventCount, HandleSet readSet, HandleSet writeSet, HandleSet exceptionSet)
{
_keys = keys;
_eventCount = eventCount;
_readSet = readSet;
_writeSet = writeSet;
_exceptionSet = exceptionSet;
}
    @property size_t length()
{
return _eventCount;
}

    int opApply(scope int delegate(ref SelectionKey) dg);
}

version (Windows)
{
    private struct HandleSet
{
    const uint DefaultSize = 63;

    uint[] _buffer;
    void setup(uint size = DefaultSize)
{
_buffer = new uint[](1 + size);
_buffer[0] = 0;
}
    @property bool initialized()
{
return _buffer.length > 0;
}

    @property uint length()
{
return _buffer[0];
}

    void set(ISelectable.Handle handle)
in
{
assert(handle);
}
body
{
if (!isSet(handle))
{
if (++_buffer[0] >= _buffer.length)
{
_buffer.length = _buffer[0] + 1;
}
_buffer[_buffer[0]] = cast(uint)handle;
}
}
    void clear(ISelectable.Handle handle)
{
{
for (uint i = 1;
 i <= _buffer[0]; ++i)
{
{
if (_buffer[i] == cast(uint)handle)
{
if (i != _buffer[0])
{
_buffer[i] = _buffer[_buffer[0]];
}
_buffer[0]--;
return ;
}
}
}
}
}
    HandleSet copy(HandleSet handleSet)
{
if (handleSet._buffer.length > _buffer.length)
{
_buffer.length = handleSet._buffer[0] + 1;
}
_buffer[] = handleSet._buffer[0.._buffer.length];
return this;
}
    public bool isSet(ISelectable.Handle handle)
{
if (_buffer.length == 0)
return false;
uint* start;
uint* stop;
{
for (start = _buffer.ptr + 1 , stop = start + _buffer[0]; start != stop; start++)
{
{
if (*start == cast(uint)handle)
return true;
}
}
}
return false;
}

    public fd_set* opCast()
{
return cast(fd_set*)_buffer.ptr;
}

    debug (selector)
{
    void dump(const(char)[] name = null)
{
if (_buffer !is null && _buffer.length > 0 && _buffer[0] > 0)
{
const(char)[] handleStr = new char[](16);
const(char)[] handleListStr;
bool isFirst = true;
if (name is null)
{
name = "HandleSet";
}
{
for (uint i = 1;
 i < _buffer[0]; ++i)
{
{
if (!isFirst)
{
handleListStr ~= ", ";
}
else
{
isFirst = false;
}
handleListStr ~= itoa(handleStr,_buffer[i]);
}
}
}
Stdout.formatln("--- {0}[{1}]: {2}",name,_buffer[0],handleListStr);
}
}
}
}

}
else
{
    version (Posix)
{
    private import tango.core.BitManip;

    private struct HandleSet
{
    const uint DefaultSize = 1024;

    BitArray _buffer;
    void setup(uint size = DefaultSize)
{
if (size < 1024)
size = 1024;
_buffer.length = size;
}
    @property bool initialized()
{
return _buffer.length > 0;
}

    public void set(ISelectable.Handle handle)
{
uint fd = cast(uint)handle;
if (fd >= _buffer.length)
_buffer.length = fd + 1;
_buffer[fd] = true;
}

    public void clear(ISelectable.Handle handle)
{
auto fd = cast(uint)handle;
if (fd < _buffer.length)
_buffer[fd] = false;
}

    HandleSet copy(HandleSet handleSet)
{
if (handleSet._buffer.length != _buffer.length)
_buffer.length = handleSet._buffer.length;
_buffer[] = handleSet._buffer;
return this;
}
    bool isSet(ISelectable.Handle handle)
{
auto fd = cast(uint)handle;
if (fd < _buffer.length)
return _buffer[fd];
return false;
}
    fd_set* opCast()
{
return cast(fd_set*)_buffer.ptr;
}
    debug (selector)
{
    void dump(const(char)[] name = null)
{
if (_buffer !is null && _buffer.length > 0)
{
const(char)[] handleStr = new char[](16);
const(char)[] handleListStr;
bool isFirst = true;
if (name is null)
{
name = "HandleSet";
}
{
for (uint i = 0;
 i < _buffer.length * _buffer[0].sizeof; ++i)
{
{
if (isSet(cast(ISelectable.Handle)i))
{
if (!isFirst)
{
handleListStr ~= ", ";
}
else
{
isFirst = false;
}
handleListStr ~= itoa(handleStr,i);
}
}
}
}
Stdout.formatln("--- {0}: {1}",name,handleListStr);
}
}
}
}

}
}
