// D import file generated from 'tango/io/stream/Buffered.d'
module tango.io.stream.Buffered;
public import tango.io.model.IConduit;

private import tango.io.device.Conduit;

public alias BufferedInput Bin;

public alias BufferedOutput Bout;

extern (C) 
{
    int printf(char*,...);
    private void* memmove(void* dst, const(void)* src, size_t);

}
private enum immutable(char)[] underflow = "input buffer is empty";

private enum immutable(char)[] eofRead = "end-of-flow whilst reading";

private enum immutable(char)[] eofWrite = "end-of-flow whilst writing";

private enum immutable(char)[] overflow = "output buffer is full";

class BufferedInput : InputFilter,InputBuffer
{
    alias flush clear;
    alias InputFilter.input input;
    private void[] data;

    private size_t index;

    private size_t extent;

    private size_t dimension;

        this(InputStream stream)
{
assert(stream);
this(stream,stream.conduit.bufferSize);
}
    this(InputStream stream, size_t capacity)
{
set(new ubyte[](capacity),0);
super(source = stream);
}
    static InputBuffer create(InputStream stream);

    final size_t populate()
{
return writer(&input.read);
}

    final void[] opSlice(size_t start, size_t end)
{
assert(start <= extent && end <= extent && start <= end);
return data[start..end];
}

    final void[] slice()
{
return data[index..extent];
}

    final void[] slice(size_t size, bool eat = true);

    final size_t reader(scope size_t delegate(const(void)[]) dg)
{
auto count = dg(data[index..extent]);
if (count != Eof)
{
index += count;
assert(index <= extent);
}
return count;
}

    public size_t writer(scope size_t delegate(void[]) dg)
{
auto count = dg(data[extent..dimension]);
if (count != Eof)
{
extent += count;
assert(extent <= dimension);
}
return count;
}

    final override size_t read(void[] dst)
{
size_t content = readable;
if (content)
{
if (content >= dst.length)
content = dst.length;
dst[0..content] = data[index..index + content];
index += content;
}
else
if (dst.length > dimension)
content = source.read(dst);
else
{
if (writable is 0)
index = (extent = 0);
if ((content = writer(&source.read)) != Eof && content > 0)
content = read(dst);
}
return content;
}

    final size_t fill(void[] dst, bool exact = false);

    final bool skip(ptrdiff_t size)
{
if (size < 0)
{
size = -size;
if (index >= size)
{
index -= size;
return true;
}
return false;
}
return slice(size) !is null;
}

    final override long seek(long offset, Anchor start = Anchor.Begin)
{
if (start is Anchor.Current)
{
offset -= readable;
auto bpos = offset + limit;
if (bpos >= 0 && bpos < limit)
{
skip(cast(int)bpos - cast(int)position);
auto pos = source.seek(0,Anchor.Current);
if (pos != Eof)
return pos - readable;
return Eof;
}
}
clear();
return source.seek(offset,start);
}

    final bool next(scope size_t delegate(const(void)[]) scan);

    final size_t reserve(size_t space)
{
assert(space < dimension);
if (dimension - index < space)
compress;
return index;
}

    final @property BufferedInput compress()
{
auto r = readable;
if (index > 0 && r > 0)
memmove(&data[0],&data[index],r);
index = 0;
extent = r;
return this;
}

    final size_t drain(OutputStream dst)
{
assert(dst);
size_t ret = reader(&dst.write);
compress;
return ret;
}

    const final @property size_t limit()
{
return extent;
}

    const final size_t capacity()
{
return dimension;
}

    const final @property size_t position()
{
return index;
}

    const final @property size_t readable()
{
return extent - index;
}

    static template convert(T)
{
inout(T)[] convert(inout(void)[] x)
{
return (cast(inout(T)*)x.ptr)[0..x.length / T.sizeof];
}
}

    final override BufferedInput flush()
{
index = (extent = 0);
if (source)
super.flush();
return this;
}

    final @property void input(InputStream source)
{
this.source = source;
}

    final override void[] load(size_t max = size_t.max)
{
load(super.input,super.conduit.bufferSize,max);
return slice();
}

    private size_t load(InputStream src, size_t increment, size_t max);

    private final BufferedInput set(void[] data, size_t readable)
{
this.data = data;
this.extent = readable;
this.dimension = data.length;
this.index = 0;
return this;
}


    @property private const final size_t writable()
{
return dimension - extent;
}



}
class BufferedOutput : OutputFilter,OutputBuffer
{
    alias OutputFilter.output output;
    private void[] data;

    private size_t index;

    private size_t extent;

    private size_t dimension;

        this(OutputStream stream)
{
assert(stream);
this(stream,stream.conduit.bufferSize);
}
    this(OutputStream stream, size_t capacity)
{
set(new ubyte[](capacity),0);
super(sink = stream);
}
    static OutputBuffer create(OutputStream stream);

    final void[] slice()
{
return data[index..extent];
}

    final override size_t write(const(void)[] src)
{
append(src.ptr,src.length);
return src.length;
}

    final BufferedOutput append(const(void)[] src)
{
return append(src.ptr,src.length);
}

    final BufferedOutput append(const(void)* src, size_t length);

    const final @property size_t writable()
{
return dimension - extent;
}

    const final @property size_t limit()
{
return extent;
}

    const final size_t capacity()
{
return dimension;
}

    final bool truncate(size_t length)
{
if (length <= data.length)
{
extent = length;
return true;
}
return false;
}

    static template convert(T)
{
T[] convert(void[] x)
{
return (cast(T*)x.ptr)[0..x.length / T.sizeof];
}
}

    final override BufferedOutput flush();

    final override BufferedOutput copy(InputStream src, size_t max = -1);

    final size_t drain(OutputStream dst)
{
assert(dst);
size_t ret = reader(&dst.write);
compress;
return ret;
}

    final BufferedOutput clear()
{
index = (extent = 0);
return this;
}

    final @property void output(OutputStream sink)
{
this.sink = sink;
}

    final override long seek(long offset, Anchor start = Anchor.Begin)
{
clear();
return super.seek(offset,start);
}

    final size_t writer(scope size_t delegate(void[]) dg)
{
auto count = dg(data[extent..dimension]);
if (count != Eof)
{
extent += count;
assert(extent <= dimension);
}
return count;
}

    private final size_t reader(scope size_t delegate(const(void)[]) dg)
{
auto count = dg(data[index..extent]);
if (count != Eof)
{
index += count;
assert(index <= extent);
}
return count;
}


    @property private const final size_t readable()
{
return extent - index;
}



    private final BufferedOutput set(void[] data, size_t readable)
{
this.data = data;
this.extent = readable;
this.dimension = data.length;
this.index = 0;
return this;
}


    @property private final BufferedOutput compress()
{
size_t r = readable;
if (index > 0 && r > 0)
memmove(&data[0],&data[index],r);
index = 0;
extent = r;
return this;
}



}
debug (Buffered)
{
    void main()
{
auto input = new BufferedInput(null);
auto output = new BufferedOutput(null);
}
}
