// D import file generated from 'tango/io/stream/Zlib.d'
module tango.io.stream.Zlib;
private import tango.util.compress.c.zlib;

private import tango.stdc.stringz;

private import tango.core.Exception;

private import tango.io.device.Conduit;

private import tango.io.model.IConduit;

private import tango.text.convert.Integer;

private enum 
{
CHUNKSIZE = 256 * 1024,
}

private enum 
{
WINDOWBITS_DEFAULT = 15,
}

class ZlibInput : InputFilter
{
    enum Encoding : int
{
Guess,
Zlib,
Gzip,
None,
}
    private 
{
    bool zs_valid = false;
    z_stream zs;
    ubyte[] in_chunk;
}
    this(InputStream stream, Encoding encoding, int windowBits = WINDOWBITS_DEFAULT);
    this(InputStream stream)
{
this(stream,Encoding.init);
}
    private void init(InputStream stream, Encoding encoding, int windowBits);

    ~this()
{
if (zs_valid)
kill_zs();
}
    void reset(InputStream stream, Encoding encoding, int windowBits = WINDOWBITS_DEFAULT);
    void reset(InputStream stream)
{
reset(stream,Encoding.init);
}
    override size_t read(void[] dst);

    override void close()
{
if (zs_valid)
kill_zs();
super.close();
}

    override long seek(long offset, Anchor anchor = Anchor.Begin);

    private void kill_zs()
{
check_valid();
inflateEnd(&zs);
zs_valid = false;
}

    private void check_valid();

}
class ZlibOutput : OutputFilter
{
    enum Level : int
{
Normal = -1,
None = 0,
Fast = 1,
Best = 9,
}
    enum Encoding : int
{
Zlib,
Gzip,
None,
}
    private 
{
    bool zs_valid = false;
    z_stream zs;
    ubyte[] out_chunk;
    size_t _written = 0;
}
    this(OutputStream stream, Level level, Encoding encoding, int windowBits = WINDOWBITS_DEFAULT);
    this(OutputStream stream, Level level = Level.Normal)
{
this(stream,level,Encoding.init);
}
    private void init(OutputStream stream, Level level, Encoding encoding, int windowBits);

    ~this()
{
if (zs_valid)
kill_zs();
}
    void reset(OutputStream stream, Level level, Encoding encoding, int windowBits = WINDOWBITS_DEFAULT);
    void reset(OutputStream stream, Level level = Level.Normal)
{
reset(stream,level,Encoding.init);
}
    override size_t write(const(void)[] src);

    size_t written()
{
return _written;
}
    override void close()
{
if (zs_valid)
commit();
super.close();
}

    void commit();
    override long seek(long offset, Anchor anchor = Anchor.Begin);

    private void kill_zs()
{
check_valid();
deflateEnd(&zs);
zs_valid = false;
}

    private void check_valid();

}
class ZlibClosedException : IOException
{
    this()
{
super("cannot operate on closed zlib stream");
}
}
class ZlibStillOpenException : IOException
{
    this()
{
super("cannot reset an open zlib stream");
}
}
class ZlibException : IOException
{
    this(immutable(char)[] msg)
{
super(msg);
}
    this(int code)
{
super(codeName(code));
}
    this(int code, char* msg)
{
super(codeName(code) ~ ": " ~ fromStringz(msg).idup);
}
    protected immutable(char)[] codeName(int code);

}
debug (UnitTest)
{
    import tango.io.device.Array;
    template check_array(immutable(char)[] FILE = __FILE__,int LINE = __LINE__)
{
void check_array(const(ubyte)[] as, const(ubyte)[] bs, lazy const(char)[] msg)
{
assert(as.length == bs.length,FILE ~ ":" ~ toString(LINE) ~ ": " ~ msg() ~ "array lengths differ (" ~ toString(as.length) ~ " vs " ~ toString(bs.length) ~ ")");
foreach (i, a; as)
{
auto b = bs[i];
assert(a == b,FILE ~ ":" ~ toString(LINE) ~ ": " ~ msg() ~ "arrays differ at " ~ toString(i) ~ " (" ~ toString(cast(int)a) ~ " vs " ~ toString(cast(int)b) ~ ")");
}
}
}
    }
