// D import file generated from 'tango/io/vfs/FileFolder.d'
module tango.io.vfs.FileFolder;
private import tango.io.device.File;

private import Path = tango.io.Path;

private import tango.core.Exception;

public import tango.io.vfs.model.Vfs;

private import tango.io.model.IConduit;

private import tango.time.Time;

class FileFolder : VfsFolder
{
    private const(char)[] path;

    private VfsStats stats;

    this(const(char)[] path, bool create = false)
{
this.path = open(Path.standard(path.dup),create);
}
    private this(const(char)[] path, const(char)[] name)
{
this.path = Path.join(path,name);
}

    private this(FileFolder parent, const(char)[] name, bool create = false)
{
assert(parent);
this.path = open(Path.join(parent.path,name),create);
}

    final @property const(char)[] name()
{
return Path.parse(path).name;
}

    final override string toString()
{
return path.idup;
}

    final void verify(VfsFolder folder, bool mounting)
{
if (mounting && cast(FileFolder)folder)
{
auto src = Path.FS.padded(this.toString());
auto dst = Path.FS.padded(folder.toString());
auto len = src.length;
if (len > dst.length)
len = dst.length;
if (src[0..len] == dst[0..len])
error("folders '" ~ dst.idup ~ "' and '" ~ src.idup ~ "' overlap");
}
}

    final @property VfsFile file(const(char)[] name)
{
return new FileHost(Path.join(path,name));
}

    final @property VfsFolderEntry folder(const(char)[] path)
{
return new FolderHost(this,path);
}

    final VfsFolder clear()
{
Path.remove(Path.collate(path,"*",true));
return this;
}

    final @property bool writable()
{
return Path.isWritable(path);
}

    final @property VfsFolders self()
{
return new FolderGroup(this,false);
}

    final @property VfsFolders tree()
{
return new FolderGroup(this,true);
}

    final int opApply(scope int delegate(ref VfsFolder) dg);

    VfsFolder close(bool commit = true)
{
return this;
}
    @property private FileFolder[] folders(bool collect);


    private char[][] files(ref VfsStats stats, VfsFilter filter = null);

    private const(char)[] error(string msg);

    private const(char)[] open(const(char)[] path, bool create)
{
if (Path.exists(path))
{
if (!Path.isFolder(path))
error("FileFolder.open :: path exists but not as a folder: " ~ path.idup);
}
else
if (create)
Path.createPath(path);
else
error("FileFolder.open :: path does not exist: " ~ path.idup);
return path;
}

}
class FileGroup : VfsFiles
{
    private const(char)[][] group;

    private const(char)[][] hosts;

    private VfsStats stats;

    this(FolderGroup host, VfsFilter filter);
    final int opApply(scope int delegate(ref VfsFile) dg);

    final @property size_t files()
{
return group.length;
}

    final @property ulong bytes()
{
return stats.bytes;
}

}
private class FolderGroup : VfsFolders
{
    private FileFolder[] members;

    private this()
{
}

    private this(FileFolder root, bool recurse)
{
members = root ~ scan(root,recurse);
}

    final int opApply(scope int delegate(ref VfsFolder) dg);

    final @property size_t files();

    final @property ulong bytes();

    final @property size_t folders()
{
if (members.length is 1)
return members[0].stats.folders;
return members.length;
}

    final @property size_t entries()
{
return files + folders;
}

    final VfsFolders subset(const(char)[] pattern);

    final @property FileGroup catalog(const(char)[] pattern);

    final @property FileGroup catalog(VfsFilter filter = null)
{
return new FileGroup(this,filter);
}

    private final FileFolder[] scan(FileFolder root, bool recurse);


}

private class FolderHost : VfsFolderEntry
{
    private const(char)[] path;

    private FileFolder parent;

    private this(FileFolder parent, const(char)[] path)
{
this.path = path;
this.parent = parent;
}

    final VfsFolder create()
{
return new FileFolder(parent,path,true);
}

    final VfsFolder open()
{
return new FileFolder(parent,path,false);
}

    @property bool exists();

}

private class FileHost : VfsFile
{
    private Path.PathParser!(const(char)) path;

    this(const(char)[] path = null)
{
this.path = Path.PathParser!(const(char))(path);
}
    final @property const(char)[] name()
{
return path.file;
}

    final override string toString()
{
return path.toString().idup;
}

    final @property bool exists()
{
return Path.exists(path.toString());
}

    final @property ulong size()
{
return Path.fileSize(path.toString());
}

    final VfsFile create()
{
Path.createFile(path.toString());
return this;
}

    final VfsFile create(InputStream input)
{
create().output.copy(input).close();
return this;
}

    VfsFile copy(VfsFile source);
    final VfsFile move(VfsFile source)
{
copy(source);
source.remove();
return this;
}

    final @property InputStream input()
{
return new File(path.toString());
}

    final @property OutputStream output()
{
return new File(path.toString(),File.WriteExisting);
}

    final VfsFile remove()
{
Path.remove(path.toString());
return this;
}

    final @property VfsFile dup()
{
auto ret = new FileHost;
ret.path = path.dup;
return ret;
}

    final @property Time modified()
{
return Path.timeStamps(path.toString()).modified;
}

}

debug (FileFolder)
{
    import tango.io.Stdout;
    import tango.io.device.Array;
    void main()
{
auto root = new FileFolder("d:/d/import/temp",true);
root.folder("test").create;
root.file("test.txt").create(new Array("hello"));
Stdout.formatln("test.txt.length = {}",root.file("test.txt").size);
root = new FileFolder("c:/");
auto set = root.self;
Stdout.formatln("self.files = {}",set.files);
Stdout.formatln("self.bytes = {}",set.bytes);
Stdout.formatln("self.folders = {}",set.folders);
Stdout.formatln("self.entries = {}",set.entries);
}
}
