// D import file generated from 'tango/io/vfs/VirtualFolder.d'
module tango.io.vfs.VirtualFolder;
private import tango.core.Exception;

private import tango.io.model.IFile;

private import tango.io.vfs.model.Vfs;

private import tango.io.Path;

private import tango.text.Util;

class VirtualFolder : VfsHost
{
    private const(char)[] name_;

    private VfsFile[const(char)[]] files;

    private VfsFolder[const(char)[]] mounts;

    private VfsFolderEntry[const(char)[]] folders;

    private VirtualFolder parent;

    this(const(char)[] name)
{
validate(this.name_ = name);
}
    final @property const(char)[] name()
{
return name_;
}

    final override string toString()
{
return name.idup;
}

    VfsHost mount(VfsFolder folder, const(char)[] name = null);
    VfsHost mount(VfsFolders group);
    VfsHost dismount(VfsFolder folder);
    final VfsHost map(VfsFile file, const(char)[] name)
{
assert(name);
files[name] = file;
return this;
}

    final VfsHost map(VfsFolderEntry folder, const(char)[] name)
{
assert(name);
folders[name] = folder;
return this;
}

    final int opApply(scope int delegate(ref VfsFolder) dg);

    final VfsFolderEntry folder(const(char)[] path)
{
const(char)[] tail;
auto text = head(path,cast(const(char)[])FileConst.PathSeparatorString,tail);
auto child = text in mounts;
if (child)
return child.folder(tail);
auto sym = text in folders;
if (sym is null)
error("'" ~ text.idup ~ "' is not a recognized member of '" ~ name.idup ~ "'");
return *sym;
}

    @property VfsFile file(const(char)[] path)
{
auto tail = locatePrior(path,cast(const(char))FileConst.PathSeparatorChar);
if (tail < path.length)
return folder(path[0..tail]).open().file(path[tail..$]);
auto sym = path in files;
if (sym is null)
error("'" ~ path.idup ~ "' is not a recognized member of '" ~ name.idup ~ "'");
return *sym;
}

    final VfsFolder clear();

    final @property bool writable();

    final @property VfsFolders self()
{
return new VirtualFolders(this,false);
}

    final @property VfsFolders tree()
{
return new VirtualFolders(this,true);
}

    final void verify(VfsFolder folder, bool mounting);

    VfsFolder close(bool commit = true);
    package final string error(string msg);


    private final void validate(const(char)[] name)
{
assert(name);
if (locatePrior(name,'.') != name.length || locatePrior(name,FileConst.PathSeparatorChar) != name.length)
error("'" ~ name.idup ~ "' contains invalid characters");
}


}
private class VirtualFolders : VfsFolders
{
    private VfsFolders[] members;

    private this()
{
}

    private this(VirtualFolder root, bool recurse);

    final int opApply(scope int delegate(ref VfsFolder) dg);

    final @property size_t files();

    final @property ulong bytes();

    final @property size_t folders();

    final @property size_t entries();

    final VfsFolders subset(const(char)[] pattern);

    final @property VfsFiles catalog(const(char)[] pattern);

    final @property VfsFiles catalog(VfsFilter filter = null)
{
return new VirtualFiles(this,filter);
}

}

private class VirtualFiles : VfsFiles
{
    private VfsFiles[] members;

    private this(VirtualFolders host, VfsFilter filter);

    final int opApply(scope int delegate(ref VfsFile) dg);

    final @property size_t files();

    final @property ulong bytes();

}

debug (VirtualFolder)
{
    import tango.io.Stdout;
    import tango.io.vfs.FileFolder;
    void main();
}
