// D import file generated from 'tango/io/vfs/ZipFolder.d'
module tango.io.vfs.ZipFolder;
import Path = tango.io.Path;
import tango.io.device.File;
import tango.io.FilePath;
import tango.io.device.TempFile;
import tango.util.compress.Zip;
import tango.io.model.IConduit;
import tango.io.vfs.model.Vfs;
import tango.time.Time;
debug (ZipFolder)
{
    import tango.io.Stdout;
}
version = Bug_HeapCorruption;
private 
{
    enum EntryType 
{
Dir,
File,
}
    struct Entry
{
    EntryType type;
    union
{
DirEntry dir;
FileEntry file;
}
    const(char)[] fullname;
    const(char)[] name;
    VfsFilterInfo vfsFilterInfo;
    VfsInfo vfsInfo()
{
return &vfsFilterInfo;
}
    void makeVfsInfo();
    @property bool isDir()
{
return type == EntryType.Dir;
}

    @property bool isFile()
{
return type == EntryType.File;
}

    @property ulong fileSize()
in
{
assert(type == EntryType.File);
}
body
{
if (file.zipEntry !is null)
return file.zipEntry.size();
else
if (file.tempFile !is null)
{
assert(file.tempFile.length >= 0);
return cast(ulong)file.tempFile.length;
}
else
return 0;
}

    InputStream openInput();
    OutputStream openOutput();
    void dispose();
    void dispose_children();
}
    struct DirEntry
{
    Entry*[const(char)[]] children;
}
    struct FileEntry
{
    ZipEntry zipEntry;
    TempFile tempFile;
    }
}
class ZipSubFolder : VfsFolder,VfsSync
{
    final @property const(char)[] name()
in
{
assert(valid);
}
body
{
return entry.name;
}

    final override string toString()
in
{
assert(valid);
}
body
{
return entry.fullname.idup;
}

    final @property VfsFile file(const(char)[] path)
in
{
assert(valid);
assert(!Path.parse(path).isAbsolute);
}
body
{
auto fp = Path.parse(path);
auto dir = fp.path;
auto name = fp.file;
if (dir.length > 0 && '/' == dir[$ - 1])
{
dir = dir[0..$ - 1];
}
if (dir.nz())
{
auto dir_ent = this.folder(dir);
auto dir_obj = dir_ent.open();
return dir_obj.file(name);
}
else
{
if (auto file_entry = name in this.entry.dir.children)
{
return new ZipFile(archive,this.entry,*file_entry);
}
else
{
return new ZipFile(archive,this.entry,name);
}
}
}

    final @property VfsFolderEntry folder(const(char)[] path);

    final @property VfsFolders self()
in
{
assert(valid);
}
body
{
return new ZipSubFolderGroup(archive,this,false);
}

    final @property VfsFolders tree()
in
{
assert(valid);
}
body
{
return new ZipSubFolderGroup(archive,this,true);
}

    final int opApply(scope int delegate(ref VfsFolder) dg);

    final VfsFolder clear();

    final @property bool writable()
in
{
assert(valid);
}
body
{
return !archive.readonly;
}

    version (GNU)
{
    override VfsFolder close(bool commit = true)
in
{
assert(valid);
}
body
{
return closeImpl(commit);
}

    override VfsFolder sync()
{
assert(valid);
return syncImpl();
}

}
else
{
    override VfsFolder close(bool commit = true)
in
{
assert(valid);
}
body
{
return closeImpl(commit);
}

    override VfsFolder sync()
in
{
assert(valid);
}
body
{
return syncImpl();
}

}
    protected VfsFolder closeImpl(bool commit = true)
{
if (commit)
sync();
archive = null;
entry = null;
return this;
}

    protected VfsFolder syncImpl()
in
{
assert(valid);
}
body
{
archive.sync();
return this;
}

    final void verify(VfsFolder folder, bool mounting)
in
{
assert(valid);
}
body
{
auto zipfolder = cast(ZipSubFolder)folder;
if (mounting && zipfolder !is null && zipfolder.archive is archive)
{
auto src = this.toString();
auto dst = zipfolder.toString();
auto len = src.length > dst.length ? dst.length : src.length;
if (src[0..len] == dst[0..len])
error("folders \"" ~ dst ~ "\" and \"" ~ src ~ "\" in archive \"" ~ archive.path ~ "\" overlap");
}
}

    final @property ZipFolder archive()
{
return _archive;
}

    private 
{
    ZipFolder _archive;
    Entry* entry;
    VfsStats stats;
    final @property ZipFolder archive(ZipFolder v)
{
return _archive = v;
}

    this(ZipFolder archive, Entry* entry)
{
this.reset(archive,entry);
}
    final void reset(ZipFolder archive, Entry* entry)
in
{
assert(archive !is null);
assert(entry.isDir);
}
out
{
assert(valid);
}
body
{
this.archive = archive;
this.entry = entry;
}

    final @property bool valid()
{
return archive !is null && !archive.closed;
}

    final void enforce_mutable();

    final void mutate()
in
{
assert(valid);
}
body
{
enforce_mutable();
archive.modified = true;
}

    final ZipSubFolder[] folders(bool collect);

    final Entry*[] files(ref VfsStats stats, VfsFilter filter = null);

}
}
class ZipFolder : ZipSubFolder
{
    this(const(char)[] path, bool readonly = false);
    final override VfsFolder close(bool commit = true);

    final override VfsFolder sync();

    final @property bool readonly()
{
return _readonly;
}

    final @property const(char)[] path()
{
return _path;
}

    final @property const(char)[] path(const(char)[] v)
{
return _path = v;
}

    private 
{
    ZipReader zr;
    Entry* root;
    const(char)[] _path;
    bool _readonly;
    bool modified = false;
    final @property bool readonly(bool v)
{
return _readonly = v;
}

    final @property bool closed();

    final @property bool valid();

    final OutputStream mutateStream(OutputStream source)
{
return new EventSeekOutputStream(source,EventSeekOutputStream.Callbacks(null,null,&mutate_write,null));
}

    void mutate_write(size_t bytes, const(void)[] src)
{
if (!(bytes == 0 || bytes == IConduit.Eof))
this.modified = true;
}
    void resetArchive(const(char)[] path, bool readonly = false);
}
}
class ZipFile : VfsFile
{
    final @property const(char)[] name()
in
{
assert(valid);
}
body
{
if (entry)
return entry.name;
else
return name_;
}

    final override string toString()
in
{
assert(valid);
}
body
{
if (entry)
return entry.fullname.idup;
else
return (parent.fullname ~ "/" ~ name_).idup;
}

    final @property bool exists()
in
{
assert(valid);
}
body
{
return !!entry;
}

    final @property ulong size()
in
{
assert(valid);
}
body
{
if (exists)
return entry.fileSize;
else
error("ZipFile.size: cannot reliably determine size of a non-existent file");
assert(false);
}

    final VfsFile copy(VfsFile source);

    final VfsFile move(VfsFile source);

    final VfsFile create();

    final VfsFile create(InputStream stream);

    final VfsFile remove();

    final @property InputStream input()
in
{
assert(valid);
}
body
{
if (exists)
return entry.openInput();
else
error("ZipFile.input: cannot open non-existent file for input; results would not be very useful");
assert(false);
}

    final @property OutputStream output();

    final @property VfsFile dup()
in
{
assert(valid);
}
body
{
if (entry)
return new ZipFile(archive,parent,entry);
else
return new ZipFile(archive,parent,name);
}

    final @property Time modified()
{
return entry.file.zipEntry.info.modified;
}

    private 
{
    ZipFolder archive;
    Entry* entry;
    Entry* parent;
    const(char)[] name_;
    this()
out
{
assert(!valid);
}
body
{
}
    this(ZipFolder archive, Entry* parent, Entry* entry)
in
{
assert(archive !is null);
assert(parent);
assert(parent.isDir);
assert(entry);
assert(entry.isFile);
assert(parent.dir.children[entry.name] is entry);
}
out
{
assert(valid);
}
body
{
this.reset(archive,parent,entry);
}
    this(ZipFolder archive, Entry* parent, const(char)[] name)
in
{
assert(archive !is null);
assert(parent);
assert(parent.isDir);
assert(name.nz());
assert(!(name in parent.dir.children));
}
out
{
assert(valid);
}
body
{
this.reset(archive,parent,name);
}
    final @property bool valid()
{
return archive !is null && !archive.closed;
}

    final void enforce_mutable();

    final void mutate()
in
{
assert(valid);
}
body
{
enforce_mutable();
archive.modified = true;
}

    final void reset(ZipFolder archive, Entry* parent, Entry* entry)
in
{
assert(archive !is null);
assert(parent);
assert(parent.isDir);
assert(entry);
assert(entry.isFile);
assert(parent.dir.children[entry.name] is entry);
}
out
{
assert(valid);
}
body
{
this.parent = parent;
this.archive = archive;
this.entry = entry;
this.name_ = null;
}

    final void reset(ZipFolder archive, Entry* parent, const(char)[] name)
in
{
assert(archive !is null);
assert(parent);
assert(parent.isDir);
assert(name.nz());
assert(!(name in parent.dir.children));
}
out
{
assert(valid);
}
body
{
this.archive = archive;
this.parent = parent;
this.entry = null;
this.name_ = name;
}

    final void close()
in
{
assert(valid);
}
out
{
assert(!valid);
}
body
{
archive = null;
parent = null;
entry = null;
name_ = null;
}

}
}
class ZipSubFolderEntry : VfsFolderEntry
{
    final VfsFolder open()
in
{
assert(valid);
}
body
{
auto entry = name in parent.dir.children;
if (entry)
return new ZipSubFolder(archive,*entry);
else
{
error("ZipSubFolderEntry.open: \"" ~ parent.fullname ~ "/" ~ name ~ "\" does not exist");
assert(false);
}
}

    final VfsFolder create();

    final @property bool exists()
in
{
assert(valid);
}
body
{
return !!(name in parent.dir.children);
}

    private 
{
    ZipFolder archive;
    Entry* parent;
    const(char)[] name;
    this(ZipFolder archive, Entry* parent, const(char)[] name)
in
{
assert(archive !is null);
assert(parent.isDir);
assert(name.nz());
assert(name.single_path_part());
}
out
{
assert(valid);
}
body
{
this.archive = archive;
this.parent = parent;
this.name = name;
}
    final @property bool valid()
{
return archive !is null && !archive.closed;
}

    final void enforce_mutable();

    final void mutate()
in
{
assert(valid);
}
body
{
enforce_mutable();
archive.modified = true;
}

}
}
class ZipSubFolderGroup : VfsFolders
{
    final int opApply(scope int delegate(ref VfsFolder) dg);

    final @property size_t files();

    final @property size_t folders()
in
{
assert(valid);
}
body
{
return members.length;
}

    final @property size_t entries()
in
{
assert(valid);
}
body
{
return files + folders;
}

    final @property ulong bytes();

    final VfsFolders subset(const(char)[] pattern);

    final @property VfsFiles catalog(const(char)[] pattern);

    final @property VfsFiles catalog(VfsFilter filter = null)
in
{
assert(valid);
}
body
{
return new ZipFileGroup(archive,this,filter);
}

    private 
{
    ZipFolder archive;
    ZipSubFolder[] members;
    this(ZipFolder archive, ZipSubFolder root, bool recurse)
out
{
assert(valid);
}
body
{
this.archive = archive;
members = root ~ scan(root,recurse);
}
    this(ZipFolder archive, ZipSubFolder[] members)
out
{
assert(valid);
}
body
{
this.archive = archive;
this.members = members;
}
    final @property bool valid()
{
return archive !is null && !archive.closed;
}

    final ZipSubFolder[] scan(ZipSubFolder root, bool recurse);

}
}
class ZipFileGroup : VfsFiles
{
    final int opApply(scope int delegate(ref VfsFile) dg);

    final @property size_t files()
in
{
assert(valid);
}
body
{
return group.length;
}

    final @property ulong bytes()
in
{
assert(valid);
}
body
{
return stats.bytes;
}

    private 
{
    ZipFolder archive;
    FileEntry[] group;
    VfsStats stats;
    struct FileEntry
{
    Entry* parent;
    Entry* entry;
}
    this(ZipFolder archive, ZipSubFolderGroup host, VfsFilter filter);
    final @property bool valid()
{
return archive !is null && !archive.closed;
}

}
}
private 
{
    void error(const(char)[] msg);
    void mutate_error(const(char)[] method)
{
error(method ~ ": mutating the contents of a ZipFolder is not supported yet; terribly sorry");
}
    bool nz(const(char)[] s)
{
return s.length > 0;
}
    bool zero(const(char)[] s)
{
return s.length == 0;
}
    bool single_path_part(const(char)[] s);
    char[] dir_app(const(char)[] dir, const(char)[] name)
{
return dir ~ (dir[$ - 1] != '/' ? "/" : "") ~ name;
}
    inout(char)[] headTail(inout(char)[] path, out inout(char)[] head, out inout(char)[] tail);
    debug (UnitTest)
{
    }
    private 
{
    import tango.io.device.Conduit;
    private deprecated class DummyInputStream : InputStream
{
    override InputStream input()
{
return null;
}

    override IConduit conduit()
{
return null;
}

    override void close()
{
}

    override size_t read(void[] dst)
{
return IConduit.Eof;
}

    override InputStream flush()
{
return this;
}

    override void[] load(size_t max = -1)
{
return Conduit.load(this,max);
}

    override long seek(long offset, Anchor anchor = cast(Anchor)0)
{
return 0;
}

}


    private deprecated class DummyOutputStream : OutputStream
{
    override OutputStream output()
{
return null;
}

    override IConduit conduit()
{
return null;
}

    override void close()
{
}

    override size_t write(const(void)[] src)
{
return IConduit.Eof;
}

    override OutputStream copy(InputStream src, size_t max = -1)
{
Conduit.transfer(src,this,max);
return this;
}

    override OutputStream flush()
{
return this;
}

    override long seek(long offset, Anchor anchor = cast(Anchor)0)
{
return 0;
}

}


    class EventSeekInputStream : InputStream
{
    struct Callbacks
{
    void delegate() close;
    void delegate() clear;
    void delegate(size_t, void[]) read;
    void delegate(long, long, Anchor) seek;
}
    this(InputStream source, Callbacks callbacks)
in
{
assert(source !is null);
assert(cast(IConduit.Seek)source.conduit !is null);
}
body
{
this.source = source;
this.seeker = source;
this.callbacks = callbacks;
}
    override IConduit conduit()
{
return source.conduit;
}

    InputStream input()
{
return source;
}
    override void close()
{
source.close();
source = null;
seeker = null;
if (callbacks.close)
callbacks.close();
}

    override size_t read(void[] dst)
{
auto result = source.read(dst);
if (callbacks.read)
callbacks.read(result,dst);
return result;
}

    override InputStream flush()
{
source.flush();
if (callbacks.clear)
callbacks.clear();
return this;
}

    override void[] load(size_t max = -1)
{
return Conduit.load(this,max);
}

    override long seek(long offset, Anchor anchor = cast(Anchor)0)
{
auto result = seeker.seek(offset,anchor);
if (callbacks.seek)
callbacks.seek(result,offset,anchor);
return result;
}

    private 
{
    InputStream source;
    InputStream seeker;
    Callbacks callbacks;
    }
}
    class EventSeekOutputStream : OutputStream
{
    struct Callbacks
{
    void delegate() close;
    void delegate() flush;
    void delegate(size_t, const(void)[]) write;
    void delegate(long, long, Anchor) seek;
}
    this(OutputStream source, Callbacks callbacks)
in
{
assert(source !is null);
assert(cast(IConduit.Seek)source.conduit !is null);
}
body
{
this.source = source;
this.seeker = source;
this.callbacks = callbacks;
}
    override IConduit conduit()
{
return source.conduit;
}

    override OutputStream output()
{
return source;
}

    override void close()
{
source.close();
source = null;
seeker = null;
if (callbacks.close)
callbacks.close();
}

    override size_t write(const(void)[] dst)
{
auto result = source.write(dst);
if (callbacks.write)
callbacks.write(result,dst);
return result;
}

    override OutputStream flush()
{
source.flush();
if (callbacks.flush)
callbacks.flush();
return this;
}

    override long seek(long offset, Anchor anchor = cast(Anchor)0)
{
auto result = seeker.seek(offset,anchor);
if (callbacks.seek)
callbacks.seek(result,offset,anchor);
return result;
}

    override OutputStream copy(InputStream src, size_t max = -1)
{
Conduit.transfer(src,this,max);
return this;
}

    private 
{
    OutputStream source;
    OutputStream seeker;
    Callbacks callbacks;
    }
}
    class WrapSeekInputStream : InputStream
{
    this(InputStream source)
in
{
assert(source !is null);
assert(cast(IConduit.Seek)source.conduit !is null);
}
body
{
this.source = source;
this.seeker = source;
this._position = seeker.seek(0,Anchor.Current);
}
    this(InputStream source, long position)
in
{
assert(position >= 0);
}
body
{
this(source);
this._position = position;
}
    override IConduit conduit()
{
return source.conduit;
}

    InputStream input()
{
return source;
}
    override void close()
{
source = null;
seeker = null;
}

    override size_t read(void[] dst)
{
if (seeker.seek(0,Anchor.Current) != _position)
seeker.seek(_position,Anchor.Begin);
auto read = source.read(dst);
if (read != IConduit.Eof)
_position += read;
return read;
}

    override InputStream flush()
{
source.flush();
return this;
}

    override void[] load(size_t max = -1)
{
return Conduit.load(this,max);
}

    override long seek(long offset, Anchor anchor = cast(Anchor)0)
{
seeker.seek(_position,Anchor.Begin);
return _position = seeker.seek(offset,anchor);
}

    private 
{
    InputStream source;
    InputStream seeker;
    long _position;
    }
}
    class WrapSeekOutputStream : OutputStream
{
    this(OutputStream source)
in
{
assert(cast(IConduit.Seek)source.conduit !is null);
}
body
{
this.source = source;
this.seeker = source;
this._position = seeker.seek(0,Anchor.Current);
}
    this(OutputStream source, long position)
in
{
assert(position >= 0);
}
body
{
this(source);
this._position = position;
}
    override IConduit conduit()
{
return source.conduit;
}

    override OutputStream output()
{
return source;
}

    override void close()
{
source = null;
seeker = null;
}

    size_t write(const(void)[] src)
{
if (seeker.seek(0,Anchor.Current) != _position)
seeker.seek(_position,Anchor.Begin);
auto wrote = source.write(src);
if (wrote != IConduit.Eof)
_position += wrote;
return wrote;
}
    override OutputStream copy(InputStream src, size_t max = -1)
{
Conduit.transfer(src,this,max);
return this;
}

    override OutputStream flush()
{
source.flush();
return this;
}

    override long seek(long offset, Anchor anchor = cast(Anchor)0)
{
seeker.seek(_position,Anchor.Begin);
return _position = seeker.seek(offset,anchor);
}

    private 
{
    OutputStream source;
    OutputStream seeker;
    long _position;
    }
}
}
}
