// D import file generated from 'tango/math/Bracket.d'
module tango.math.Bracket;
import tango.math.Math;
import tango.math.IEEE;
private 
{
    template oppositeSigns(T)
{
bool oppositeSigns(T a, T b)
{
return (signbit(a) ^ signbit(b)) != 0;
}
}
    template BracketResult(T,R)
{
struct BracketResult
{
    T xlo;
    T xhi;
    R fxlo;
    R fxhi;
}
}
    public 
{
    template findRoot(T,R)
{
T findRoot(scope R delegate(T) f, T ax, T bx)
{
auto r = findRoot(f,ax,bx,f(ax),f(bx),delegate (BracketResult!(T,R) r)
{
return r.xhi == nextUp(r.xlo);
}
);
return fabs(r.fxlo) <= fabs(r.fxhi) ? r.xlo : r.xhi;
}
}
    private 
{
    template findRoot(T,R)
{
BracketResult!(T,R) findRoot(scope R delegate(T) f, T ax, T bx, R fax, R fbx, scope bool delegate(BracketResult!(T,R) r) tolerance)
in
{
assert(ax <= bx,"Parameters ax and bx out of order.");
assert(ax <>= 0 && bx <>= 0,"Limits must not be NaN");
assert(oppositeSigns(fax,fbx),"Parameters must bracket the root.");
}
body
{
T a = ax,b = bx,d;
R fa = fax,fb = fbx,fd;
void bracket(T c)
{
T fc = f(c);
if (fc == 0)
{
a = c;
fa = fc;
d = c;
fd = fc;
return ;
}
if (oppositeSigns(fa,fc))
{
d = b;
fd = fb;
b = c;
fb = fc;
}
else
{
d = a;
fd = fa;
a = c;
fa = fc;
}
}
T secant_interpolate(T a, T b, T fa, T fb)
{
if (a - b == a && b != 0 || a != 0 && b - a == b)
{
if (a == 0)
a = copysign(0L,b);
else
if (b == 0)
b = copysign(0L,a);
else
if (oppositeSigns(a,b))
return 0;
T c = ieeeMean(a,b);
return c;
}
if (b - a > T.max)
return b / 2 + a / 2;
if (fb - fa > T.max)
return a - (b - a) / 2;
T c = a - fa / (fb - fa) * (b - a);
if (c == a || c == b)
return (a + b) / 2;
return c;
}
T newtonQuadratic(int numsteps)
{
T a0 = fa;
T a1 = (fb - fa) / (b - a);
T a2 = ((fd - fb) / (d - b) - a1) / (d - a);
T c = oppositeSigns(a2,fa) ? a : b;
{
for (int i = 0;
 i < numsteps; ++i)
{
{
T pc = a0 + (a1 + a2 * (c - b)) * (c - a);
T pdc = a1 + a2 * (2 * c - (a + b));
if (pdc == 0)
return a - a0 / a1;
else
c = c - pc / pdc;
}
}
}
return c;
}
if (fa != 0)
{
bracket(secant_interpolate(a,b,fa,fb));
}
int itnum = 1;
int baditer = 1;
T c,e;
R fe;
whileloop:
while (fa != 0 && !tolerance(BracketResult!(T,R)(a,b,fa,fb)))
{
T a0 = a,b0 = b;
{
for (int QQ = 0;
 QQ < 2; ++QQ)
{
{
bool distinct = fa != fb && fa != fd && fa != fe && fb != fd && fb != fe && fd != fe;
if (itnum < 2)
distinct = false;
bool ok = distinct;
if (distinct)
{
real q11 = (d - e) * fd / (fe - fd);
real q21 = (b - d) * fb / (fd - fb);
real q31 = (a - b) * fa / (fb - fa);
real d21 = (b - d) * fd / (fd - fb);
real d31 = (a - b) * fb / (fb - fa);
real q22 = (d21 - q11) * fb / (fe - fb);
real q32 = (d31 - q21) * fa / (fd - fa);
real d32 = (d31 - q21) * fd / (fd - fa);
real q33 = (d32 - q22) * fa / (fe - fa);
c = a + (q31 + q32 + q33);
if (c !<>= 0 || c <= a || c >= b)
{
if (c == a && a - b != a)
{
c = nextUp(a);
}
else
if (c == b && a - b != -b)
{
c = nextDown(b);
}
else
{
ok = false;
}
}
}
if (!ok)
{
c = newtonQuadratic(distinct ? 3 : 2);
if (c !<>= 0 || c <= a || c >= b)
{
c = secant_interpolate(a,b,fa,fb);
}
}
++itnum;
e = d;
fe = fd;
bracket(c);
if (fa == 0 || tolerance(BracketResult!(T,R)(a,b,fa,fb)))
break whileloop;
if (itnum == 2)
continue whileloop;
}
}
}
T u;
R fu;
if (fabs(fa) < fabs(fb))
{
u = a;
fu = fa;
}
else
{
u = b;
fu = fb;
}
c = u - 2 * (fu / (fb - fa)) * (b - a);
if (c == a || c == b || c !<>= 0 || fabs(c - u) > (b - a) / 2)
{
if (a - b == a || b - a == b)
{
if (a > 0 && b < 0 || a < 0 && b > 0)
c = 0;
else
{
if (a == 0)
c = ieeeMean(copysign(0L,b),b);
else
if (b == 0)
c = ieeeMean(copysign(0L,a),a);
else
c = ieeeMean(a,b);
}
}
else
{
c = a + (b - a) / 2;
}
}
e = d;
fe = fd;
bracket(c);
if (fa == 0 || tolerance(BracketResult!(T,R)(a,b,fa,fb)))
break;
if ((a == 0 || b == 0 || fabs(a) >= 0.5 * fabs(b) && fabs(b) >= 0.5 * fabs(a)) && b - a < 0.25 * (b0 - a0))
{
baditer = 1;
continue;
}
if (b - a < 0.25 * (b0 - a0))
baditer = 1;
{
for (int QQ = 0;
 QQ < baditer; ++QQ)
{
{
e = d;
fe = fd;
T w;
if (a > 0 && b < 0 || a < 0 && b > 0)
w = 0;
else
{
T usea = a;
T useb = b;
if (a == 0)
usea = copysign(0L,b);
else
if (b == 0)
useb = copysign(0L,a);
w = ieeeMean(usea,useb);
}
bracket(w);
}
}
}
++baditer;
}
if (fa == 0)
return BracketResult!(T,R)(a,a,fa,fa);
else
if (fb == 0)
return BracketResult!(T,R)(b,b,fb,fb);
else
return BracketResult!(T,R)(a,b,fa,fb);
}
}
    public 
{
    template findMinimum(T,R)
{
T findMinimum(scope R delegate(T) func, T xlo, T xhi, T xinitial, out R funcMin)
in
{
assert(xlo <= xhi);
assert(xinitial >= xlo);
assert(xinitial <= xhi);
assert(func(xinitial) <= func(xlo) && func(xinitial) <= func(xhi));
}
body
{
enum real GOLDENRATIO = 0xc.3910c8d016b005dp-5;
T stepBeforeLast = 0;
T lastStep;
T bestx = xinitial;
R fbest = func(bestx);
T second = xinitial;
R fsecond = fbest;
T third = xinitial;
R fthird = fbest;
int numiter = 0;
for (;;)
{
{
++numiter;
T xmid = 0.5 * (xlo + xhi);
enum real SQRTEPSILON = 3e-10L;
T tol1 = SQRTEPSILON * fabs(bestx);
T tol2 = 2 * tol1;
if (fabs(bestx - xmid) <= tol2 - 0.5 * (xhi - xlo))
{
funcMin = fbest;
return bestx;
}
if (fabs(stepBeforeLast) > tol1)
{
real r = (bestx - second) * (fbest - fthird);
real denom = (bestx - third) * (fbest - fsecond);
real numerator = (bestx - third) * denom - (bestx - second) * r;
denom = 2 * (denom - r);
if (denom > 0)
numerator = -numerator;
denom = fabs(denom);
if (fabs(numerator) >= fabs(0.5 * denom * stepBeforeLast) || numerator <= denom * (xlo - bestx) || numerator >= denom * (xhi - bestx))
{
stepBeforeLast = bestx >= xmid ? xlo - bestx : xhi - bestx;
lastStep = GOLDENRATIO * stepBeforeLast;
}
else
{
stepBeforeLast = lastStep;
lastStep = numerator / denom;
real xtest = bestx + lastStep;
if (xtest - xlo < tol2 || xhi - xtest < tol2)
{
if (xmid - bestx > 0)
lastStep = tol1;
else
lastStep = -tol1;
}
}
}
else
{
stepBeforeLast = bestx >= xmid ? xlo - bestx : xhi - bestx;
lastStep = GOLDENRATIO * stepBeforeLast;
}
T xtest;
if (fabs(lastStep) < tol1 || lastStep !<>= 0)
{
if (lastStep > 0)
lastStep = tol1;
else
lastStep = -tol1;
}
xtest = bestx + lastStep;
R ftest = func(xtest);
if (ftest <= fbest)
{
if (xtest >= bestx)
xlo = bestx;
else
xhi = bestx;
third = second;
fthird = fsecond;
second = bestx;
fsecond = fbest;
bestx = xtest;
fbest = ftest;
}
else
{
if (xtest < bestx)
xlo = xtest;
else
xhi = xtest;
if (ftest < fsecond || second == bestx)
{
third = second;
fthird = fsecond;
second = xtest;
fsecond = ftest;
}
else
if (ftest <= fthird || third == bestx || third == second)
{
third = xtest;
fthird = ftest;
}
}
}
}
}
}
    private debug (UnitTest)
{
        }

}
}
}
}
