// D import file generated from 'tango/math/Probability.d'
module tango.math.Probability;
static import tango.math.ErrorFunction;
private import tango.math.GammaFunction;

private import tango.math.Math;

private import tango.math.IEEE;

real normalDistribution(real a)
{
return tango.math.ErrorFunction.normalDistributionImpl(a);
}
real normalDistributionCompl(real a)
{
return -tango.math.ErrorFunction.normalDistributionImpl(-a);
}
real normalDistributionInv(real p)
{
return tango.math.ErrorFunction.normalDistributionInvImpl(p);
}
real normalDistributionComplInv(real p)
{
return -tango.math.ErrorFunction.normalDistributionInvImpl(-p);
}
debug (UnitTest)
{
    }
real studentsTDistribution(int nu, real t);
real studentsTDistributionInv(int nu, real p)
in
{
assert(nu > 0);
assert(p >= 0L && p <= 1L);
}
body
{
if (p == 0)
return -(real).infinity;
if (p == 1)
return (real).infinity;
real rk,z;
rk = nu;
if (p > 0.25L && p < 0.75L)
{
if (p == 0.5L)
return 0;
z = 1L - 2L * p;
z = betaIncompleteInv(0.5L,0.5L * rk,fabs(z));
real t = sqrt(rk * z / (1L - z));
if (p < 0.5L)
t = -t;
return t;
}
int rflg = -1;
if (p >= 0.5L)
{
p = 1L - p;
rflg = 1;
}
z = betaIncompleteInv(0.5L * rk,0.5L,2L * p);
if (z < 0)
return rflg * (real).infinity;
return rflg * sqrt(rk / z - rk);
}
debug (UnitTest)
{
    }
real fDistribution(int df1, int df2, real x)
in
{
assert(df1 >= 1 && df2 >= 1);
assert(x >= 0);
}
body
{
real a = cast(real)df1;
real b = cast(real)df2;
real w = a * x;
w = w / (b + w);
return betaIncomplete(0.5L * a,0.5L * b,w);
}
real fDistributionCompl(int df1, int df2, real x)
in
{
assert(df1 >= 1 && df2 >= 1);
assert(x >= 0);
}
body
{
real a = cast(real)df1;
real b = cast(real)df2;
real w = b / (b + a * x);
return betaIncomplete(0.5L * b,0.5L * a,w);
}
real fDistributionComplInv(int df1, int df2, real p)
in
{
assert(df1 >= 1 && df2 >= 1);
assert(p >= 0 && p <= 1);
}
body
{
real a = df1;
real b = df2;
real w = betaIncomplete(0.5L * b,0.5L * a,0.5L);
if (w > p || p < 0.001L)
{
w = betaIncompleteInv(0.5L * b,0.5L * a,p);
return (b - b * w) / (a * w);
}
else
{
w = betaIncompleteInv(0.5L * a,0.5L * b,1L - p);
return b * w / (a * (1L - w));
}
}
debug (UnitTest)
{
    }
real chiSqrDistribution(real v, real x)
in
{
assert(x >= 0);
assert(v >= 1);
}
body
{
return gammaIncomplete(0.5 * v,0.5 * x);
}
real chiSqrDistributionCompl(real v, real x)
in
{
assert(x >= 0);
assert(v >= 1);
}
body
{
return gammaIncompleteCompl(0.5L * v,0.5L * x);
}
real chiSqrDistributionComplInv(real v, real p)
in
{
assert(p >= 0 && p <= 1L);
assert(v >= 1L);
}
body
{
return 2 * gammaIncompleteComplInv(0.5 * v,p);
}
debug (UnitTest)
{
    }
real gammaDistribution(real a, real b, real x)
in
{
assert(x >= 0);
}
body
{
return gammaIncomplete(b,a * x);
}
real gammaDistributionCompl(real a, real b, real x)
in
{
assert(x >= 0);
}
body
{
return gammaIncompleteCompl(b,a * x);
}
debug (UnitTest)
{
    }
real betaDistribution(real a, real b, real x)
{
return betaIncomplete(a,b,x);
}
real betaDistributionCompl(real a, real b, real x)
{
return betaIncomplete(b,a,1 - x);
}
real betaDistributionInv(real a, real b, real y)
{
return betaIncompleteInv(a,b,y);
}
real betaDistributionComplInv(real a, real b, real y)
{
return 1 - betaIncompleteInv(b,a,y);
}
debug (UnitTest)
{
    }
real poissonDistribution(int k, real m)
in
{
assert(k >= 0);
assert(m > 0);
}
body
{
return gammaIncompleteCompl(k + 1,m);
}
real poissonDistributionCompl(int k, real m)
in
{
assert(k >= 0);
assert(m > 0);
}
body
{
return gammaIncomplete(k + 1,m);
}
real poissonDistributionInv(int k, real p)
in
{
assert(k >= 0);
assert(p >= 0 && p <= 1);
}
body
{
return gammaIncompleteComplInv(k + 1,p);
}
debug (UnitTest)
{
    }
real binomialDistribution(int k, int n, real p)
in
{
assert(p >= 0 && p <= 1);
assert(k >= 0 && k <= n);
}
body
{
real dk,dn,q;
if (k == n)
return 1L;
q = 1L - p;
dn = n - k;
if (k == 0)
{
return pow(q,dn);
}
else
{
return betaIncomplete(dn,k + 1,q);
}
}
debug (UnitTest)
{
    }
real binomialDistributionCompl(int k, int n, real p)
in
{
assert(p >= 0 && p <= 1);
assert(k >= 0 && k <= n);
}
body
{
if (k == n)
{
return 0;
}
real dn = n - k;
if (k == 0)
{
if (p < 0.01L)
return -expm1(dn * log1p(-p));
else
return 1L - pow(1L - p,dn);
}
else
{
return betaIncomplete(k + 1,dn,p);
}
}
debug (UnitTest)
{
    }
real binomialDistributionInv(int k, int n, real y)
in
{
assert(y >= 0 && y <= 1);
assert(k >= 0 && k <= n);
}
body
{
real dk,p;
real dn = n - k;
if (k == 0)
{
if (y > 0.8L)
p = -expm1(log1p(y - 1L) / dn);
else
p = 1L - pow(y,1L / dn);
}
else
{
dk = k + 1;
p = betaIncomplete(dn,dk,y);
if (p > 0.5)
p = betaIncompleteInv(dk,dn,1L - y);
else
p = 1 - betaIncompleteInv(dn,dk,y);
}
return p;
}
debug (UnitTest)
{
    }
real negativeBinomialDistribution(int k, int n, real p)
in
{
assert(p >= 0 && p <= 1);
assert(k >= 0);
}
body
{
if (k == 0)
return pow(p,n);
return betaIncomplete(n,k + 1,p);
}
real negativeBinomialDistributionInv(int k, int n, real p)
in
{
assert(p >= 0 && p <= 1);
assert(k >= 0);
}
body
{
return betaIncompleteInv(n,k + 1,p);
}
debug (UnitTest)
{
    }
