// D import file generated from 'tango/math/random/NormalSource.d'
module tango.math.random.NormalSource;
private import Integer = tango.text.convert.Integer;

import tango.math.Math;
import tango.math.ErrorFunction;
import tango.math.random.Ziggurat;
import tango.core.Traits;
template NormalSource(RandG,T)
{
class NormalSource
{
    static assert(isRealType!(T),T.stringof ~ " not acceptable, only floating point variables supported");
    static real probDensityF(real x)
{
return exp(-0.5L * x * x);
}

    static real invProbDensityF(real x)
{
return sqrt(-2L * log(x));
}

    static real cumProbDensityFCompl(real x)
{
return sqrt(0.5L * PI) * erfc(x / sqrt(2L));
}

    static T tailGenerator(RandG r, T dMin, int iNegative)
{
T x,y;
do
{
x = -log(r.uniform!(T)()) / dMin;
y = -log(r.uniform!(T)());
}
while (y + y < x * x);return iNegative ? -x - dMin : dMin + x;
}

    alias Ziggurat!(RandG,T,probDensityF,tailGenerator,true) SourceT;
    SourceT source;
    this(RandG r)
{
source = SourceT.create!(invProbDensityF,cumProbDensityFCompl)(r,0xe.9dda4104d699791p-2L);
}
    template opCall(U,S...)
{
NormalSource opCall(ref U a, S args)
{
randomize(a,args);
return this;
}
}
    final T getRandom()
{
return source.getRandom();
}

    final T getRandom(T sigma)
{
return sigma * source.getRandom();
}

    final T getRandom(T sigma, T mu)
{
return mu + sigma * source.getRandom();
}

    template randomize(U)
{
U randomize(ref U a)
{
return source.randomize(a);
}
}
    template randomize(U,V)
{
U randomize(ref U a, V sigma)
{
return source.randomizeOp(delegate (T x)
{
return x * cast(T)sigma;
}
,a);
}
}
    template randomize(U,V,S)
{
U randomize(ref U el, V sigma, S mu)
{
return source.randomizeOp(delegate (T x)
{
return x * cast(T)sigma + cast(T)mu;
}
,a);
}
}
    template randomizeOp(U,S)
{
U randomizeOp(scope S delegate(T) op, ref U a)
{
return source.randomizeOp(op,a);
}
}
    struct NormalDistribution
{
    T sigma;
    T mu;
    NormalSource source;
    static NormalDistribution create(NormalSource source, T sigma, T mu)
{
NormalDistribution res;
res.source = source;
res.sigma = sigma;
res.mu = mu;
return res;
}

    template opCall(U,S...)
{
NormalDistribution opCall(ref U a, S args)
{
randomize(a,args);
return this;
}
}
    T getRandom()
{
return mu + sigma * source.getRandom();
}
    template randomize(U)
{
U randomize(ref U a)
{
T op(T x)
{
return mu + sigma * x;
}
return source.randomizeOp(&op,a);
}
}
    template randomize(U,V)
{
U randomize(ref U a, V s)
{
T op(T x)
{
return mu + cast(T)s * x;
}
return source.randomizeOp(&op,a);
}
}
    template randomize(U,V,S)
{
U randomize(ref U a, V s, S m)
{
T op(T x)
{
return cast(T)m + cast(T)s * x;
}
return source.randomizeOp(&op,a);
}
}
}
    NormalDistribution normalD(T sigma = cast(T)1, T mu = cast(T)0)
{
return NormalDistribution.create(this,sigma,mu);
}
}
}
