// D import file generated from 'tango/math/random/Twister.d'
module tango.math.random.Twister;
version (Win32)
{
    private extern (Windows) int QueryPerformanceCounter(ulong*);


}
version (Posix)
{
    private import tango.stdc.posix.sys.time;

}
struct Twister
{
    public alias natural toInt;

    public alias fraction toReal;

    private enum : uint
{
N = 624,
M = 397,
MATRIX_A = -1727483681u,
UPPER_MASK = -2147483648u,
LOWER_MASK = 2147483647,
}

    private uint[N] mt;

    private uint mti = mt.length + 1;

    private uint vLastRand;

    public static __gshared Twister instance;


    shared static this();
    static Twister opCall()
{
Twister rand;
rand.seed();
return rand;
}

    uint natural(uint max)
{
return natural() % max;
}
    uint natural(uint min, uint max)
{
return natural() % (max - min) + min;
}
    uint natural(bool pAddEntropy = false)
{
uint y;
static uint[2] mag01 = [0,MATRIX_A];
if (mti >= mt.length)
{
int kk;
if (pAddEntropy || mti > mt.length)
{
seed(5489u,pAddEntropy);
}
{
for (kk = 0; kk < mt.length - M; kk++)
{
{
y = mt[kk] & UPPER_MASK | mt[kk + 1] & LOWER_MASK;
mt[kk] = mt[kk + M] ^ y >> 1 ^ mag01[y & 1u];
}
}
}
for (; kk < mt.length - 1; kk++)
{
{
y = mt[kk] & UPPER_MASK | mt[kk + 1] & LOWER_MASK;
mt[kk] = mt[kk + (M - mt.length)] ^ y >> 1 ^ mag01[y & 1u];
}
}
y = mt[mt.length - 1] & UPPER_MASK | mt[0] & LOWER_MASK;
mt[mt.length - 1] = mt[M - 1] ^ y >> 1 ^ mag01[y & 1u];
mti = 0;
}
y = mt[mti++];
y ^= y >> 11;
y ^= y << 7 & -1658038656u;
y ^= y << 15 & -272236544u;
y ^= y >> 18;
vLastRand = y;
return y;
}
    double inclusive()
{
return natural() * (1 / cast(double)(uint).max);
}
    double exclusive()
{
return (cast(double)natural() + 0.5) * (1 / (cast(double)(uint).max + 1));
}
    double fraction()
{
return natural() * (1 / (cast(double)(uint).max + 1));
}
    double fractionEx()
{
uint a = natural() >> 5,b = natural() >> 6;
return (a * 0x8p+23 + b) * (1 / 0x8p+50);
}
    void seed();
    void seed(uint s, bool pAddEntropy = false)
{
mt[0] = s + (pAddEntropy ? vLastRand + cast(uint)&this : 0) & -1u;
{
for (mti = 1; mti < mt.length; mti++)
{
{
mt[mti] = 1812433253u * (mt[mti - 1] ^ mt[mti - 1] >> 30) + mti;
mt[mti] &= -1u;
}
}
}
}
    void init(const(uint[]) init_key, bool pAddEntropy = false)
{
size_t k;
int i,j;
i = 1;
j = 0;
seed(19650218u,pAddEntropy);
{
for (k = mt.length > init_key.length ? mt.length : init_key.length; k; k--)
{
{
mt[i] = (mt[i] ^ (mt[i - 1] ^ mt[i - 1] >> 30) * 1664525u) + init_key[j] + j;
mt[i] &= -1u;
i++;
j++;
if (i >= mt.length)
{
mt[0] = mt[mt.length - 1];
i = 1;
}
if (j >= init_key.length)
{
j = 0;
}
}
}
}
{
for (k = mt.length - 1; k; k--)
{
{
mt[i] = (mt[i] ^ (mt[i - 1] ^ mt[i - 1] >> 30) * 1566083941u) - i;
mt[i] &= -1u;
i++;
if (i >= mt.length)
{
mt[0] = mt[mt.length - 1];
i = 1;
}
}
}
}
mt[0] |= -2147483648u;
mti = 0;
}
}
debug (Twister)
{
    import tango.io.Stdout;
    import tango.time.StopWatch;
    void main();
}
