// D import file generated from 'tango/net/Uri.d'
module tango.net.Uri;
public import tango.net.model.UriView;

private import tango.core.Exception;

private import Integer = tango.text.convert.Integer;

extern (C) void* memchr(in void* s, int c, size_t n);

class Uri : UriView
{
    private alias size_t delegate(const(void)[]) Consumer;

    public alias port getPort;

    public alias defaultPort getDefaultPort;

    public alias scheme getScheme;

    public alias host getHost;

    public alias validPort getValidPort;

    public alias userinfo getUserInfo;

    public alias path getPath;

    public alias query getQuery;

    public alias fragment getFragment;

    public alias port setPort;

    public alias scheme setScheme;

    public alias host setHost;

    public alias userinfo setUserInfo;

    public alias query setQuery;

    public alias path setPath;

    public alias fragment setFragment;

    public enum 
{
InvalidPort = -1,
}

    private int port_;

    private 
{
    const(char)[] host_;
    const(char)[] path_;
    const(char)[] query_;
    const(char)[] scheme_;
    const(char)[] userinfo_;
    const(char)[] fragment_;
}
    private HeapSlice decoded;

    private static __gshared ubyte[] map;


    private static __gshared short[immutable(char)[]] genericSchemes;


    private enum immutable(char)[] hexDigits = "0123456789abcdef";

    private enum SchemePort[] schemePorts = [{"coffee",80},{"file",InvalidPort},{"ftp",21},{"gopher",70},{"hnews",80},{"http",80},{"http-ng",80},{"https",443},{"imap",143},{"irc",194},{"ldap",389},{"news",119},{"nfs",2049},{"nntp",119},{"pop",110},{"rwhois",4321},{"shttp",80},{"smtp",25},{"snews",563},{"telnet",23},{"wais",210},{"whois",43},{"whois++",43}];

    public enum 
{
ExcScheme = 1,
ExcAuthority = 2,
ExcPath = 4,
IncUser = 128,
IncPath = 16,
IncQuery = 32,
IncQueryAll = 64,
IncScheme = 128,
IncGeneric = IncScheme | IncUser | IncPath | IncQuery | IncQueryAll,
}

    private struct SchemePort
{
    const(char)[] name;
    short port;
}

    shared static this();
    this()
{
port_ = InvalidPort;
decoded.expand(512);
}
    this(const(char)[] uri)
{
this();
parse(uri);
}
    this(const(char)[] scheme, const(char)[] host, const(char)[] path, const(char)[] query = null)
{
this();
this.scheme_ = scheme;
this.query_ = query;
this.host_ = host;
this.path_ = path;
}
    this(UriView other);
    const final override int defaultPort(const(char)[] scheme)
{
short* port = scheme in genericSchemes;
if (port is null)
return InvalidPort;
return *port;
}

    const final override @property const(char)[] scheme()
{
return scheme_;
}

    const final override @property const(char)[] host()
{
return host_;
}

    const final override @property int port()
{
return port_;
}

    const final override int validPort()
{
if (port_ is InvalidPort)
return defaultPort(scheme_);
return port_;
}

    const final override @property const(char)[] userinfo()
{
return userinfo_;
}

    const final override @property const(char)[] path()
{
return path_;
}

    const final override @property const(char)[] query()
{
return query_;
}

    const final override @property const(char)[] fragment()
{
return fragment_;
}

    const final override @property bool isGeneric()
{
return (scheme_ in genericSchemes) !is null;
}

    const final size_t produce(Consumer consume)
{
size_t ret;
if (scheme_.length)
ret += consume(scheme_) , ret += consume(":");
if (userinfo_.length || host_.length || port_ != InvalidPort)
{
ret += consume("//");
if (userinfo_.length)
ret += encode(consume,userinfo_,IncUser) , ret += consume("@");
if (host_.length)
ret += consume(host_);
if (port_ != InvalidPort && port_ != getDefaultPort(scheme_))
{
char[8] tmp;
ret += consume(":") , ret += consume(Integer.itoa(tmp,cast(uint)port_));
}
}
if (path_.length)
ret += encode(consume,path_,IncPath);
if (query_.length)
{
ret += consume("?");
ret += encode(consume,query_,IncQueryAll);
}
if (fragment_.length)
{
ret += consume("#");
ret += encode(consume,fragment_,IncQuery);
}
return ret;
}

    final override string toString();

    static size_t encode(Consumer consume, const(char)[] s, int flags);

    static char[] encode(const(char)[] text, int flags);

    private char[] decoder(const(char)[] s, char ignore = 0);

    final char[] decode(const(char)[] s)
{
return decoder(s).dup;
}

    final Uri parse(const(char)[] uri, bool relative = false)
{
char c;
size_t i,mark;
auto prefix = path_;
auto len = uri.length;
if (!relative)
reset();
{
for (i = 0; i < len && !(map[c = uri[i]] & ExcScheme); ++i)
{
{
}
}
}
if (c is ':')
{
auto lower_uri = uri[mark..i].dup;
toLower(lower_uri);
scheme_ = lower_uri;
mark = i + 1;
}
if (mark < len - 1 && uri[mark] is '/' && uri[mark + 1] is '/')
{
{
for (mark += 2 , i = mark; i < len && !(map[uri[i]] & ExcAuthority); ++i)
{
{
}
}
}
parseAuthority(uri[mark..i]);
mark = i;
}
else
if (relative)
{
auto head = uri[0] is '/' ? host_ : toLastSlash(prefix);
query_ = (fragment_ = null);
uri = head ~ uri;
len = uri.length;
mark = head.length;
}
{
for (i = mark; i < len && !(map[uri[i]] & ExcPath); ++i)
{
{
}
}
}
path_ = decoder(uri[mark..i]);
mark = i;
if (mark < len && uri[mark] is '?')
{
{
for (++mark , i = mark; i < len && uri[i] != '#'; ++i)
{
{
}
}
}
query_ = decoder(uri[mark..i],'&');
mark = i;
}
if (mark < len && uri[mark] is '#')
fragment_ = decoder(uri[mark + 1..len]);
return this;
}

    final void reset()
{
decoded.reset();
port_ = InvalidPort;
host_ = (path_ = (query_ = (scheme_ = (userinfo_ = (fragment_ = null)))));
}

    final Uri relParse(const(char)[] uri)
{
return parse(uri,true);
}

    final @property Uri scheme(const(char)[] scheme)
{
this.scheme_ = scheme;
return this;
}

    final @property Uri host(const(char)[] host)
{
this.host_ = host;
return this;
}

    final @property Uri port(int port)
{
this.port_ = port;
return this;
}

    final @property Uri userinfo(const(char)[] userinfo)
{
this.userinfo_ = userinfo;
return this;
}

    final @property Uri query(const(char)[] query)
{
this.query_ = query;
return this;
}

    final const(char)[] extendQuery(const(char)[] tail)
{
if (tail.length)
{
if (query_.length)
query_ = query_ ~ "&" ~ tail;
else
query_ = tail;
}
return query_;
}

    final @property Uri path(const(char)[] path)
{
this.path_ = path;
return this;
}

    final @property Uri fragment(const(char)[] fragment)
{
this.fragment_ = fragment;
return this;
}

    private void parseAuthority(const(char)[] auth);

    private final static template toLastSlash(T)
{
T[] toLastSlash(T[] path)
{
if (path.ptr)
{
for (auto p = path.ptr + path.length;
 --p >= path.ptr;)
{
if (*p is '/')
return path[0..p - path.ptr + 1];
}
}
return path;
}
}



    private final static char[] toLower(ref char[] src);



}
private struct HeapSlice
{
    private size_t used;

    private void[] buffer;

    final void reset()
{
used = 0;
}

    final void* expand(size_t size)
{
auto len = used + size;
if (len > buffer.length)
buffer.length = len + len / 2;
return &buffer[used];
}

    final void[] slice(int size)
{
size_t i = used;
used += size;
return buffer[i..used];
}

}

debug (UnitTest)
{
    import tango.util.log.Trace;
    }
