// D import file generated from 'tango/net/device/SSLSocket.d'
module tango.net.device.SSLSocket;
private import tango.net.util.PKI;

private import tango.net.device.Socket;

private import tango.net.util.c.OpenSSL;

class SSLSocket : Socket
{
    protected BIO* sslSocket = null;

    protected SSLCtx sslCtx = null;

    this(bool config = true)
{
super();
if (config)
setCtx(new SSLCtx,true);
}
    override void detach()
{
if (sslSocket)
{
BIO_reset(sslSocket);
BIO_free_all(sslSocket);
sslSocket = null;
}
super.detach();
}

    override size_t write(const(void)[] src)
{
if (src.length is 0)
return 0;
int bytes = BIO_write(sslSocket,cast(void*)src.ptr,cast(uint)src.length);
if (bytes <= 0)
return Eof;
return cast(size_t)bytes;
}

    override size_t read(void[] dst)
{
int bytes = BIO_read(sslSocket,dst.ptr,cast(uint)dst.length);
if (bytes <= 0)
return Eof;
return cast(size_t)bytes;
}

    override SSLSocket shutdown()
{
SSL* obj;
BIO_get_ssl(sslSocket,&obj);
if (obj)
{
if (!SSL_get_shutdown)
SSL_set_shutdown(obj,SSL_SENT_SHUTDOWN | SSL_RECEIVED_SHUTDOWN);
}
return this;
}

    void setCtx(SSLCtx ctx, bool clientMode = true)
{
sslCtx = ctx;
sslSocket = _convertToSSL(sslCtx,false,clientMode);
}
    private BIO* _convertToSSL(SSLCtx sslCtx, bool close, bool client)
{
BIO* rtn = null;
BIO* socketBio = BIO_new_socket(native.handle,close ? BIO_CLOSE : BIO_NOCLOSE);
if (socketBio)
{
rtn = BIO_new_ssl(sslCtx.native,client);
if (rtn)
rtn = BIO_push(rtn,socketBio);
if (!rtn)
BIO_free_all(socketBio);
}
if (rtn is null)
throwOpenSSLError();
return rtn;
}

}
class SSLServerSocket : ServerSocket
{
    private SSLCtx sslCtx;

    this(ushort port, SSLCtx ctx, int backlog = 32, bool reuse = false)
{
scope addr = new IPv4Address(port);
this(addr,ctx,backlog,reuse);
}
    this(Address addr, SSLCtx ctx, int backlog = 32, bool reuse = false)
{
super(addr,backlog,reuse);
sslCtx = ctx;
}
    alias ServerSocket.accept accept;
    SSLSocket accept(SSLSocket recipient = null)
{
if (recipient is null)
recipient = new SSLSocket(false);
super.accept(recipient);
recipient.setCtx(sslCtx,false);
return recipient;
}
}
version (Test)
{
    import tetra.util.Test;
    import tango.io.Stdout;
    import tango.io.device.File;
    import tango.io.FilePath;
    import tango.core.Thread;
    import tango.stdc.stringz;
    extern (C) int blah(int booger, void* x)
{
return 1;
}

    }
