// D import file generated from 'tango/net/ftp/FtpClient.d'
module tango.net.ftp.FtpClient;
private 
{
    import tango.net.ftp.Telnet;
    import tango.text.Util;
    import tango.time.Clock;
    import tango.text.Regex;
    import tango.time.chrono.Gregorian;
    import tango.core.Array;
    import tango.net.device.Socket;
    import tango.io.device.Conduit;
    import tango.io.device.Array;
    import tango.io.device.File;
    import Text = tango.text.Util;
    import Ascii = tango.text.Ascii;
    import Integer = tango.text.convert.Integer;
    import Timestamp = tango.text.convert.TimeStamp;
}
alias void delegate(in size_t pos) FtpProgress;
enum FtpFormat 
{
ascii,
image,
}
struct FtpAddress
{
    static FtpAddress* opCall(const(char)[] str);

    const(char)[] address;
    const(char)[] directory;
    const(char)[] user = "anonymous";
    const(char)[] pass = "anonymous@anonymous";
    uint port = 21;
}
struct FtpResponse
{
    char[3] code = "000";
    const(char)[] message = null;
}
enum FtpConnectionType 
{
active,
passive,
}
struct FtpConnectionDetail
{
    FtpConnectionType type = FtpConnectionType.passive;
    Address address = null;
    Address listen = null;
}
struct FtpFeature
{
    const(char)[] command = null;
    const(char)[] params = null;
}
enum FtpFileType 
{
unknown,
file,
cdir,
pdir,
dir,
other,
}
struct FtpFileInfo
{
    const(char)[] name = null;
    FtpFileType type = FtpFileType.unknown;
    ulong size = (ulong).max;
    Time modify = Time.max;
    Time create = Time.max;
    const(char)[] mime = null;
    const(char)[][const(char)[]] facts;
}
class FtpException : Exception
{
    char[3] responseCode_ = "000";
    this(string message, char[3] code = "420")
{
this.responseCode_[] = code;
super(message);
}
    this(FtpResponse r)
{
this.responseCode_[] = r.code;
super(r.message.idup);
}
    override string toString()
{
char[] buffer = new char[](this.msg.length + 4);
buffer[0..3] = this.responseCode_;
buffer[3] = ' ';
buffer[4..buffer.length] = this.msg;
return buffer.idup;
}

}
class FTPConnection : Telnet
{
    FtpFeature[] supportedFeatures_ = null;
    FtpConnectionDetail inf_;
    size_t restartPos_ = 0;
    const(char)[] currFile_ = "";
    Socket dataSocket_;
    TimeSpan timeout_ = TimeSpan.fromMillis(5000);
    @property public TimeSpan timeout()
{
return timeout_;
}


    @property public void timeout(TimeSpan t)
{
timeout_ = t;
}


    public TimeSpan shutdownTime()
{
return timeout_ + timeout_;
}

    public FtpFeature[] supportedFeatures()
{
if (supportedFeatures_ !is null)
{
return supportedFeatures_;
}
getFeatures();
return supportedFeatures_;
}

    override void exception(string message);

    void exception(FtpResponse fr)
{
exception(fr.message.idup);
}
    public this()
{
}

    public this(const(char)[] hostname, const(char)[] username = "anonymous", const(char)[] password = "anonymous@anonymous", uint port = 21)
{
this.connect(hostname,username,password,port);
}

    public this(FtpAddress fad)
{
connect(fad);
}

    public void connect(FtpAddress fad)
{
this.connect(fad.address,fad.user,fad.pass,fad.port);
}

    public void connect(const(char)[] hostname, const(char)[] username = "anonymous", const(char)[] password = "anonymous@anonymous", uint port = 21);

    public void close();

    public void setPassive()
{
inf_.type = FtpConnectionType.passive;
delete inf_.address;
delete inf_.listen;
}

    public void setActive(const(char)[] ip, ushort port, const(char)[] listen_ip = null, ushort listen_port = 0)
in
{
assert(ip.length > 0);
assert(port > 0);
}
body
{
inf_.type = FtpConnectionType.active;
inf_.address = new IPv4Address(ip,port);
if (listen_port == 0)
listen_port = port;
if (listen_ip == null)
inf_.listen = new IPv4Address(IPv4Address.ADDR_ANY,listen_port);
else
inf_.listen = new IPv4Address(listen_ip,listen_port);
}

    public void cd(const(char)[] dir)
in
{
assert(dir.length > 0);
}
body
{
sendCommand("CWD",dir);
readResponse("250");
}

    public void cdup()
{
sendCommand("CDUP");
FtpResponse fr = readResponse();
if (fr.code == "200" || fr.code == "250")
return ;
else
exception(fr);
}

    public const(char)[] cwd()
{
sendCommand("PWD");
auto response = readResponse("257");
return parse257(response);
}

    public void chmod(const(char)[] path, int mode)
in
{
assert(path.length > 0);
assert(mode >= 0 && mode >> 16 == 0);
}
body
{
char[3] tmp = "000";
Integer.format(tmp,cast(long)mode,"o");
sendCommand("SITE CHMOD",tmp,path);
readResponse("200");
}

    public void del(const(char)[] path)
in
{
assert(path.length > 0);
}
body
{
sendCommand("DELE",path);
auto response = readResponse("250");
if (response.code != "250")
rm(path);
}

    public void rm(const(char)[] path)
in
{
assert(path.length > 0);
}
body
{
sendCommand("RMD",path);
readResponse("250");
}

    public void rename(const(char)[] old_path, const(char)[] new_path)
in
{
assert(old_path.length > 0);
assert(new_path.length > 0);
}
body
{
sendCommand("RNFR",old_path);
readResponse("350");
sendCommand("RNTO",new_path);
readResponse("250");
}

    int exist(const(char)[] file);
    public size_t size(const(char)[] path, FtpFormat format = FtpFormat.image);

    public void type(FtpFormat format)
{
if (format == FtpFormat.ascii)
sendCommand("TYPE","A");
else
sendCommand("TYPE","I");
readResponse("200");
}

    Time modified(const(char)[] file)
in
{
assert(file.length > 0);
}
body
{
this.sendCommand("MDTM",file);
auto response = this.readResponse("213");
return this.parseTimeval(response.message);
}
    protected Time parseTimeval(const(char)[] timeval);

    public void noop()
{
this.sendCommand("NOOP");
this.readResponse("200");
}

    public const(char)[] mkdir(const(char)[] path)
in
{
assert(path.length > 0);
}
body
{
this.sendCommand("MKD",path);
auto response = this.readResponse("257");
return this.parse257(response);
}

    public void getFeatures()
{
this.sendCommand("FEAT");
auto response = this.readResponse();
if (response.code != "211")
delete supportedFeatures_;
else
{
const(char)[][] lines = Text.splitLines(response.message);
supportedFeatures_ = new FtpFeature[](lines.length - 1);
supportedFeatures_[0].command = "FEAT";
{
for (size_t i = 1;
 i < lines.length - 1; i++)
{
{
size_t pos = Text.locate(lines[i],' ');
supportedFeatures_[i].command = lines[i][0..pos];
if (pos < lines[i].length - 1)
supportedFeatures_[i].params = lines[i][pos + 1..lines[i].length];
}
}
}
delete lines;
}
}

    public void sendCommand(const(char)[] command, const(char)[][] parameters...);

    public FtpResponse readResponse(const(char)[] expected_code);

    public FtpResponse readResponse();

    protected const(char)[] parse257(FtpResponse response);

    protected Socket getDataSocket();

    version (TangoDoc)
{
    public Socket connectPassive();

}
else
{
    public Socket connectPassive();

}
    public bool isSupported(const(char)[] command);

    public bool is_supported(const(char)[] command)
{
if (this.supportedFeatures_.length == 0)
return false;
return this.isSupported(command);
}

    protected void prepareDataSocket(ref Socket data);

    public void finishDataCommand(Socket data)
{
data.close();
data.detach();
FtpResponse r = readResponse();
if (!(r.code == "226" || r.code == "420"))
exception("Bad finish");
}

    public Socket processDataCommand(const(char)[] command, const(char)[][] parameters...);

    public FtpFileInfo[] ls(const(char)[] path = "");

    protected void readStream(Socket data, OutputStream stream, FtpProgress progress = null);

    protected void sendStream(Socket data, InputStream stream, FtpProgress progress = null);

    protected FtpFileInfo[] sendListCommand(const(char)[] path);

    protected FtpFileInfo parseListLine(const(char)[] line);

    protected FtpFileInfo parseMlstLine(const(char)[] line);

    public FtpFileInfo getFileInfo(const(char)[] path);

    public void put(const(char)[] path, const(char)[] local_file, FtpProgress progress = null, FtpFormat format = FtpFormat.image);

    public void put(const(char)[] path, InputStream stream = null, FtpProgress progress = null, FtpFormat format = FtpFormat.image)
in
{
assert(path.length > 0);
}
body
{
this.type(format);
Socket data = this.processDataCommand("STOR",path);
if (stream !is null)
this.sendStream(data,stream,progress);
this.finishDataCommand(data);
}

    public void append(const(char)[] path, InputStream stream, FtpProgress progress = null, FtpFormat format = FtpFormat.image)
in
{
assert(path.length > 0);
assert(stream !is null);
}
body
{
this.type(format);
Socket data = this.processDataCommand("APPE",path);
this.sendStream(data,stream,progress);
this.finishDataCommand(data);
}

    public void restartSeek(size_t offset)
{
char[16] tmp;
this.sendCommand("REST",Integer.format(tmp,cast(long)offset));
this.readResponse("350");
this.restartPos_ = offset;
}

    public void allocate(long bytes)
in
{
assert(bytes > 0);
}
body
{
char[16] tmp;
this.sendCommand("ALLO",Integer.format(tmp,bytes));
auto response = this.readResponse();
if (response.code != "200" && response.code != "202")
exception(response);
}

    public void get(const(char)[] path, const(char)[] local_file, FtpProgress progress = null, FtpFormat format = FtpFormat.image);

    public void enableUTF8()
{
sendCommand("OPTS UTF8 ON");
readResponse("200");
}

    public void get(const(char)[] path, OutputStream stream, FtpProgress progress = null, FtpFormat format = FtpFormat.image)
in
{
assert(path.length > 0);
assert(stream !is null);
}
body
{
this.type(format);
Socket data = this.processDataCommand("RETR",path);
this.readStream(data,stream,progress);
this.finishDataCommand(data);
}

    public InputStream input(const(char)[] path)
{
type(FtpFormat.image);
dataSocket_ = this.processDataCommand("RETR",path);
return dataSocket_;
}

    public OutputStream output(const(char)[] path)
{
type(FtpFormat.image);
dataSocket_ = this.processDataCommand("STOR",path);
return dataSocket_;
}

}
