// D import file generated from 'tango/net/http/HttpClient.d'
module tango.net.http.HttpClient;
private import tango.time.Time;

private 
{
    import tango.net.Uri;
    import tango.net.device.Socket;
    import tango.net.InternetAddress;
}
private import tango.io.device.Array;

private import tango.io.stream.Lines;

private import tango.io.stream.Buffered;

private 
{
    import tango.net.http.HttpConst;
    import tango.net.http.HttpParams;
    import tango.net.http.HttpHeaders;
    import tango.net.http.HttpTriplet;
    import tango.net.http.HttpCookies;
}
private import tango.core.Exception;

private import Integer = tango.text.convert.Integer;

class HttpClient
{
    alias void delegate(OutputBuffer) Pump;
    private struct RequestMethod
{
    const(char)[] name;
}

    private Uri uri;

    private BufferedInput input;

    private BufferedOutput output;

    private Array tokens;

    private Lines!(char) line;

    private Socket socket;

    private RequestMethod method;

    private InternetAddress address;

    private HttpParams paramsOut;

    private HttpHeadersView headersIn;

    private HttpHeaders headersOut;

    private HttpCookies cookiesOut;

    private ResponseLine responseLine;

    private float timeout = 3;

    private bool encode = true;

    private bool doRedirect = true;

    private bool keepalive = false;

    private 
{
    uint redirections;
    uint redirectionLimit = 5;
}
    private const(char)[] httpVersion;

    public enum Version 
{
OnePointZero,
OnePointOne,
}

    enum RequestMethod Get = {"GET"};
    enum RequestMethod Put = {"PUT"};
    enum RequestMethod Head = {"HEAD"};
    enum RequestMethod Post = {"POST"};
    enum RequestMethod Trace = {"TRACE"};
    enum RequestMethod Delete = {"DELETE"};
    enum RequestMethod Options = {"OPTIONS"};
    enum RequestMethod Connect = {"CONNECT"};
    this(RequestMethod method, const(char)[] url)
{
this(method,new Uri(url));
}
    this(RequestMethod method, Uri uri)
{
this.uri = uri;
this.method = method;
responseLine = new ResponseLine;
headersIn = new HttpHeadersView;
tokens = new Array(1024 * 4);
input = new BufferedInput(null,1024 * 16);
output = new BufferedOutput(null,1024 * 16);
paramsOut = new HttpParams;
headersOut = new HttpHeaders;
cookiesOut = new HttpCookies(headersOut,HttpHeader.Cookie);
auto host = uri.getHost;
if (host)
address = new InternetAddress(host,uri.getValidPort());
else
error("invalid url provided to HttpClient ctor");
paramsOut.parse(new Array(cast(void[])uri.query));
setVersion(Version.OnePointZero);
}
    HttpHeadersView getResponseHeaders()
{
return headersIn;
}
    HttpHeaders getRequestHeaders()
{
return headersOut;
}
    HttpParams getRequestParams()
{
return paramsOut;
}
    UriView getUri()
{
return uri;
}
    ResponseLine getResponse()
{
return responseLine;
}
    int getStatus()
{
return responseLine.getStatus();
}
    bool isResponseOK()
{
return getStatus() is HttpResponseCode.OK;
}
    HttpClient addCookie(Cookie cookie)
{
cookiesOut.add(cookie);
return this;
}
    void close()
{
if (socket)
{
socket.shutdown();
socket.detach();
socket = null;
}
}
    HttpClient reset()
{
headersIn.reset();
headersOut.reset();
paramsOut.reset();
redirections = 0;
return this;
}
    HttpClient setRequest(RequestMethod method)
{
this.method = method;
return this;
}
    HttpClient setVersion(Version v)
{
enum versions = ["HTTP/1.0","HTTP/1.1"];
httpVersion = versions[v];
return this;
}
    HttpClient enableRedirect(bool yes = true)
{
doRedirect = yes;
return this;
}
    HttpClient setTimeout(float interval)
{
timeout = interval;
return this;
}
    HttpClient keepAlive(bool yes = true)
{
keepalive = yes;
return this;
}
    HttpClient encodeUri(bool yes = true)
{
encode = yes;
return this;
}
    InputBuffer open()
{
return open(method,null);
}
    InputBuffer open(Pump pump)
{
return open(method,pump);
}
    InputBuffer open(RequestMethod method, Pump pump);
    void read(scope void delegate(const(void)[]) sink, size_t len = size_t.max);
    InputBuffer redirectPost(Pump pump, int status);
    bool canRepost(uint status)
{
return false;
}
    protected Socket createSocket()
{
return new Socket;
}

    private void error(const(char)[] msg);

}
private class ResponseLine : HttpTriplet
{
    private 
{
    const(char)[] vers;
    const(char)[] reason;
}
    private int status;

    override bool test()
{
vers = tokens[0];
reason = tokens[2];
status = cast(int)Integer.convert(tokens[1]);
if (status is 0)
{
status = cast(int)Integer.convert(tokens[2]);
if (status is 0)
{
failed = "Invalid HTTP response: '" ~ tokens[0] ~ "' '" ~ tokens[1] ~ "' '" ~ tokens[2] ~ "'";
return false;
}
}
return true;
}

    const(char)[] getVersion()
{
return vers;
}
    const(char)[] getReason()
{
return reason;
}
    int getStatus()
{
return status;
}
}

debug (HttpClient)
{
    import tango.io.Stdout;
    void main();
}
