// D import file generated from 'tango/net/http/HttpCookies.d'
module tango.net.http.HttpCookies;
private import tango.stdc.ctype;

private import tango.io.device.Array;

private import tango.io.model.IConduit;

private import tango.io.stream.Iterator;

private import tango.net.http.HttpHeaders;

private import Integer = tango.text.convert.Integer;

class Cookie
{
    const(char)[] name;
    const(char)[] path;
    const(char)[] value;
    const(char)[] domain;
    const(char)[] comment;
    uint vrsn = 1;
    bool secure = false;
    long maxAge = (long).min;
    this()
{
}
    this(const(char)[] name, const(char)[] value)
{
setName(name);
setValue(value);
}
    Cookie setName(const(char)[] name)
{
this.name = name;
return this;
}
    Cookie setValue(const(char)[] value)
{
this.value = value;
return this;
}
    Cookie setVersion(uint vrsn)
{
this.vrsn = vrsn;
return this;
}
    Cookie setPath(const(char)[] path)
{
this.path = path;
return this;
}
    Cookie setDomain(const(char)[] domain)
{
this.domain = domain;
return this;
}
    Cookie setComment(const(char)[] comment)
{
this.comment = comment;
return this;
}
    Cookie setMaxAge(long maxAge)
{
this.maxAge = maxAge;
return this;
}
    Cookie setSecure(bool secure)
{
this.secure = secure;
return this;
}
    void produce(scope size_t delegate(const(void)[]) consume)
{
consume(name);
if (value.length)
consume("=") , consume(value);
if (path.length)
consume(";Path=") , consume(path);
if (domain.length)
consume(";Domain=") , consume(domain);
if (vrsn)
{
char[16] tmp = void;
consume(";Version=");
consume(Integer.format(tmp,vrsn));
if (comment.length)
consume(";Comment=\"") , consume(comment) , consume("\"");
if (secure)
consume(";Secure");
if (maxAge != maxAge.min)
consume(";Max-Age="c) , consume(Integer.format(tmp,maxAge));
}
}
    Cookie clear()
{
vrsn = 1;
secure = false;
maxAge = maxAge.min;
name = (path = (domain = (comment = null)));
return this;
}
}
class CookieStack
{
    private int depth;

    private Cookie[] cookies;

    this(int size)
{
cookies = new Cookie[](0);
resize(cookies,size);
}
    final void reset()
{
depth = 0;
}

    final Cookie push()
{
if (depth == cookies.length)
resize(cookies,depth * 2);
return cookies[depth++];
}

    private final static void resize(ref Cookie[] cookies, size_t size)
{
size_t i = cookies.length;
{
for (cookies.length = size; i < cookies.length; ++i)
{
cookies[i] = new Cookie;
}
}
}



    int opApply(scope int delegate(ref Cookie) dg);
}
class HttpCookiesView
{
    private bool parsed;

    private CookieStack stack;

    private CookieParser parser;

    private HttpHeadersView headers;

    this(HttpHeadersView headers)
{
this.headers = headers;
stack = new CookieStack(10);
parser = new CookieParser(stack);
}
    void produce(scope size_t delegate(const(void)[]) consume, const(char)[] eol = HttpConst.Eol);
    void reset()
{
stack.reset();
parsed = false;
}
    CookieStack parse();
}
class HttpCookies
{
    private HttpHeaderName name;

    private HttpHeaders headers;

    this(HttpHeaders headers, const(HttpHeaderName) name = HttpHeader.SetCookie)
{
this.headers = headers;
this.name = name;
}
    void add(Cookie cookie);
}
class CookieParser : Iterator!(char)
{
    private enum State 
{
Begin,
LValue,
Equals,
RValue,
Token,
SQuote,
DQuote,
}

    private CookieStack stack;

    private Array array;

    private static __gshared bool[128] charMap;


    shared static this();
    this(CookieStack stack)
{
super();
this.stack = stack;
array = new Array(0);
}
    protected override size_t scan(const(void)[] data);


    bool parse(const(char)[] header)
{
super.set(array.assign(cast(void[])header));
return next.ptr > null;
}
    final static char[] toLower(ref char[] src);


    private static bool isToken(char c)
{
return c > 32 && c < 127 && !charMap[c];
}


}
