// D import file generated from 'tango/net/util/MemCache.d'
module tango.net.util.MemCache;
private import tango.io.Console;

private 
{
    import tango.core.Thread;
    import tango.core.Exception;
}
private 
{
    import tango.io.stream.Lines;
    import tango.io.stream.Buffered;
}
private 
{
    import tango.net.device.Socket;
    import tango.net.InternetAddress;
}
private import Integer = tango.text.convert.Integer;

class MemCache : Thread
{
    private shared(Connection)[] hosts;

    private bool active;

    private uint watchdog;

    this(const(char)[][] hosts, uint watchdog = 3)
{
super(&run);
setHosts(hosts);
this.watchdog = watchdog;
active = true;
super.start();
}
    final void close();

    final bool set(const(void)[][] key, const(void)[][] value, int flags = 0, int timeout = 0)
{
return select(key).put("set",key,value,flags,timeout);
}

    final bool add(const(void)[][] key, const(void)[][] value, int flags = 0, int timeout = 0)
{
return select(key).put("add",key,value,flags,timeout);
}

    final bool replace(const(void)[][] key, const(void)[][] value, int flags = 0, int timeout = 0)
{
return select(key).put("replace",key,value,flags,timeout);
}

    final bool remove(const(void)[][] key, int timeout = 0)
{
return select(key).remove(key,timeout);
}

    final bool get(const(void)[][] key, Buffer buffer)
{
return select(key).get(key,buffer);
}

    final bool incr(const(void)[][] key, uint value)
{
uint result;
return incr(key,value,result);
}

    final bool decr(const(void)[][] key, uint value)
{
uint result;
return decr(key,value,result);
}

    final bool incr(const(void)[][] key, uint value, ref uint result)
{
return select(key).bump("incr",key,value,result);
}

    final bool decr(const(void)[][] key, uint value, ref uint result)
{
return select(key).bump("decr",key,value,result);
}

    final void status(void delegate(const(char)[], const(char)[][] list) dg);

    final Buffer buffer(uint size)
{
return new Buffer(size);
}

    final void setHosts(const(char)[][] hosts);

    private void run();

    private shared(Connection) select(const(void)[][] key)
{
return hosts[jhash(key) % hosts.length];
}

    private void connect(shared(Connection)[] hosts);

    static class Buffer
{
    private size_t extent;

    private void[] content;

    private this(size_t size)
{
this.content = new byte[](size);
}

    bool expand(size_t size)
{
if (size > content.length)
content.length = size;
return true;
}
    void[] set(size_t size)
{
extent = size;
return get();
}
    void[] get()
{
return content[0..extent];
}
}

    static final uint jhash(const(void)[][] x, uint c = 0);

}
private class Connection
{
    private alias Lines!(char) Line;

    private const(char)[] host;

    private __gshared Line line;


    private __gshared Bin input;


    private __gshared Bout output;


    private __gshared Socket conduit;


    private InternetAddress address;

    private bool connected;

    this(const(char)[] host)
{
this.host = host;
conduit = new Socket;
output = new Bout(conduit);
input = new Bin(conduit);
line = new Line(input);
address = new InternetAddress(host);
}
    private void connect();

    private synchronized void close()
{
bool alive = connected;
connected = false;
if (alive)
conduit.close();
}


    private synchronized void error()
{
close();
conduit.socket.reopen();
}


    private synchronized bool put(const(char)[] cmd, const(void)[][] key, const(void)[][] value, int flags, int timeout);


    private synchronized bool get(const(void)[][] key, MemCache.Buffer buffer);


    private synchronized bool remove(const(void)[][] key, int timeout = 0);


    private synchronized bool bump(const(char)[] cmd, const(void)[][] key, uint value, ref uint result);


    private synchronized void status(scope void delegate(const(char)[], const(char)[][] list) dg);


}

debug (TangoMemCache)
{
    void main();
}
