// D import file generated from 'tango/net/util/PKI.d'
module tango.net.util.PKI;
private import tango.time.Time;

private import tango.stdc.stringz;

private import tango.net.util.c.OpenSSL;

const int SSL_VERIFY_NONE = 0;

const int SSL_VERIFY_PEER = 1;

const int SSL_VERIFY_FAIL_IF_NO_PEER_CERT = 2;

const int SSL_VERIFY_CLIENT_ONCE = 4;

const int SSL_SESS_CACHE_SERVER = 2;

extern (C) alias int function(int, X509_STORE_CTX* ctx) SSLVerifyCallback;

class SSLCtx
{
    private SSL_CTX* _ctx = null;

    private Certificate _cert = null;

    private PrivateKey _key = null;

    private CertificateStore _store = null;

    this()
{
if ((_ctx = SSL_CTX_new(SSLv23_method())) is null)
throwOpenSSLError();
}
    ~this()
{
if (_ctx)
{
SSL_CTX_free(_ctx);
_ctx = null;
}
_cert = null;
_key = null;
_store = null;
}
    @property SSL_CTX* native()
{
return _ctx;
}

    SSLCtx certificate(Certificate cert)
{
if (SSL_CTX_use_certificate(_ctx,cert._cert))
_cert = cert;
else
throwOpenSSLError();
return this;
}
    SSLCtx privateKey(PrivateKey key)
{
if (SSL_CTX_use_PrivateKey(_ctx,key._evpKey))
_key = key;
else
throwOpenSSLError();
return this;
}
    @property SSLCtx checkKey()
{
if (!SSL_CTX_check_private_key(_ctx))
throwOpenSSLError();
return this;
}

    SSLCtx setVerification(int flags, SSLVerifyCallback cb)
{
SSL_CTX_set_verify(_ctx,flags,cb);
return this;
}
    SSLCtx store(CertificateStore store)
{
SSL_CTX_set_cert_store(_ctx,store._store);
_store = store;
return this;
}
    SSLCtx caCertsPath(const(char)[] path)
{
if (!SSL_CTX_load_verify_locations(_ctx,null,toStringz(path)))
throwOpenSSLError();
return this;
}
}
class CertificateStoreCtx
{
    private X509_STORE_CTX* _ctx = null;

    this(X509_STORE_CTX* ctx)
{
_ctx = ctx;
}
    Certificate cert()
{
X509* cert = X509_STORE_CTX_get_current_cert(_ctx);
if (cert is null)
throwOpenSSLError();
return new Certificate(cert);
}
    int error()
{
return X509_STORE_CTX_get_error(_ctx);
}
    int errorDepth()
{
return X509_STORE_CTX_get_error_depth(_ctx);
}
}
class CertificateStore
{
    package X509_STORE* _store = null;

    Certificate[] _certs;
    this()
{
if ((_store = X509_STORE_new()) is null)
throwOpenSSLError();
}
    ~this()
{
if (_store)
{
X509_STORE_free(_store);
_store = null;
}
}
    CertificateStore add(Certificate cert)
{
if (X509_STORE_add_cert(_store,cert._cert))
_certs ~= cert;
else
throwOpenSSLError();
return this;
}
}
class PublicKey
{
    package RSA* _evpKey = null;

    private PrivateKey _existingKey = null;

    this(char[] publicPemData)
{
BIO* bp = BIO_new_mem_buf(publicPemData.ptr,cast(int)publicPemData.length);
if (bp)
{
_evpKey = PEM_read_bio_RSAPublicKey(bp,null,null,null);
BIO_free_all(bp);
}
if (_evpKey is null)
throwOpenSSLError();
}
    package this(PrivateKey key)
{
this._evpKey = cast(RSA*)key._evpKey.pkey;
this._existingKey = key;
}

    ~this()
{
if (_existingKey !is null)
{
_existingKey = null;
_evpKey = null;
}
else
if (_evpKey)
{
RSA_free(_evpKey);
_evpKey = null;
}
}
    char[] pemFormat()
{
char[] rtn = null;
BIO* bp = BIO_new(BIO_s_mem());
if (bp)
{
if (PEM_write_bio_RSAPublicKey(bp,_evpKey))
{
char* pemData = null;
int pemSize = BIO_get_mem_data(bp,&pemData);
rtn = pemData[0..pemSize].dup;
}
BIO_free_all(bp);
}
if (rtn is null)
throwOpenSSLError();
return rtn;
}
    bool verify(ubyte[] data, ubyte[] signature)
{
ubyte[MD5_DIGEST_LENGTH] digest;
MD5_CTX c;
MD5_Init(&c);
MD5_Update(&c,data.ptr,data.length);
MD5_Final(digest.ptr,&c);
if (RSA_verify(NID_md5,digest.ptr,MD5_DIGEST_LENGTH,signature.ptr,cast(uint)signature.length,_evpKey))
return true;
return false;
}
    ubyte[] encrypt(ubyte[] data);
    ubyte[] decrypt(ubyte[] data)
{
ubyte[] rtn;
uint maxSize = RSA_size(_evpKey);
ubyte[] tmpRtn = new ubyte[](maxSize);
int numBytes = RSA_public_decrypt(cast(int)data.length,data.ptr,tmpRtn.ptr,_evpKey,RSA_PKCS1_PADDING);
if (numBytes >= 0)
rtn = tmpRtn[0..numBytes];
if (rtn is null)
throwOpenSSLError();
return rtn;
}
}
class PrivateKey
{
    package EVP_PKEY* _evpKey = null;

    this(char[] privatePemData, char[] certPass = null)
{
BIO* bp = BIO_new_mem_buf(privatePemData.ptr,cast(int)privatePemData.length);
if (bp)
{
_evpKey = PEM_read_bio_PrivateKey(bp,null,null,certPass ? toStringz(certPass) : null);
BIO_free_all(bp);
}
if (_evpKey is null)
throwOpenSSLError();
}
    this(int bits)
{
RSA* rsa = RSA_generate_key(bits,RSA_F4,null,null);
if (rsa)
{
if ((_evpKey = EVP_PKEY_new()) !is null)
EVP_PKEY_assign_RSA(_evpKey,rsa);
if (_evpKey is null)
RSA_free(rsa);
}
if (_evpKey is null)
throwOpenSSLError();
}
    ~this()
{
if (_evpKey)
{
EVP_PKEY_free(_evpKey);
_evpKey = null;
}
}
    override bool opEquals(Object obj)
{
auto pk = cast(PrivateKey)obj;
if (pk !is null)
return cast(bool)EVP_PKEY_cmp_parameters(pk._evpKey,this._evpKey);
return false;
}

    char[] pemFormat(char[] pass = null)
{
char[] rtn = null;
BIO* bp = BIO_new(BIO_s_mem());
if (bp)
{
if (PEM_write_bio_PKCS8PrivateKey(bp,_evpKey,pass ? EVP_aes_256_cbc() : null,null,0,null,pass ? toStringz(pass) : null))
{
char* pemData = null;
int pemSize = BIO_get_mem_data(bp,&pemData);
rtn = pemData[0..pemSize].dup;
}
BIO_free_all(bp);
}
if (rtn is null)
throwOpenSSLError();
return rtn;
}
    PublicKey publicKey()
{
auto rtn = new PublicKey(this);
return rtn;
}
    ubyte[] sign(const(ubyte)[] data, ubyte[] sigbuf);
    ubyte[] encrypt(ubyte[] data);
    ubyte[] decrypt(ubyte[] data)
{
ubyte[] rtn;
uint maxSize = RSA_size(cast(RSA*)_evpKey.pkey);
ubyte[] tmpRtn = new ubyte[](maxSize);
int numBytes = RSA_private_decrypt(cast(int)data.length,data.ptr,tmpRtn.ptr,cast(RSA*)_evpKey.pkey,RSA_PKCS1_OAEP_PADDING);
if (numBytes >= 0)
rtn = tmpRtn[0..numBytes];
if (rtn is null)
throwOpenSSLError();
return rtn;
}
}
class Certificate
{
    package X509* _cert = null;

    private bool readOnly = true;

    private bool freeIt = true;

    package this(X509* cert)
{
_cert = cert;
freeIt = false;
}

    this(char[] publicPemData)
{
BIO* data = BIO_new_mem_buf(publicPemData.ptr,cast(int)publicPemData.length);
if (data)
{
_cert = PEM_read_bio_X509(data,null,null,null);
BIO_free_all(data);
}
if (_cert is null)
throwOpenSSLError();
}
    this()
{
if ((_cert = X509_new()) !is null)
{
if (!X509_set_version(_cert,2))
{
X509_free(_cert);
_cert = null;
}
else
readOnly = false;
}
if (_cert is null)
throwOpenSSLError();
}
    ~this()
{
if (_cert && freeIt)
{
X509_free(_cert);
_cert = null;
}
}
    Certificate serialNumber(uint serial)
{
checkFlag();
if (!ASN1_INTEGER_set(X509_get_serialNumber(_cert),serial))
throwOpenSSLError();
return this;
}
    uint serialNumber()
{
if (!X509_get_serialNumber(_cert))
throwOpenSSLError();
return ASN1_INTEGER_get(X509_get_serialNumber(_cert));
}
    Certificate dateBeforeOffset(TimeSpan t)
{
checkFlag();
if (!X509_gmtime_adj(X509_get_notBefore(_cert),cast(int)t.seconds))
throwOpenSSLError();
return this;
}
    Certificate dateAfterOffset(TimeSpan t)
{
checkFlag();
if (!X509_gmtime_adj(X509_get_notAfter(_cert),cast(int)t.seconds))
throwOpenSSLError();
return this;
}
    char[] dateAfter()
{
char[] rtn;
ASN1_GENERALIZEDTIME* genTime = ASN1_TIME_to_generalizedtime(X509_get_notAfter(_cert),null);
if (genTime)
{
rtn = genTime.data[0..genTime.length].dup;
ASN1_STRING_free(cast(ASN1_STRING*)genTime);
}
return rtn;
}
    char[] dateBefore()
{
char[] rtn;
ASN1_GENERALIZEDTIME* genTime = ASN1_TIME_to_generalizedtime(X509_get_notBefore(_cert),null);
if (genTime)
{
rtn = genTime.data[0..genTime.length].dup;
ASN1_STRING_free(cast(ASN1_STRING*)genTime);
}
return rtn;
}
    Certificate privateKey(PrivateKey key)
{
checkFlag();
if (key)
{
if (!X509_set_pubkey(_cert,key._evpKey))
throwOpenSSLError();
}
return this;
}
    Certificate setSubject(const(char)[] country, const(char)[] stateProvince, const(char)[] city, const(char)[] organization, const(char)[] cn, const(char)[] organizationalUnit = null, const(char)[] email = null)
in
{
assert(country);
assert(stateProvince);
assert(organization);
assert(cn);
}
body
{
checkFlag();
X509_NAME* name = X509_get_subject_name(_cert);
if (name)
{
addNameEntry(name,"C",country);
addNameEntry(name,"ST",stateProvince);
addNameEntry(name,"L",city);
addNameEntry(name,"O",organization);
if (organizationalUnit !is null)
addNameEntry(name,"OU",organizationalUnit);
if (email)
addNameEntry(name,"emailAddress",email);
addNameEntry(name,"CN",cn);
}
else
throwOpenSSLError();
return this;
}
    char[] subject()
{
char[] rtn = null;
X509_NAME* subjectName = X509_get_subject_name(_cert);
if (subjectName)
{
BIO* subjectBIO = BIO_new(BIO_s_mem());
if (subjectBIO)
{
if (X509_NAME_print_ex(subjectBIO,subjectName,0,XN_FLAG_MULTILINE))
{
char* subjectPtr = null;
int length = BIO_get_mem_data(subjectBIO,&subjectPtr);
rtn = subjectPtr ? subjectPtr[0..length].dup : null;
}
BIO_free_all(subjectBIO);
}
}
if (rtn is null)
throwOpenSSLError();
return rtn;
}
    Certificate sign(Certificate caCert, PrivateKey caKey)
in
{
assert(caCert);
assert(caKey);
}
body
{
checkFlag();
X509_NAME* issuer = X509_get_subject_name(caCert._cert);
if (issuer)
{
if (X509_set_issuer_name(_cert,issuer))
{
if (X509_sign(_cert,caKey._evpKey,EVP_sha1()))
readOnly = true;
}
}
if (!readOnly)
throwOpenSSLError();
return this;
}
    override bool opEquals(Object obj)
{
auto c = cast(Certificate)obj;
if (c !is null)
return !X509_cmp(c._cert,this._cert);
return false;
}

    bool verify(CertificateStore store)
{
bool rtn = false;
X509_STORE_CTX* verifyCtx = X509_STORE_CTX_new();
if (verifyCtx)
{
if (X509_STORE_CTX_init(verifyCtx,store._store,_cert,null))
{
if (X509_verify_cert(verifyCtx))
rtn = true;
}
X509_STORE_CTX_free(verifyCtx);
}
return rtn;
}
    char[] pemFormat()
{
char[] rtn = null;
BIO* bp = BIO_new(BIO_s_mem());
if (bp)
{
if (PEM_write_bio_X509(bp,_cert))
{
char* pemData = null;
int pemSize = BIO_get_mem_data(bp,&pemData);
rtn = pemData[0..pemSize].dup;
}
BIO_free_all(bp);
}
if (rtn is null)
throwOpenSSLError();
return rtn;
}
    private void addNameEntry(X509_NAME* name, const(char)* type, const(char)[] value)
{
if (!X509_NAME_add_entry_by_txt(name,type,MBSTRING_ASC,toStringz(value),cast(int)value.length,-1,0))
throwOpenSSLError();
}

    private void checkFlag();

}
version (Test)
{
    import util.Test;
    import tango.io.Stdout;
    auto t1 = TimeSpan.zero;
    auto t2 = TimeSpan.fromDays(365);
    }
