// D import file generated from 'tango/sys/Environment.d'
module tango.sys.Environment;
private import tango.sys.Common;

private 
{
    import tango.io.Path;
    import tango.io.FilePath;
}
private import tango.core.Exception;

private import tango.io.model.IFile;

private import Text = tango.text.Util;

private import tango.core.Octal;

version (Windows)
{
    private import tango.text.convert.Utf;

    pragma (lib, "kernel32.lib");
    extern (Windows) 
{
    private void* GetEnvironmentStringsW();

    private bool FreeEnvironmentStringsW(wchar**);

}
    extern (Windows) 
{
    private int SetEnvironmentVariableW(const(wchar)*, const(wchar)*);

    private uint GetEnvironmentVariableW(const(wchar)*, wchar*, uint);

    private const int ERROR_ENVVAR_NOT_FOUND = 203;


}
}
else
{
    version (darwin)
{
    extern (C) char*** _NSGetEnviron();

    private __gshared char** environ;


    shared static this();
}
else
{
    private extern (C) extern __gshared char** environ;



}
    import tango.stdc.posix.stdlib;
    import tango.stdc.string;
}
struct Environment
{
    public alias cwd directory;

    private static void exception(immutable(char)[] msg);


    static char[] toAbsolute(char[] path)
{
scope fp = new FilePath(path);
if (fp.isAbsolute)
return path;
fp.absolute(cwd());
return fp.cString()[0..$ - 1];
}

    static FilePath exePath(char[] file);

    version (Windows)
{
    static char[] get(const(char)[] variable, char[] def = null)
{
const(wchar)[] var = toString16(variable) ~ "\x00";
uint size = GetEnvironmentVariableW(var.ptr,cast(wchar*)null,0);
if (size is 0)
{
if (SysError.lastCode is ERROR_ENVVAR_NOT_FOUND)
return def;
else
exception(SysError.lastMsg.idup);
}
auto buffer = new wchar[](size);
size = GetEnvironmentVariableW(var.ptr,buffer.ptr,size);
if (size is 0)
exception(SysError.lastMsg.idup);
return toString(buffer[0..size]);
}

    static void set(const(char)[] variable, const(char)[] value = null)
{
const(wchar)* var,val;
var = (toString16(variable) ~ "\x00").ptr;
if (value.length > 0)
val = (toString16(value) ~ "\x00").ptr;
if (!SetEnvironmentVariableW(var,val))
exception(SysError.lastMsg.idup);
}

    static char[][char[]] get();

    static void cwd(const(char)[] path);

    static char[] cwd();

}
    version (Posix)
{
    static char[] get(const(char)[] variable, char[] def = null)
{
char* ptr = getenv((variable ~ '\x00').ptr);
if (ptr is null)
return def;
return ptr[0..strlen(ptr)].dup;
}

    static void set(const(char)[] variable, const(char)[] value = null)
{
int result;
if (value.length is 0)
unsetenv((variable ~ '\x00').ptr);
else
result = setenv((variable ~ '\x00').ptr,(value ~ '\x00').ptr,1);
if (result != 0)
exception(SysError.lastMsg.idup);
}

    static char[][char[]] get();

    static void cwd(const(char)[] path)
{
char[512] tmp = void;
tmp[path.length] = 0;
tmp[0..path.length] = path;
if (tango.stdc.posix.unistd.chdir(tmp.ptr))
exception("Failed to set current directory");
}

    static char[] cwd()
{
char[512] tmp = void;
char* s = tango.stdc.posix.unistd.getcwd(tmp.ptr,tmp.length);
if (s is null)
exception("Failed to get current directory");
auto path = s[0..strlen(s) + 1].dup;
if (path[$ - 2] is '/')
path.length = path.length - 1;
else
path[$ - 1] = '/';
return path;
}

}
}
debug (Environment)
{
    import tango.io.Console;
    void main(const(char)[][] args);
}
