// D import file generated from 'tango/sys/Process.d'
module tango.sys.Process;
private import tango.io.model.IFile;

private import tango.io.Console;

private import tango.sys.Common;

private import tango.sys.Pipe;

private import tango.core.Exception;

private import tango.text.Util;

private import Integer = tango.text.convert.Integer;

private import tango.stdc.stdlib;

private import tango.stdc.string;

private import tango.stdc.stringz;

version (Posix)
{
    private import tango.stdc.errno;

    private import tango.stdc.posix.fcntl;

    private import tango.stdc.posix.unistd;

    private import tango.stdc.posix.sys.wait;

    version (darwin)
{
    extern (C) char*** _NSGetEnviron();

    private __gshared char** environ;


    shared static this();
}
else
{
    private extern (C) extern __gshared char** environ;



}
}
version (Windows)
{
    version (Win32SansUnicode)
{
}
else
{
    private import tango.text.convert.Utf;

}
}
debug (Process)
{
    private import tango.io.Stdout;

}
enum Redirect 
{
None = 0,
Output = 1,
Error = 2,
Input = 4,
All = Output | Error | Input,
ErrorToOutput = 16,
OutputToError = 32,
}
class Process
{
    public struct Result
{
    public enum 
{
Exit,
Signal,
Stop,
Continue,
Error,
}

    public int reason;

    public int status;

    public const(char)[] toString();

}

    enum uint DefaultStdinBufferSize = 512;
    enum uint DefaultStdoutBufferSize = 8192;
    enum uint DefaultStderrBufferSize = 512;
    enum Redirect DefaultRedirectFlags = Redirect.All;
    private const(char)[][] _args;

    private const(char)[][char[]] _env;

    private const(char)[] _workDir;

    private PipeConduit _stdin;

    private PipeConduit _stdout;

    private PipeConduit _stderr;

    private bool _running = false;

    private bool _copyEnv = false;

    private Redirect _redirect = DefaultRedirectFlags;

    version (Windows)
{
    private PROCESS_INFORMATION* _info = null;

    private bool _gui = false;

}
else
{
    private pid_t _pid = cast(pid_t)-1;

}
    public this(const(char)[][] args...)
{
if (args.length == 1)
_args = splitArgs(args[0]);
else
_args = args;
}

    public this(bool copyEnv, const(char)[][] args...)
{
_copyEnv = copyEnv;
this(args);
}

    public this(const(char)[] command, const(char)[][char[]] env)
in
{
assert(command.length > 0);
}
body
{
_args = splitArgs(command);
_env = env;
}

    public this(const(char)[][] args, const(char)[][char[]] env)
in
{
assert(args.length > 0);
assert(args[0].length > 0);
}
body
{
_args = args;
_env = env;
}

    public const bool isRunning()
{
return _running;
}


    @property public const int pid();



    @property public const const(char)[] programName()
{
return _args !is null ? _args[0] : null;
}



    @property public const(char)[] programName(const(char)[] name)
{
if (_args.length == 0)
{
_args.length = 1;
}
return _args[0] = name;
}


    public Process setProgramName(const(char)[] name)
{
programName = name;
return this;
}

    public const const(char[])[] args()
{
return _args;
}


    public const(char)[][] args(const(char)[] progname, const(char)[][] args...)
{
return _args = progname ~ args;
}

    public Process setArgs(const(char)[] progname, const(char)[][] args...)
{
this.args(progname,args);
return this;
}

    public const bool copyEnv()
{
return _copyEnv;
}


    public bool copyEnv(bool b)
{
return _copyEnv = b;
}

    public Process setCopyEnv(bool b)
{
_copyEnv = b;
return this;
}

    @property public const const(char[])[const(char)[]] env()
{
return _env;
}



    @property public const(char)[][char[]] env(const(char)[][char[]] env)
{
_copyEnv = false;
return _env = env;
}


    public Process setEnv(const(char)[][char[]] env)
{
_copyEnv = false;
_env = env;
return this;
}

    public override string toString()
{
immutable(char)[] command;
{
for (size_t i = 0;
 i < _args.length; ++i)
{
{
if (i > 0)
{
command ~= ' ';
}
if (contains(_args[i],' ') || _args[i].length == 0)
{
command ~= '"';
command ~= _args[i].substitute("\\","\\\\").substitute("\"","\\\"");
command ~= '"';
}
else
{
command ~= _args[i].substitute("\\","\\\\").substitute("\"","\\\"");
}
}
}
}
return command;
}


    @property public const const(char)[] workDir()
{
return _workDir;
}



    @property public const(char)[] workDir(const(char)[] dir)
{
return _workDir = dir;
}


    public Process setWorkDir(const(char)[] dir)
{
_workDir = dir;
return this;
}

    @property public const Redirect redirect()
{
return _redirect;
}



    @property public Redirect redirect(Redirect flags)
{
return _redirect = flags;
}


    public Process setRedirect(Redirect flags)
{
_redirect = flags;
return this;
}

    @property public const bool gui();



    @property public bool gui(bool value);


    public Process setGui(bool value);

    @property public PipeConduit stdin()
{
return _stdin;
}


    @property public PipeConduit stdout()
{
return _stdout;
}


    @property public PipeConduit stderr()
{
return _stderr;
}


    deprecated public void execute(const(char)[] arg1, const(char)[][] args...)
in
{
assert(!_running);
}
body
{
this._args = arg1 ~ args;
execute();
}


    deprecated public void execute(const(char)[] command, const(char)[][const(char)[]] env)
in
{
assert(!_running);
assert(command.length > 0);
}
body
{
_args = splitArgs(command);
_copyEnv = false;
_env = env;
execute();
}


    deprecated public void execute(const(char)[][] args, const(char)[][char[]] env)
in
{
assert(!_running);
assert(args.length > 0);
}
body
{
_args = args;
_env = env;
_copyEnv = false;
execute();
}


    public Process execute();

    public Result wait();

    public void kill();

    protected static const(char)[][] splitArgs(ref const(char)[] command, const(char)[] delims = " \x09\x0d\x0a");


    protected void cleanPipes()
{
delete _stdin;
delete _stdout;
delete _stderr;
}

    public void close()
{
this.cleanPipes();
}

    version (Windows)
{
    protected static char[] toNullEndedBuffer(const(char)[][char[]] src);


}
else
{
    version (Posix)
{
    protected static const(char)*[] toNullEndedArray(const(char)[][] src);


    protected static const(char)*[] toNullEndedArray(const(char)[][char[]] src);


    protected static int execvpe(const(char)[] filename, const(char)*[] argv, const(char)*[] envp);


}
}
}
class ProcessCreateException : ProcessException
{
    public this(const(char)[] command, const(char)[] file, uint line)
{
this(command,SysError.lastMsg,file,line);
}

    public this(const(char)[] command, const(char)[] message, const(char)[] file, uint line)
{
super("Could not create process for " ~ command.idup ~ " : " ~ message.idup);
}

}
class ProcessForkException : ProcessException
{
    public this(int pid, const(char)[] file, uint line)
{
super(format("Could not fork process ",pid).idup ~ " : " ~ SysError.lastMsg.idup);
}

}
class ProcessKillException : ProcessException
{
    public this(int pid, const(char)[] file, uint line)
{
super(format("Could not kill process ",pid).idup ~ " : " ~ SysError.lastMsg.idup);
}

}
class ProcessWaitException : ProcessException
{
    public this(int pid, const(char)[] file, uint line)
{
super(format("Could not wait on process ",pid).idup ~ " : " ~ SysError.lastMsg.idup);
}

}
private char[] format(const(char)[] msg, int value)
{
char[10] tmp;
return msg ~ Integer.format(tmp,value);
}

extern (C) uint sleep(uint s);

debug (UnitTest)
{
    import tango.io.Stdout;
    import tango.stdc.stdio;
    }
