// D import file generated from 'tango/text/Search.d'
module tango.text.Search;
private import Util = tango.text.Util;

template find(T)
{
FindFruct!(T) find(const(T)[] what)
{
return FindFruct!(T)(what);
}
}
template search(T)
{
SearchFruct!(T) search(const(T)[] what)
{
return SearchFruct!(T)(what);
}
}
private template FindFruct(T)
{
struct FindFruct
{
    private const(T)[] what;

    size_t forward(const(T)[] content, size_t ofs = 0)
{
return Util.index(content,what,ofs);
}
    size_t reverse(const(T)[] content, size_t ofs = size_t.max)
{
return Util.rindex(content,what,ofs);
}
    @property const(T)[] match()
{
return what;
}

    @property void match(const(T)[] what)
{
this.what = what;
}

    bool within(const(T)[] content)
{
return forward(content) != content.length;
}
    size_t count(const(T)[] content)
{
size_t mark,count;
while ((mark = Util.index(content,what,mark)) != content.length)
++count , ++mark;
return count;
}
    T[] replace(const(T)[] content, T chr)
{
return replace(content,(&chr)[0..1]);
}
    T[] replace(const(T)[] content, const(T)[] sub = null)
{
T[] output;
foreach (s; tokens(content,sub))
{
output ~= s;
}
return output;
}
    Util.PatternFruct!(T) tokens(const(T)[] content, const(T)[] sub = null)
{
return Util.patterns(content,what,sub);
}
    Indices indices(const(T)[] content)
{
return Indices(what,content);
}
    private struct Indices
{
    const(T)[] what;
    const(T)[] content;
    int opApply(scope int delegate(ref size_t index) dg)
{
int ret;
size_t mark;
while ((mark = Util.index(content,what,mark)) != content.length)
if ((ret = dg(mark)) is 0)
++mark;
else
break;
return ret;
}
}

}
}

private template SearchFruct(T)
{
struct SearchFruct
{
    private const(T)[] what;

    private bool fore;

    private int[256] offsets = void;

    static SearchFruct opCall(const(T)[] what)
{
SearchFruct find = void;
find.match = what;
return find;
}

    @property const(T)[] match()
{
return what;
}

    @property void match(const(T)[] what)
{
offsets[] = cast(int)(what.length + 1);
this.fore = true;
this.what = what;
reset();
}

    size_t forward(const(T)[] content, size_t ofs = 0)
{
if (!fore)
flip();
if (ofs > content.length)
ofs = content.length;
return find(cast(char*)what.ptr,what.length * T.sizeof,cast(char*)content.ptr,content.length * T.sizeof,ofs * T.sizeof) / T.sizeof;
}
    size_t reverse(const(T)[] content, size_t ofs = size_t.max)
{
if (fore)
flip();
if (ofs > content.length)
ofs = content.length;
return rfind(cast(char*)what.ptr,what.length * T.sizeof,cast(char*)content.ptr,content.length * T.sizeof,ofs * T.sizeof) / T.sizeof;
}
    bool within(const(T)[] content)
{
return forward(content) != content.length;
}
    size_t count(const(T)[] content)
{
size_t mark,count;
while ((mark = forward(content,mark)) != content.length)
++count , ++mark;
return count;
}
    T[] replace(const(T)[] content, T chr)
{
return replace(content,(&chr)[0..1]);
}
    T[] replace(const(T)[] content, const(T)[] sub = null)
{
T[] output;
foreach (s; tokens(content,sub))
{
output ~= s;
}
return output;
}
    Substitute tokens(const(T)[] content, const(T)[] sub = null)
{
return Substitute(sub,what,content,&forward);
}
    Indices indices(const(T)[] content)
{
return Indices(content,&forward);
}
    private size_t find(char* what, size_t wlen, char* content, size_t len, size_t ofs)
{
auto s = content;
content += ofs;
auto e = s + len - wlen;
while (content <= e)
if (*what is *content && matches(what,content,wlen))
return content - s;
else
content += offsets[content[wlen]];
return len;
}

    private size_t rfind(char* what, size_t wlen, char* content, size_t len, size_t ofs)
{
auto s = content;
auto e = s + ofs - wlen;
while (e >= content)
if (*what is *e && matches(what,e,wlen))
return e - s;
else
e -= offsets[*(e - 1)];
return len;
}

    private static bool matches(char* a, char* b, size_t length)
{
while (length > size_t.sizeof)
if (*cast(size_t*)a is *cast(size_t*)b)
a += size_t.sizeof , b += size_t.sizeof , length -= size_t.sizeof;
else
return false;
while (length--)
if (*a++ != *b++)
return false;
return true;
}


    private void reset()
{
auto what = cast(char[])this.what;
if (fore)
{
for (int i = 0;
 i < cast(int)what.length; ++i)
{
offsets[what[i]] = cast(int)what.length - i;
}
}
else
{
for (int i = cast(int)what.length;
 i--;)
{
offsets[what[i]] = cast(int)(i + 1);
}
}
}

    private void flip()
{
fore ^= true;
reset();
}

    private struct Indices
{
    const(T)[] content;
    size_t delegate(const(T)[], size_t) call;
    int opApply(scope int delegate(ref size_t index) dg)
{
int ret;
size_t mark;
while ((mark = call(content,mark)) != content.length)
if ((ret = dg(mark)) is 0)
++mark;
else
break;
return ret;
}
}

    private struct Substitute
{
    private 
{
    const(T)[] sub;
    const(T)[] what;
    const(T)[] content;
}
    size_t delegate(const(T)[], size_t) call;
    int opApply(scope int delegate(ref const(T)[] token) dg)
{
int ret;
size_t pos,mark;
const(T)[] token;
while ((pos = call(content,mark)) < content.length)
{
token = content[mark..pos];
if ((ret = dg(token)) != 0)
return ret;
if (sub.ptr && (ret = dg(sub)) != 0)
return ret;
mark = pos + what.length;
}
token = content[mark..$];
if (mark <= content.length)
ret = dg(token);
return ret;
}
}

}
}

debug (Search)
{
    import tango.io.Stdout;
    import tango.time.StopWatch;
    auto x = import("Search.d");
    void main();
}
