// D import file generated from 'tango/text/convert/DateTime.d'
module tango.text.convert.DateTime;
private import tango.core.Exception;

private import tango.time.WallClock;

private 
{
    import tango.time.chrono.Calendar;
    import tango.time.chrono.Gregorian;
}
private import Utf = tango.text.convert.Utf;

private import Integer = tango.text.convert.Integer;

version (WithExtensions)
{
    private import tango.text.convert.Extensions;

}
version (Windows)
{
    private import tango.sys.win32.UserGdi;

}
else
{
    private import tango.stdc.stringz;

    private import tango.stdc.posix.langinfo;

}
public __gshared DateTimeLocale DateTimeDefault;


shared static this();
struct DateTimeLocale
{
    enum immutable(char)[] rfc1123Pattern = "ddd, dd MMM yyyy HH':'mm':'ss 'GMT'";
    enum immutable(char)[] sortableDateTimePattern = "yyyy'-'MM'-'dd'T'HH':'mm':'ss";
    enum immutable(char)[] universalSortableDateTimePattern = "yyyy'-'MM'-'dd' 'HH':'mm':'ss'Z'";
    Calendar assignedCalendar;
    const(char)[] shortDatePattern;
    const(char)[] shortTimePattern;
    const(char)[] longDatePattern;
    const(char)[] longTimePattern;
    const(char)[] fullDateTimePattern;
    const(char)[] generalShortTimePattern;
    const(char)[] generalLongTimePattern;
    const(char)[] monthDayPattern;
    const(char)[] yearMonthPattern;
    const(char)[] amDesignator;
    const(char)[] pmDesignator;
    const(char)[] timeSeparator;
    const(char)[] dateSeparator;
    const(char)[][] dayNames;
    const(char)[][] monthNames;
    const(char)[][] abbreviatedDayNames;
    const(char)[][] abbreviatedMonthNames;
    char[] format(char[] output, Time dateTime, const(char)[] layout)
{
if (layout.length is 0)
layout = "G";
if (layout.length is 1)
layout = expandKnownFormat(layout);
auto res = Result(output);
return formatCustom(res,dateTime,layout);
}
    template formatWide(T)
{
T[] formatWide(T[] output, Time dateTime, const(T)[] fmt)
{
static if(is(T == char))
{
return format(output,dateTime,fmt);
}
else
{
char[128] tmp0 = void;
char[128] tmp1 = void;
return Utf.fromString8(format(tmp0,dateTime,Utf.toString(fmt,tmp1)),output);
}

}
}
    @property static DateTimeLocale* generic()
{
return &EngUS;
}


    @property Calendar calendar()
{
if (assignedCalendar is null)
assignedCalendar = Gregorian.generic;
return assignedCalendar;
}

    const(char)[] abbreviatedDayName(Calendar.DayOfWeek dayOfWeek)
{
return abbreviatedDayNames[cast(int)dayOfWeek];
}
    const(char)[] dayName(Calendar.DayOfWeek dayOfWeek)
{
return dayNames[cast(int)dayOfWeek];
}
    const(char)[] abbreviatedMonthName(int month)
{
assert(month > 0 && month < 13);
return abbreviatedMonthNames[month - 1];
}
    const(char)[] monthName(int month)
{
assert(month > 0 && month < 13);
return monthNames[month - 1];
}
    version (Windows)
{
    static DateTimeLocale create();

}
else
{
    static DateTimeLocale create();

    private static char[] convert(const(char)[] fmt);


}
    private const(char)[] expandKnownFormat(const(char)[] format);

    private char[] formatCustom(ref Result result, Time dateTime, const(char)[] format);

    private const(char)[] formatMonth(int month, int rpt)
{
if (rpt is 3)
return abbreviatedMonthName(month);
return monthName(month);
}

    private const(char)[] formatDayOfWeek(Calendar.DayOfWeek dayOfWeek, int rpt)
{
if (rpt is 3)
return abbreviatedDayName(dayOfWeek);
return dayName(dayOfWeek);
}

    private template bridge(T)
{
T[] bridge(T[] result, void* arg, const(T)[] format)
{
return formatWide(result,*cast(Time*)arg,format);
}
}

    private static int parseRepeat(const(char)[] format, int pos, char c);


    private static char[] formatInt(char[] tmp, int v, int minimum);


    private static int parseQuote(ref Result result, const(char)[] format, int pos);


}
private DateTimeLocale EngUS = {shortDatePattern:"M/d/yyyy",shortTimePattern:"h:mm",longDatePattern:"dddd, MMMM d, yyyy",longTimePattern:"h:mm:ss tt",fullDateTimePattern:"dddd, MMMM d, yyyy h:mm:ss tt",generalShortTimePattern:"M/d/yyyy h:mm",generalLongTimePattern:"M/d/yyyy h:mm:ss tt",monthDayPattern:"MMMM d",yearMonthPattern:"MMMM, yyyy",amDesignator:"AM",pmDesignator:"PM",timeSeparator:":",dateSeparator:"/",dayNames:["Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"],monthNames:["January","February","March","April","May","June","July","August","September","OctoberNovember","December"],abbreviatedDayNames:["Sun","Mon","Tue","Wed","Thu","Fri","Sat"],abbreviatedMonthNames:["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","OctNov","Dec"]};

private struct Result
{
    private size_t index;

    private char[] target_;

    private static Result opCall(char[] target)
{
Result result;
result.target_ = target;
return result;
}


    private void opCatAssign(const(char)[] rhs)
{
auto end = index + rhs.length;
assert(end < target_.length);
target_[index..end] = rhs;
index = end;
}

    private void opCatAssign(char rhs)
{
assert(index < target_.length);
target_[index++] = rhs;
}

    @property private char[] get()
{
return target_[0..index];
}


}

debug (DateTime)
{
    import tango.io.Stdout;
    void main()
{
char[100] tmp;
auto time = WallClock.now;
auto locale = DateTimeLocale.create;
Stdout.formatln("d: {}",locale.format(tmp,time,"d"));
Stdout.formatln("D: {}",locale.format(tmp,time,"D"));
Stdout.formatln("f: {}",locale.format(tmp,time,"f"));
Stdout.formatln("F: {}",locale.format(tmp,time,"F"));
Stdout.formatln("g: {}",locale.format(tmp,time,"g"));
Stdout.formatln("G: {}",locale.format(tmp,time,"G"));
Stdout.formatln("r: {}",locale.format(tmp,time,"r"));
Stdout.formatln("s: {}",locale.format(tmp,time,"s"));
Stdout.formatln("t: {}",locale.format(tmp,time,"t"));
Stdout.formatln("T: {}",locale.format(tmp,time,"T"));
Stdout.formatln("y: {}",locale.format(tmp,time,"y"));
Stdout.formatln("u: {}",locale.format(tmp,time,"u"));
Stdout.formatln("@: {}",locale.format(tmp,time,"@"));
Stdout.formatln("{}",locale.generic.format(tmp,time,"ddd, dd MMM yyyy HH':'mm':'ss zzzz"));
}
}
