// D import file generated from 'tango/text/convert/Float.d'
module tango.text.convert.Float;
private import tango.core.Exception;

version = float_internal;
private alias real NumType;

private extern (C) 
{
    double log10(double x);
    double ceil(double num);
    double modf(double num, double* i);
    double pow(double base, double exp);
    real log10l(real x);
    real ceill(real num);
    real modfl(real num, real* i);
    real powl(real base, real exp);
    int printf(char*,...);
    version (float_lib)
{
    version (Windows)
{
    alias ecvt econvert;
    alias ecvt fconvert;
    char* ecvt(double d, int digits, int* decpt, int* sign);
    char* fcvt(double d, int digits, int* decpt, int* sign);
}
else
{
    alias ecvtl econvert;
    alias ecvtl fconvert;
    char* ecvtl(real d, int digits, int* decpt, int* sign);
    char* fcvtl(real d, int digits, int* decpt, int* sign);
}
}
}

private enum 
{
Pad = 0,
Dec = 2,
Exp = 10,
}

template toFloat(T)
{
NumType toFloat(const(T[]) src)
{
size_t len;
auto x = parse(src,&len);
if (len < src.length || len == 0)
throw new IllegalArgumentException("Float.toFloat :: invalid number");
return x;
}
}
char[] toString(NumType d, uint decimals = Dec, int e = Exp)
{
char[64] tmp = void;
return format(tmp,d,decimals,e).dup;
}
wchar[] toString16(NumType d, uint decimals = Dec, int e = Exp)
{
wchar[64] tmp = void;
return format(tmp,d,decimals,e).dup;
}
dchar[] toString32(NumType d, uint decimals = Dec, int e = Exp)
{
dchar[64] tmp = void;
return format(tmp,d,decimals,e).dup;
}
template truncate(T)
{
T[] truncate(T[] s)
{
auto tmp = s;
int i = tmp.length;
foreach (int idx, T c; tmp)
{
if (c is '.')
while (--i >= idx)
if (tmp[i] != '0')
{
if (tmp[i] is '.')
--i;
s = tmp[0..i + 1];
while (--i >= idx)
if (tmp[i] is 'e')
return tmp;
break;
}
}
return s;
}
}
private bool negative(NumType x);

template format(T,D = NumType,U = int)
{
T[] format(T[] dst, D x, U decimals = Dec, U e = Exp, bool pad = Pad)
{
return format!(T)(dst,x,decimals,e,pad);
}
}
template format(T)
{
T[] format(T[] dst, NumType x, int decimals = Dec, int e = Exp, bool pad = Pad)
{
char* end,str;
int exp,sign,mode = 5;
char[32] buf = void;
exp = x is 0 ? 1 : cast(int)log10l(x < 0 ? -x : x);
if (exp <= -e || exp >= e)
mode = 2 , ++decimals;
version (float_internal)
{
str = convertl(buf.ptr,x,decimals,&exp,&sign,mode is 5);
}

version (float_dtoa)
{
str = dtoa(x,mode,decimals,&exp,&sign,&end);
}

version (float_lib)
{
if (mode is 5)
str = fconvert(x,decimals,&exp,&sign);
else
str = econvert(x,decimals,&exp,&sign);
}

auto p = dst.ptr;
if (sign)
*p++ = '-';
if (exp is 9999)
while (*str)
*p++ = *str++;
else
{
if (mode is 2)
{
--exp;
*p++ = *str++;
if (*str || pad)
{
auto d = p;
*p++ = '.';
while (*str)
*p++ = *str++;
if (pad)
while (p - d < decimals)
*p++ = '0';
}
*p++ = 'e';
if (exp < 0)
*p++ = '-' , exp = -exp;
else
*p++ = '+';
if (exp >= 1000)
{
*p++ = cast(T)(exp / 1000 + '0');
exp %= 1000;
}
if (exp >= 100)
{
*p++ = cast(char)(exp / 100 + '0');
exp %= 100;
}
*p++ = cast(char)(exp / 10 + '0');
*p++ = cast(char)(exp % 10 + '0');
}
else
{
if (exp <= 0)
*p++ = '0';
else
for (; exp > 0; --exp)
{
*p++ = *str ? *str++ : '0';
}
if (*str || pad)
{
*p++ = '.';
auto d = p;
for (; exp < 0; ++exp)
{
*p++ = '0';
}
while (*str)
*p++ = *str++;
if (pad)
while (p - d < decimals)
*p++ = '0';
}
}
}
*p = 0;
return dst[0..p - dst.ptr];
}
}
version (float_internal)
{
    private char* convertl(char* buf, real value, int ndigit, int* decpt, int* sign, int fflag);

}
version (float_dtoa)
{
    private extern (C) 
{
    double strtod(const(char*) s00, const(char*)* se);
    char* dtoa(double d, int mode, int ndigits, int* decpt, int* sign, char** rve);
}

    NumType parse(const(char[]) src, size_t* ate = null)
{
const(char)* end;
auto value = strtod(src.ptr,&end);
assert(end <= src.ptr + src.length);
if (ate)
*ate = end - src.ptr;
return value;
}
    NumType parse(const(wchar[]) src, size_t* ate = null);
    NumType parse(const(dchar[]) src, size_t* ate = null);
}
else
{
    private import Integer = tango.text.convert.Integer;

    template parse(T)
{
NumType parse(const(T[]) src, size_t* ate = null)
{
T c;
const(T)* p;
int exp;
bool sign;
uint radix;
NumType value = 0;
static bool match(const(T)* aa, const(T[]) bb)
{
foreach (b; bb)
{
auto a = cast(T)*aa++;
if (a >= 'A' && a <= 'Z')
a += 'a' - 'A';
if (a != b)
return false;
}
return true;
}
p = src.ptr + Integer.trim(src,sign,radix);
if (src.length is 0 || p > &src[$ - 1])
return NumType.nan;
c = *p;
if (radix != 10)
{
long v = Integer.parse(src,radix,ate);
return cast(NumType)v;
}
auto begin = p;
auto end = src.ptr + src.length;
while (c >= '0' && c <= '9' && p < end)
{
value = value * 10 + (c - '0');
c = *++p;
}
if (c is '.' && p < end)
c = *++p;
while (c >= '0' && c <= '9' && p < end)
{
value = value * 10 + (c - '0');
c = *++p;
--exp;
}
if (p > begin)
{
if ((c is 'e' || c is 'E') && p < end)
{
size_t eaten;
exp += Integer.parse(src[++p - src.ptr..$],0,&eaten);
p += eaten;
}
if (exp < 0)
value /= pow10(-exp);
else
value *= pow10(exp);
}
else
if (end - p >= 3)
switch (*p)
{
case 'I':
{
}
case 'i':
{
if (match(p + 1,"nf"))
{
value = value.infinity;
p += 3;
if (end - p >= 5 && match(p,"inity"))
p += 5;
}
break;
}
case 'N':
{
}
case 'n':
{
if (match(p + 1,"an"))
{
value = value.nan;
p += 3;
}
break;
}
default:
{
break;
}
}
if (ate)
*ate = p - src.ptr;
if (sign)
value = -value;
return value;
}
}
    private NumType pow10(uint exp);

}
version (float_old)
{
    template format(T,D = double,U = uint)
{
T[] format(T[] dst, D x, U decimals = Dec, int e = Exp, bool pad = Pad)
{
return format!(T)(dst,x,decimals,e,pad);
}
}
    template format(T)
{
T[] format(T[] dst, NumType x, uint decimals = Dec, int e = Exp, bool pad = Pad)
{
enum immutable(T)[] inf = "-inf";
enum immutable(T)[] nan = "-nan";
static int toDigit(ref NumType v, ref int count)
{
int digit;
if (--count <= 0)
digit = 0;
else
{
digit = cast(int)v;
v = (v - digit) * 10;
}
return digit + '0';
}
bool sign = negative(x);
if (sign)
x = -x;
if (x !<>= x)
return sign ? nan : nan[1..$];
if (x is x.infinity)
return sign ? inf : inf[1..$];
int exp = 0;
int abs = 0;
if (x > 0)
{
exp = cast(int)log10l(x);
auto d = decimals;
if (exp < 0)
d -= exp;
x += 0.5 / pow10(d);
abs = (exp = cast(int)log10l(x));
if (exp > 0)
x /= pow10(exp);
else
abs = -exp;
if (abs >= e)
e = 0;
}
T* p = dst.ptr;
int count = NumType.dig;
if (sign)
*p++ = '-';
if (e is 0)
{
assert(dst.length > decimals + 7);
if (exp < 0)
x *= pow10(abs + 1);
*p++ = cast(T)toDigit(x,count);
if (decimals)
{
*p++ = '.';
while (decimals-- > 0)
*p++ = cast(T)toDigit(x,count);
if (pad is false)
{
while (*(p - 1) is '0')
--p;
if (*(p - 1) is '.')
--p;
}
}
if (abs)
{
*p++ = 'e';
*p++ = exp < 0 ? '-' : '+';
if (abs >= 1000)
{
*p++ = cast(T)(abs / 1000 + '0');
abs %= 1000;
*p++ = cast(T)(abs / 100 + '0');
abs %= 100;
}
else
if (abs >= 100)
{
*p++ = cast(T)(abs / 100 + '0');
abs %= 100;
}
*p++ = cast(T)(abs / 10 + '0');
*p++ = cast(T)(abs % 10 + '0');
}
}
else
{
assert(dst.length >= (exp < 0 ? 0 : exp) + decimals + 1);
if (exp < 0)
{
x *= pow10(abs);
*p++ = '0';
}
else
for (; exp >= 0; --exp)
{
*p++ = cast(T)toDigit(x,count);
}
if (decimals)
{
*p++ = '.';
{
for (++exp; exp < 0 && decimals > 0; --decimals , ++exp)
{
*p++ = '0';
}
}
while (decimals-- > 0)
*p++ = cast(T)toDigit(x,count);
if (pad is false)
{
while (*(p - 1) is '0')
--p;
if (*(p - 1) is '.')
--p;
}
}
}
return dst[0..p - dst.ptr];
}
}
}
debug (UnitTest)
{
    import tango.io.Console;
    }
debug (Float)
{
    import tango.io.Console;
    void main();
}
