// D import file generated from 'tango/text/convert/UnicodeBom.d'
module tango.text.convert.UnicodeBom;
private import tango.core.ByteSwap;

private import Utf = tango.text.convert.Utf;

private extern (C) void onUnicodeError(const(char[]) msg, size_t idx = 0);


enum Encoding 
{
Unknown,
UTF_8N,
UTF_8,
UTF_16,
UTF_16BE,
UTF_16LE,
UTF_32,
UTF_32BE,
UTF_32LE,
}
template UnicodeBom(T)
{
class UnicodeBom : BomSniffer
{
    static if(!is(T == char) && !is(T == wchar) && !is(T == dchar))
{
    pragma (msg, "Template type must be char, wchar, or dchar");
}
    this(Encoding encoding)
{
setup(encoding);
}
    final T[] decode(void[] content, T[] dst = null, size_t* ate = null)
{
auto info = test(content);
if (lookup[encoding].test)
if (info)
{
setup(info.encoding,true);
content = content[info.bom.length..$];
}
else
if (settings.fallback)
setup(settings.fallback,false);
else
onUnicodeError("UnicodeBom.decode :: unknown or missing BOM");
else
if (info)
onUnicodeError("UnicodeBom.decode :: explicit encoding does not permit BOM");
auto ret = into(swapBytes(content),settings.type,dst,ate);
if (ate && info)
*ate += info.bom.length;
return ret;
}

    final void[] encode(T[] content, void[] dst = null)
{
if (settings.test)
onUnicodeError("UnicodeBom.encode :: cannot write to a non-specific encoding");
return swapBytes(from(content,settings.type,dst));
}

    private final void[] swapBytes(void[] content)
{
bool endian = settings.endian;
bool swap = settings.bigEndian;
version (BigEndian)
{
swap = !swap;
}

if (endian && swap)
{
if (settings.type == Utf16)
ByteSwap.swap16(content.ptr,content.length);
else
ByteSwap.swap32(content.ptr,content.length);
}
return content;
}


    static T[] into(void[] x, uint type, T[] dst = null, size_t* ate = null)
{
T[] ret;
static if(is(T == char))
{
if (type == Utf8)
{
if (ate)
*ate = x.length;
ret = cast(char[])x;
}
else
if (type == Utf16)
ret = Utf.toString(cast(wchar[])x,dst,ate);
else
if (type == Utf32)
ret = Utf.toString(cast(dchar[])x,dst,ate);
}

static if(is(T == wchar))
{
if (type == Utf16)
{
if (ate)
*ate = x.length;
ret = cast(wchar[])x;
}
else
if (type == Utf8)
ret = Utf.toString16(cast(char[])x,dst,ate);
else
if (type == Utf32)
ret = Utf.toString16(cast(dchar[])x,dst,ate);
}

static if(is(T == dchar))
{
if (type == Utf32)
{
if (ate)
*ate = x.length;
ret = cast(dchar[])x;
}
else
if (type == Utf8)
ret = Utf.toString32(cast(char[])x,dst,ate);
else
if (type == Utf16)
ret = Utf.toString32(cast(wchar[])x,dst,ate);
}

return ret;
}

    static void[] from(T[] x, uint type, void[] dst = null, size_t* ate = null)
{
void[] ret;
static if(is(T == char))
{
if (type == Utf8)
{
if (ate)
*ate = x.length;
ret = x;
}
else
if (type == Utf16)
ret = Utf.toString16(x,cast(wchar[])dst,ate);
else
if (type == Utf32)
ret = Utf.toString32(x,cast(dchar[])dst,ate);
}

static if(is(T == wchar))
{
if (type == Utf16)
{
if (ate)
*ate = x.length;
ret = x;
}
else
if (type == Utf8)
ret = Utf.toString(x,cast(char[])dst,ate);
else
if (type == Utf32)
ret = Utf.toString32(x,cast(dchar[])dst,ate);
}

static if(is(T == dchar))
{
if (type == Utf32)
{
if (ate)
*ate = x.length;
ret = x;
}
else
if (type == Utf8)
ret = Utf.toString(x,cast(char[])dst,ate);
else
if (type == Utf16)
ret = Utf.toString16(x,cast(wchar[])dst,ate);
}

return ret;
}

}
}
class BomSniffer
{
    private bool found;

    private Encoding encoder;

    private const(Info)* settings;

    private struct Info
{
    int type;
    Encoding encoding;
    const(char)[] bom;
    bool test;
    bool endian;
    bool bigEndian;
    Encoding fallback;
}

    private enum 
{
Utf8,
Utf16,
Utf32,
}

    private const Info[] lookup = [{Utf8,Encoding.Unknown,null,true,false,false,Encoding.UTF_8},{Utf8,Encoding.UTF_8N,null,true,false,false,Encoding.UTF_8},{Utf8,Encoding.UTF_8,"\xef\xbb\xbf",false},{Utf16,Encoding.UTF_16,null,true,false,false,Encoding.UTF_16BE},{Utf16,Encoding.UTF_16BE,"\xfe\xff",false,true,true},{Utf16,Encoding.UTF_16LE,"\xff\xfe",false,true},{Utf32,Encoding.UTF_32,null,true,false,false,Encoding.UTF_32BE},{Utf32,Encoding.UTF_32BE,"\x00\x00\xfe\xff",false,true,true},{Utf32,Encoding.UTF_32LE,"\xff\xfe\x00\x00",false,true}];


    final @property Encoding encoding()
{
return encoder;
}

    final @property bool encoded()
{
return found;
}

    final @property const(void)[] signature()
{
return settings.bom;
}

    final void setup(Encoding encoding, bool found = false)
{
this.settings = &lookup[encoding];
this.encoder = encoding;
this.found = found;
}

    static final const(Info)* test(void[] content)
{
{
for (const(Info)* info = lookup.ptr + lookup.length;
 --info >= lookup.ptr;)
{
if (info.bom)
{
size_t len = info.bom.length;
if (len <= content.length)
if (content[0..len] == info.bom[0..len])
return info;
}
}
}
return null;
}

}
debug (UnitTest)
{
    }
debug (UnicodeBom)
{
    import tango.io.Stdout;
    void main()
{
void[] INPUT2 = "abc\xe3\x81\x82\xe3\x81\x84\xe3\x81\x86".dup;
void[] INPUT = "\xef\xbb\xbf" ~ INPUT2;
auto bom = new UnicodeBom!(char)(Encoding.Unknown);
size_t ate;
char[256] buf;
auto temp = bom.decode(INPUT,buf,&ate);
assert(temp == INPUT2);
assert(ate == INPUT.length);
assert(bom.encoding == Encoding.UTF_8);
temp = bom.decode(INPUT2,buf,&ate);
assert(temp == INPUT2);
assert(ate == INPUT2.length);
assert(bom.encoding == Encoding.UTF_8);
}
}
