// D import file generated from 'tango/text/json/Json.d'
module tango.text.json.Json;
private import tango.core.Vararg;

private import tango.io.model.IConduit;

private import tango.text.json.JsonEscape;

private import tango.text.json.JsonParser;

private import Float = tango.text.convert.Float;

template Json(T)
{
class Json : JsonParser!(T)
{
    public alias JsonValue* Value;

    public alias NameValue* Attribute;

    public alias JsonObject* Composite;

    public enum Type 
{
Null,
String,
RawString,
Number,
Object,
Array,
True,
False,
}

    private Value root;

    this()
{
arrays.length = 16;
parse(null);
}
    final Value parse(const(T)[] json)
{
nesting = 0;
attrib.reset();
values.reset();
objects.reset();
foreach (ref p; arrays)
{
p.index = 0;
}
root = createValue();
if (super.reset(json))
{
if (curType is Token.BeginObject)
root.set(parseObject());
else
if (curType is Token.BeginArray)
root.set(parseArray());
else
exception("invalid json document");
}
return root;
}

    final T[] toString(const(T)[] space = null, int decimals = 2)
{
return root.print(space,decimals);
}

    final Value value()
{
return root;
}

    final Value value(Value v)
{
return root = v;
}

    final Value value(const(T)[] v)
{
return createValue().set(v);
}

    final Value value(bool v)
{
return createValue().set(v);
}

    final Value value(double v)
{
return createValue().set(v);
}

    final Value value(Value[] vals)
{
return createValue().set(vals);
}

    final Value array(...)
{
return createValue().set(this,_arguments,_argptr);
}

    Attribute pair(const(T)[] name, Value value = null)
{
if (value is null)
value = createValue();
return createAttribute().set(name,value);
}
    final Value object(Attribute[] set...)
{
return createValue().set(createObject().add(set));
}

    private Value createValue()
{
return values.allocate().reset();
}

    private Composite createObject()
{
return objects.allocate().reset();
}

    private Attribute createAttribute()
{
return attrib.allocate();
}

    private void exception(immutable(char)[] msg)
{
throw new Exception(msg);
}

    private Value parseValue()
{
auto v = values.allocate();
switch (super.curType)
{
case Token.True:
{
v.set(Type.True);
break;
}
case Token.False:
{
v.set(Type.False);
break;
}
case Token.Null:
{
v.set(Type.Null);
break;
}
case Token.BeginObject:
{
v.set(parseObject());
break;
}
case Token.BeginArray:
{
v.set(parseArray());
break;
}
case Token.String:
{
v.set(super.value,true);
break;
}
case Token.Number:
{
v.set(Float.parse(super.value));
break;
}
default:
{
v.set(Type.Null);
break;
}
}
return v;
}

    private Composite parseObject()
{
auto o = objects.allocate().reset();
while (super.next)
{
if (super.curType is Token.EndObject)
return o;
if (super.curType != Token.Name)
super.expected("an attribute-name",super.str.ptr);
auto name = super.value;
if (!super.next)
super.expected("an attribute-value",super.str.ptr);
o.append(attrib.allocate().set(name,parseValue()));
}
return o;
}

    private Value[] parseArray()
{
if (nesting >= arrays.length)
exception("array nesting too deep within document");
auto array = &arrays[nesting++];
auto start = array.index;
while (super.next && super.curType != Token.EndArray)
{
if (array.index >= array.content.length)
array.content.length = array.content.length + 300;
array.content[array.index++] = parseValue();
}
if (super.curType != Token.EndArray)
exception("malformed array");
--nesting;
return array.content[start..array.index];
}

    struct NameValue
{
    private Attribute next;

    public const(T)[] name;

    public Value value;

    Attribute set(const(T)[] key, Value val)
{
name = key;
value = val;
return &this;
}
}
    struct JsonObject
{
    private 
{
    Attribute head;
    Attribute tail;
}
    Composite reset()
{
head = (tail = null);
return &this;
}
    Composite append(Attribute a)
{
if (tail)
tail.next = a , tail = a;
else
head = (tail = a);
return &this;
}
    Composite add(Attribute[] set...)
{
foreach (attr; set)
{
append(attr);
}
return &this;
}
    Value[T[]] hashmap()
{
Value[T[]] members;
auto a = head;
while (a)
{
members[a.name] = a.value;
a = a.next;
}
return members;
}
    Value value(const(T)[] name)
{
auto a = head;
while (a)
if (name == a.name)
return a.value;
else
a = a.next;
return null;
}
    @property Iterator attributes()
{
Iterator i = {head};
return i;
}

    static struct Iterator
{
    private Attribute head;

    int opApply(int delegate(ref const(T)[] key, ref Value val) dg)
{
int res;
auto a = head;
while (a)
{
if ((res = dg(a.name,a.value)) != 0)
break;
a = a.next;
}
return res;
}
}

}
    struct JsonValue
{
    private union
{
Value[] array;
real number;
const(T)[] string;
Composite object;
}

    public Type type;

    alias reset set;
    bool equals(Type t)
{
return type is t;
}
    bool toBool()
{
return type is Type.True;
}
    T[] toString(T[] dst = null)
{
if (type is Type.RawString)
return string.dup;
if (type is Type.String)
return unescape(string,dst);
return null;
}
    bool toString(void delegate(const(T)[]) dg)
{
if (type is Type.RawString)
dg(string);
else
if (type is Type.String)
unescape(string,dg);
else
return false;
return true;
}
    Composite toObject()
{
return type is Type.Object ? object : null;
}
    real toNumber()
{
return type is Type.Number ? number : (real).nan;
}
    Value[] toArray()
{
return type is Type.Array ? array : null;
}
    Value set(const(T)[] str, bool escaped = false)
{
type = escaped ? Type.String : Type.RawString;
string = str;
return &this;
}
    Value set(Composite obj)
{
type = Type.Object;
object = obj;
return &this;
}
    Value set(real num)
{
type = Type.Number;
number = num;
return &this;
}
    Value set(bool b)
{
type = b ? Type.True : Type.False;
return &this;
}
    Value set(Value[] a)
{
type = Type.Array;
array = a;
return &this;
}
    Value reset()
{
type = Type.Null;
return &this;
}
    T[] print(const(T)[] space = null, int decimals = 2)
{
T[] tmp;
void append(const(T)[] s)
{
tmp ~= s;
}
print(&append,space,decimals);
return tmp;
}
    Value print(OutputStream s, const(T)[] space = null, int decimals = 2)
{
return print(delegate (const(T)[] t)
{
s.write(t);
}
,space,decimals);
}
    Value print(void delegate(const(T)[]) append, const(T)[] space = null, int decimals = 2)
{
auto indent = 0;
@property void newline()
{
if (space.length)
{
append("\x0a");
{
for (auto i = 0;
 i < indent; i++)
{
append(space);
}
}
}
}
void printValue(Value val)
{
void printObject(Composite obj)
{
if (obj is null)
return ;
bool first = true;
append("{");
indent++;
foreach (k, v; obj.attributes)
{
if (!first)
append(",");
newline;
append("\"") , append(k) , append("\":");
printValue(v);
first = false;
}
indent--;
newline;
append("}");
}
void printArray(Value[] arr)
{
bool first = true;
append("[");
indent++;
foreach (v; arr)
{
if (!first)
append(", ");
newline;
printValue(v);
first = false;
}
indent--;
newline;
append("]");
}
if (val is null)
return ;
switch (val.type)
{
T[64] tmp = void;
case Type.String:
{
append("\"") , append(val.string) , append("\"");
break;
}
case Type.RawString:
{
append("\"") , escape(val.string,append) , append("\"");
break;
}
case Type.Number:
{
append(Float.format(tmp,val.toNumber(),decimals));
break;
}
case Type.Object:
{
auto obj = val.toObject();
debug (1)
{
assert(obj !is null);
}

printObject(val.toObject());
break;
}
case Type.Array:
{
printArray(val.toArray());
break;
}
case Type.True:
{
append("true");
break;
}
case Type.False:
{
append("false");
break;
}
default:
{
}
case Type.Null:
{
append("null");
break;
}
}
}
printValue(&this);
return &this;
}
    private Value set(Type type)
{
this.type = type;
return &this;
}

    private Value set(Json host, TypeInfo[] info, va_list args)
{
Value[] list;
foreach (type; info)
{
Value v;
if (type is typeid(Value))
v = va_arg!(Value)(args);
else
{
v = host.createValue();
if (type is typeid(double))
v.set(va_arg!(double)(args));
else
if (type is typeid(int))
v.set(va_arg!(int)(args));
else
if (type is typeid(bool))
v.set(va_arg!(bool)(args));
else
if (type is typeid(long))
v.set(va_arg!(long)(args));
else
if (type is typeid(Composite))
v.set(va_arg!(Composite)(args));
else
if (type is typeid(T[]))
v.set(va_arg!(T[])(args));
else
if (type is typeid(void*))
va_arg!(void*)(args);
else
host.exception("JsonValue.set :: unexpected type: " ~ type.toString());
}
list ~= v;
}
return forwardref_buf(&this,list);
}

    private static Value forwardref_buf(Value v, Value[] list)
{
return v.set(list);
}


}
    private template Allocator(T)
{
struct Allocator
{
    private T[] list;

    private T[][] lists;

    private 
{
    int index;
    int block;
}
    void reset()
{
lists.length = 0;
block = -1;
newlist;
}
    T* allocate()
{
if (index >= list.length)
newlist;
auto p = &list[index++];
return p;
}
    @property private void newlist()
{
index = 0;
if (++block >= lists.length)
{
lists.length = lists.length + 1;
lists[$ - 1] = new T[](256);
}
list = lists[block];
}


}
}

    private struct Array
{
    uint index;
    Value[] content;
}

    private alias Allocator!(NameValue) Attrib;

    private alias Allocator!(JsonValue) Values;

    private alias Allocator!(JsonObject) Objects;

    private Attrib attrib;

    private Values values;

    private Array[] arrays;

    private Objects objects;

    private uint nesting;

}
}
debug (UnitTest)
{
            }
debug (Json)
{
    import tango.io.Stdout;
    import tango.io.device.File;
    import tango.time.StopWatch;
    void main();
}
