// D import file generated from 'tango/text/json/JsonParser.d'
module tango.text.json.JsonParser;
private import tango.util.container.more.Stack;

template JsonParser(T)
{
class JsonParser
{
    public enum Token 
{
Empty,
Name,
String,
Number,
BeginObject,
EndObject,
BeginArray,
EndArray,
True,
False,
Null,
}

    private enum State 
{
Object,
Array,
}

    private struct Iterator
{
    const(T)* ptr;
    const(T)* end;
    const(T)[] text;
    void reset(const(T)[] text)
{
this.text = text;
this.ptr = text.ptr;
this.end = ptr + text.length;
}
}

    protected Iterator str;

    private Stack!(State,16) state;

    private const(T)* curLoc;

    private size_t curLen;

    private State curState;

    protected Token curType;

    this(const(T)[] text = null)
{
reset(text);
}
    final @property bool next()
{
if (str.ptr is null || str.end is null)
return false;
auto p = str.ptr;
auto e = str.end;
while (*p <= 32 && p < e)
++p;
if ((str.ptr = p) >= e)
return false;
if (curState is State.Array)
return parseArrayValue();
switch (curType)
{
case Token.Name:
{
return parseMemberValue();
}
default:
{
break;
}
}
return parseMemberName();
}

    final @property Token type()
{
return curType;
}

    final @property const(T)[] value()
{
return curLoc[0..curLen];
}

    bool reset(const(T)[] json = null)
{
state.clear();
str.reset(json);
curType = Token.Empty;
curState = State.Object;
if (json.length)
{
auto p = str.ptr;
auto e = str.end;
while (*p <= 32 && p < e)
++p;
if (p < e)
return start(*(str.ptr = p));
}
return false;
}
    protected final void expected(immutable(char)[] token)
{
throw new Exception("expected " ~ token);
}


    protected final void expected(immutable(char)[] token, const(T)* point)
{
static char[] itoa(char[] buf, size_t i)
{
auto p = buf.ptr + buf.length;
do
{
*--p = '0' + i % 10;
}
while (i /= 10);return p[0..buf.ptr + buf.length - p];
}
char[32] tmp = void;
expected(token ~ " @input[" ~ itoa(tmp,point - str.text.ptr).idup ~ "]");
}


    private void unexpectedEOF(immutable(char)[] msg)
{
throw new Exception("unexpected end-of-input: " ~ msg);
}

    private bool start(T c)
{
if (c is '{')
return push(Token.BeginObject,State.Object);
if (c is '[')
return push(Token.BeginArray,State.Array);
expected("'{' or '[' at start of document");
assert(0);
}

    private bool parseMemberName()
{
auto p = str.ptr;
auto e = str.end;
if (*p is '}')
return pop(Token.EndObject);
if (*p is ',')
++p;
while (*p <= 32)
++p;
if (*p != '"')
{
if (*p == '}')
expected("an attribute-name after (a potentially trailing) ','",p);
else
expected("'\"' before attribute-name",p);
}
curLoc = p + 1;
curType = Token.Name;
while (++p < e)
if (*p is '"' && !escaped(p))
break;
if (p < e)
curLen = p - curLoc;
else
unexpectedEOF("in attribute-name");
str.ptr = p + 1;
return true;
}

    private bool parseMemberValue()
{
auto p = str.ptr;
if (*p != ':')
expected("':' before attribute-value",p);
auto e = str.end;
while (++p < e && *p <= 32)
{
}
return parseValue(*(str.ptr = p));
}

    private bool parseValue(T c)
{
switch (c)
{
case '{':
{
return push(Token.BeginObject,State.Object);
}
case '[':
{
return push(Token.BeginArray,State.Array);
}
case '"':
{
return doString();
}
case 'n':
{
if (match("null",Token.Null))
return true;
expected("'null'",str.ptr);
break;
}
case 't':
{
if (match("true",Token.True))
return true;
expected("'true'",str.ptr);
break;
}
case 'f':
{
if (match("false",Token.False))
return true;
expected("'false'",str.ptr);
break;
}
default:
{
break;
}
}
return parseNumber();
}

    private bool doString()
{
auto p = str.ptr;
auto e = str.end;
curLoc = p + 1;
curType = Token.String;
while (++p < e)
if (*p is '"' && !escaped(p))
break;
if (p < e)
curLen = p - curLoc;
else
unexpectedEOF("in string");
str.ptr = p + 1;
return true;
}

    private bool parseNumber()
{
auto p = str.ptr;
auto e = str.end;
T c = *(curLoc = p);
curType = Token.Number;
if (c is '-' || c is '+')
c = *++p;
while (c >= '0' && c <= '9')
c = *++p;
if (c is '.')
while (c = *++p , c >= '0' && c <= '9')
{
}
if (c is 'e' || c is 'E')
while (c = *++p , c >= '0' && c <= '9')
{
}
if (p < e)
curLen = p - curLoc;
else
unexpectedEOF("after number");
str.ptr = p;
return curLen > 0;
}

    private bool match(const(T)[] name, Token token)
{
auto i = name.length;
if (str.ptr[0..i] == name)
{
curLoc = str.ptr;
curType = token;
str.ptr += i;
curLen = i;
return true;
}
return false;
}

    private bool push(Token token, State next)
{
curLen = 0;
curType = token;
curLoc = str.ptr++;
state.push(curState);
curState = next;
return true;
}

    private bool pop(Token token)
{
curLen = 0;
curType = token;
curLoc = str.ptr++;
curState = state.pop();
return true;
}

    private bool parseArrayValue()
{
auto p = str.ptr;
if (*p is ']')
return pop(Token.EndArray);
if (*p is ',')
++p;
auto e = str.end;
while (p < e && *p <= 32)
++p;
return parseValue(*(str.ptr = p));
}

    private int escaped(const(T)* p)
{
int i;
while (*--p is '\x5c')
++i;
return i & 1;
}

}
}
debug (UnitTest)
{
    immutable(char)[] json = "{\"glossary\": {\"title\": \"example glossary\",\"GlossDiv\": {   \"title\": \"S\",   \"GlossList\": {       \"GlossEntry\": {           \"ID\": \"SGML\",           \"SortAs\": \"SGML\",           \"GlossTerm\": \"Standard Generalized Markup Language\",           \"Acronym\": \"SGML\",           \"Abbrev\": \"ISO 8879:1986\",           \"GlossDef\": {                \"para\": \"A meta-markup language, used to create markup languages such as DocBook.\",            \"GlossSeeAlso\": [\"GML\", \"XML\"]           },          \"GlossSee\": \"markup\",          \"ANumber\": 12345.6e7          \"True\": true          \"False\": false          \"Null\": null        }    }}}}";
    }
debug (JsonParser)
{
    void main()
{
auto json = new JsonParser!(char);
}
}
