// D import file generated from 'tango/text/locale/Core.d'
module tango.text.locale.Core;
private import tango.core.Exception;

private import tango.text.locale.Data;

private import tango.time.Time;

private 
{
    import tango.time.chrono.Hijri;
    import tango.time.chrono.Korean;
    import tango.time.chrono.Taiwan;
    import tango.time.chrono.Hebrew;
    import tango.time.chrono.Calendar;
    import tango.time.chrono.Japanese;
    import tango.time.chrono.Gregorian;
    import tango.time.chrono.ThaiBuddhist;
}
version (Windows)
{
    private import tango.text.locale.Win32;

}
version (Posix)
{
    private import tango.text.locale.Posix;

}
private template arrayOf(T)
{
private T[] arrayOf(T[] params...)
{
return params.dup;
}

}

public enum CultureTypes 
{
Neutral = 1,
Specific = 2,
All = Neutral | Specific,
}

public interface IFormatService
{
    Object getFormat(TypeInfo type);
}

public class Culture : IFormatService
{
    private enum int LCID_INVARIANT = 127;

    private static Culture[immutable(char)[]] namedCultures;


    private static Culture[int] idCultures;


    private static Culture[immutable(char)[]] ietfCultures;


    private static Culture currentCulture_;


    private static Culture userDefaultCulture_;


    private static Culture invariantCulture_;


    private Calendar calendar_;

    private Culture parent_;

    private const(CultureData)* cultureData_;

    private bool isReadOnly_;

    private NumberFormat numberFormat_;

    private DateTimeFormat dateTimeFormat_;

    shared static this();
        public this(const(char)[] cultureName)
{
cultureData_ = CultureData.getDataFromCultureName(cultureName);
}

    public this(int cultureID)
{
cultureData_ = CultureData.getDataFromCultureID(cultureID);
}

    public Object getFormat(TypeInfo type)
{
if (type is typeid(NumberFormat))
return numberFormat;
else
if (type is typeid(DateTimeFormat))
return dateTimeFormat;
return null;
}

    version (Clone)
{
    public Object clone()
{
Culture culture = cast(Culture)cloneObject(this);
if (!culture.isNeutral)
{
if (dateTimeFormat_ !is null)
culture.dateTimeFormat_ = cast(DateTimeFormat)dateTimeFormat_.clone();
if (numberFormat_ !is null)
culture.numberFormat_ = cast(NumberFormat)numberFormat_.clone();
}
if (calendar_ !is null)
culture.calendar_ = cast(Calendar)calendar_.clone();
return culture;
}

}
    public static Culture getCulture(int cultureID);


    public static Culture getCulture(const(char)[] cultureName)
{
if (cultureName is null)
error("Value cannot be null.");
Culture culture = getCultureInternal(0,cultureName);
if (culture is null)
error("Culture name " ~ cultureName.idup ~ " is not supported.");
return culture;
}


    public static Culture getCultureFromIetfLanguageTag(const(char)[] name)
{
if (name is null)
error("Value cannot be null.");
Culture culture = getCultureInternal(-1,name);
if (culture is null)
error("Culture IETF name " ~ name.idup ~ " is not a known IETF name.");
return culture;
}


    private static Culture getCultureInternal(int cultureID, const(char)[] cname);


    public static Culture[] getCultures(CultureTypes types);


    public override immutable(char)[] toString()
{
return cultureData_.name.idup;
}


    public override bool opEquals(Object obj)
{
if (obj is this)
return true;
Culture other = cast(Culture)obj;
if (other is null)
return false;
return other.name == name;
}


    @property public static Culture current()
{
if (currentCulture_ !is null)
return currentCulture_;
if (userDefaultCulture_ !is null)
{
if (userDefaultCulture_.id != nativeMethods.getUserCulture())
userDefaultCulture_ = null;
}
if (userDefaultCulture_ is null)
{
userDefaultCulture_ = new Culture(nativeMethods.getUserCulture());
if (userDefaultCulture_ is null)
userDefaultCulture_ = invariantCulture;
else
userDefaultCulture_.isReadOnly_ = true;
}
return userDefaultCulture_;
}



    @property public static void current(Culture value)
{
checkNeutral(value);
nativeMethods.setUserCulture(value.id);
currentCulture_ = value;
}



    @property public static Culture invariantCulture()
{
return invariantCulture_;
}



    @property public const int id()
{
return cultureData_.lcid;
}



    @property public const const(char)[] name()
{
return cultureData_.name;
}



    @property public const const(char)[] englishName()
{
return cultureData_.englishName;
}



    @property public const const(char)[] nativeName()
{
return cultureData_.nativeName;
}



    @property public const const(char)[] twoLetterLanguageName()
{
return cultureData_.isoLangName;
}



    @property public const const(char)[] threeLetterLanguageName()
{
return cultureData_.isoLangName2;
}



    @property public const final const(char)[] ietfLanguageTag()
{
return cultureData_.ietfTag;
}



    @property public Culture parent();


    @property public const bool isNeutral()
{
return cultureData_.isNeutral;
}



    @property public const final bool isReadOnly()
{
return isReadOnly_;
}



    @property public Calendar calendar()
{
if (calendar_ is null)
{
calendar_ = getCalendarInstance(cultureData_.calendarType,isReadOnly_);
}
return calendar_;
}


    @property public Calendar[] optionalCalendars();


    @property public NumberFormat numberFormat()
{
checkNeutral(this);
if (numberFormat_ is null)
{
numberFormat_ = new NumberFormat(cultureData_);
numberFormat_.isReadOnly_ = isReadOnly_;
}
return numberFormat_;
}


    @property public void numberFormat(NumberFormat value)
{
checkReadOnly();
numberFormat_ = value;
}


    @property public DateTimeFormat dateTimeFormat()
{
checkNeutral(this);
if (dateTimeFormat_ is null)
{
dateTimeFormat_ = new DateTimeFormat(cultureData_,calendar);
dateTimeFormat_.isReadOnly_ = isReadOnly_;
}
return dateTimeFormat_;
}


    @property public void dateTimeFormat(DateTimeFormat value)
{
checkReadOnly();
dateTimeFormat_ = value;
}


    private static void checkNeutral(Culture culture)
{
if (culture.isNeutral)
error("Culture '" ~ culture.name.idup ~ "' is a neutral culture. It cannot be used in formatting and therefore cannot be set as the current culture.");
}


    private void checkReadOnly()
{
if (isReadOnly_)
error("Instance is read-only.");
}

    private static Calendar getCalendarInstance(int calendarType, bool readOnly = false);


}

public class Region
{
    private const(CultureData)* cultureData_;

    private static Region currentRegion_;


    private const(char)[] name_;

    public this(int cultureID)
{
cultureData_ = CultureData.getDataFromCultureID(cultureID);
if (cultureData_.isNeutral)
error("Cannot use a neutral culture to create a region.");
name_ = cultureData_.regionName;
}

    public this(const(char)[] name)
{
cultureData_ = CultureData.getDataFromRegionName(name);
name_ = name;
if (cultureData_.isNeutral)
error("The region name " ~ name.idup ~ " corresponds to a neutral culture and cannot be used to create a region.");
}

    package this(const(CultureData)* cultureData)
{
cultureData_ = cultureData;
name_ = cultureData.regionName;
}

    @property public static Region current()
{
if (currentRegion_ is null)
currentRegion_ = new Region(Culture.current.cultureData_);
return currentRegion_;
}



    @property public const int geoID()
{
return cultureData_.geoId;
}



    @property public const const(char)[] name()
{
return name_;
}



    @property public const const(char)[] englishName()
{
return cultureData_.englishCountry;
}



    @property public const const(char)[] nativeName()
{
return cultureData_.nativeCountry;
}



    @property public const const(char)[] twoLetterRegionName()
{
return cultureData_.regionName;
}



    @property public const const(char)[] threeLetterRegionName()
{
return cultureData_.isoRegionName;
}



    @property public const const(char)[] currencySymbol()
{
return cultureData_.currency;
}



    @property public const const(char)[] isoCurrencySymbol()
{
return cultureData_.intlSymbol;
}



    @property public const const(char)[] currencyEnglishName()
{
return cultureData_.englishCurrency;
}



    @property public const const(char)[] currencyNativeName()
{
return cultureData_.nativeCurrency;
}



    @property public const bool isMetric()
{
return cultureData_.isMetric;
}



    public override immutable(char)[] toString()
{
return name_.idup;
}


}

public class NumberFormat : IFormatService
{
    package bool isReadOnly_;

    private static NumberFormat invariantFormat_;


    private int numberDecimalDigits_;

    private int numberNegativePattern_;

    private int currencyDecimalDigits_;

    private int currencyNegativePattern_;

    private int currencyPositivePattern_;

    private const(int)[] numberGroupSizes_;

    private const(int)[] currencyGroupSizes_;

    private const(char)[] numberGroupSeparator_;

    private const(char)[] numberDecimalSeparator_;

    private const(char)[] currencyGroupSeparator_;

    private const(char)[] currencyDecimalSeparator_;

    private const(char)[] currencySymbol_;

    private const(char)[] negativeSign_;

    private const(char)[] positiveSign_;

    private const(char)[] nanSymbol_;

    private const(char)[] negativeInfinitySymbol_;

    private const(char)[] positiveInfinitySymbol_;

    private const(char[])[] nativeDigits_;

    public this()
{
this(null);
}

    package this(const(CultureData)* cultureData)
{
numberDecimalDigits_ = 2;
numberNegativePattern_ = 1;
currencyDecimalDigits_ = 2;
numberGroupSizes_ = arrayOf!(int)(3);
currencyGroupSizes_ = arrayOf!(int)(3);
numberGroupSeparator_ = ",";
numberDecimalSeparator_ = ".";
currencyGroupSeparator_ = ",";
currencyDecimalSeparator_ = ".";
currencySymbol_ = "\xc2\xa4";
negativeSign_ = "-";
positiveSign_ = "+";
nanSymbol_ = "NaN";
negativeInfinitySymbol_ = "-Infinity";
positiveInfinitySymbol_ = "Infinity";
nativeDigits_ = arrayOf!(const(char)[])("0","1","2","3","4","5","6","7","8","9");
if (cultureData !is null && cultureData.lcid != Culture.LCID_INVARIANT)
{
numberDecimalDigits_ = cultureData.digits;
numberNegativePattern_ = cultureData.negativeNumber;
currencyDecimalDigits_ = cultureData.currencyDigits;
currencyNegativePattern_ = cultureData.negativeCurrency;
currencyPositivePattern_ = cultureData.positiveCurrency;
numberGroupSizes_ = cultureData.grouping;
currencyGroupSizes_ = cultureData.monetaryGrouping;
numberGroupSeparator_ = cultureData.thousand;
numberDecimalSeparator_ = cultureData.decimal;
currencyGroupSeparator_ = cultureData.monetaryThousand;
currencyDecimalSeparator_ = cultureData.monetaryDecimal;
currencySymbol_ = cultureData.currency;
negativeSign_ = cultureData.negativeSign;
positiveSign_ = cultureData.positiveSign;
nanSymbol_ = cultureData.nan;
negativeInfinitySymbol_ = cultureData.negInfinity;
positiveInfinitySymbol_ = cultureData.posInfinity;
nativeDigits_ = cultureData.nativeDigits;
}
}

    public Object getFormat(TypeInfo type)
{
return type is typeid(NumberFormat) ? this : null;
}

    version (Clone)
{
    public Object clone()
{
NumberFormat copy = cast(NumberFormat)cloneObject(this);
copy.isReadOnly_ = false;
return copy;
}

}
    public static NumberFormat getInstance(IFormatService formatService)
{
Culture culture = cast(Culture)formatService;
if (culture !is null)
{
if (culture.numberFormat_ !is null)
return culture.numberFormat_;
return culture.numberFormat;
}
if (NumberFormat numberFormat = cast(NumberFormat)formatService)
return numberFormat;
if (formatService !is null)
{
if (NumberFormat numberFormat = cast(NumberFormat)formatService.getFormat(typeid(NumberFormat)))
return numberFormat;
}
return current;
}


    @property public static NumberFormat current()
{
return Culture.current.numberFormat;
}



    public static NumberFormat invariantFormat()
{
if (invariantFormat_ is null)
{
invariantFormat_ = new NumberFormat;
invariantFormat_.isReadOnly_ = true;
}
return invariantFormat_;
}


    @property public const final bool isReadOnly()
{
return isReadOnly_;
}



    @property public const final int numberDecimalDigits()
{
return numberDecimalDigits_;
}



    @property public final void numberDecimalDigits(int value)
{
checkReadOnly();
numberDecimalDigits_ = value;
}



    @property public const final int numberNegativePattern()
{
return numberNegativePattern_;
}



    @property public final void numberNegativePattern(int value)
{
checkReadOnly();
numberNegativePattern_ = value;
}



    @property public const final int currencyDecimalDigits()
{
return currencyDecimalDigits_;
}



    @property public final void currencyDecimalDigits(int value)
{
checkReadOnly();
currencyDecimalDigits_ = value;
}



    @property public const final int currencyNegativePattern()
{
return currencyNegativePattern_;
}



    @property public final void currencyNegativePattern(int value)
{
checkReadOnly();
currencyNegativePattern_ = value;
}



    @property public const final int currencyPositivePattern()
{
return currencyPositivePattern_;
}



    @property public final void currencyPositivePattern(int value)
{
checkReadOnly();
currencyPositivePattern_ = value;
}



    @property public const final const(int)[] numberGroupSizes()
{
return numberGroupSizes_;
}



    @property public final void numberGroupSizes(const(int)[] value)
{
checkReadOnly();
numberGroupSizes_ = value;
}



    @property public const final const(int)[] currencyGroupSizes()
{
return currencyGroupSizes_;
}



    @property public final void currencyGroupSizes(const(int)[] value)
{
checkReadOnly();
currencyGroupSizes_ = value;
}



    @property public const final const(char)[] numberGroupSeparator()
{
return numberGroupSeparator_;
}



    @property public final void numberGroupSeparator(const(char)[] value)
{
checkReadOnly();
numberGroupSeparator_ = value;
}



    @property public const final const(char)[] numberDecimalSeparator()
{
return numberDecimalSeparator_;
}



    @property public final void numberDecimalSeparator(const(char)[] value)
{
checkReadOnly();
numberDecimalSeparator_ = value;
}



    @property public const final const(char)[] currencyGroupSeparator()
{
return currencyGroupSeparator_;
}



    @property public final void currencyGroupSeparator(const(char)[] value)
{
checkReadOnly();
currencyGroupSeparator_ = value;
}



    @property public const final const(char)[] currencyDecimalSeparator()
{
return currencyDecimalSeparator_;
}



    @property public final void currencyDecimalSeparator(const(char)[] value)
{
checkReadOnly();
currencyDecimalSeparator_ = value;
}



    @property public const final const(char)[] currencySymbol()
{
return currencySymbol_;
}



    @property public final void currencySymbol(const(char)[] value)
{
checkReadOnly();
currencySymbol_ = value;
}



    @property public const final const(char)[] negativeSign()
{
return negativeSign_;
}



    @property public final void negativeSign(const(char)[] value)
{
checkReadOnly();
negativeSign_ = value;
}



    @property public const final const(char)[] positiveSign()
{
return positiveSign_;
}



    @property public final void positiveSign(const(char)[] value)
{
checkReadOnly();
positiveSign_ = value;
}



    @property public const final const(char)[] nanSymbol()
{
return nanSymbol_;
}



    @property public final void nanSymbol(const(char)[] value)
{
checkReadOnly();
nanSymbol_ = value;
}



    @property public const final const(char)[] negativeInfinitySymbol()
{
return negativeInfinitySymbol_;
}



    @property public final void negativeInfinitySymbol(const(char)[] value)
{
checkReadOnly();
negativeInfinitySymbol_ = value;
}



    @property public const final const(char)[] positiveInfinitySymbol()
{
return positiveInfinitySymbol_;
}



    @property public final void positiveInfinitySymbol(const(char)[] value)
{
checkReadOnly();
positiveInfinitySymbol_ = value;
}



    @property public const final const(char[])[] nativeDigits()
{
return nativeDigits_;
}



    @property public final void nativeDigits(const(char[])[] value)
{
checkReadOnly();
nativeDigits_ = value;
}



    @property private const void checkReadOnly()
{
if (isReadOnly_)
error("NumberFormat instance is read-only.");
}



}

public class DateTimeFormat : IFormatService
{
    private enum const(char)[] rfc1123Pattern_ = "ddd, dd MMM yyyy HH':'mm':'ss 'GMT'";

    private enum const(char)[] sortableDateTimePattern_ = "yyyy'-'MM'-'dd'T'HH':'mm':'ss";

    private enum const(char)[] universalSortableDateTimePattern_ = "yyyy'-'MM'-'dd' 'HH':'mm':'ss'Z'";

    private enum const(char)[] allStandardFormats = ['d','D','f','F','g','G','m','M','r','R','s','t','T','u','U','y','Y'];

    package bool isReadOnly_;

    private static DateTimeFormat invariantFormat_;


    private const(CultureData)* cultureData_;

    private Calendar calendar_;

    private const(int)[] optionalCalendars_;

    private int firstDayOfWeek_ = -1;

    private int calendarWeekRule_ = -1;

    private const(char)[] dateSeparator_;

    private const(char)[] timeSeparator_;

    private const(char)[] amDesignator_;

    private const(char)[] pmDesignator_;

    private const(char)[] shortDatePattern_;

    private const(char)[] shortTimePattern_;

    private const(char)[] longDatePattern_;

    private const(char)[] longTimePattern_;

    private const(char)[] monthDayPattern_;

    private const(char)[] yearMonthPattern_;

    private const(char[])[] abbreviatedDayNames_;

    private const(char[])[] dayNames_;

    private const(char[])[] abbreviatedMonthNames_;

    private const(char[])[] monthNames_;

    private const(char)[] fullDateTimePattern_;

    private const(char)[] generalShortTimePattern_;

    private const(char)[] generalLongTimePattern_;

    private const(char[])[] shortTimePatterns_;

    private const(char[])[] shortDatePatterns_;

    private const(char[])[] longTimePatterns_;

    private const(char[])[] longDatePatterns_;

    private const(char[])[] yearMonthPatterns_;

    package this()
{
cultureData_ = Culture.invariantCulture.cultureData_;
calendar_ = Gregorian.generic;
initialize();
}

    package this(const(CultureData)* cultureData, Calendar calendar)
{
cultureData_ = cultureData;
this.calendar = calendar;
}

    public Object getFormat(TypeInfo type)
{
return type is typeid(DateTimeFormat) ? this : null;
}

    version (Clone)
{
    public Object clone()
{
DateTimeFormat other = cast(DateTimeFormat)cloneObject(this);
other.calendar_ = cast(Calendar)calendar.clone();
other.isReadOnly_ = false;
return other;
}

}
    @property package const(char)[][] shortTimePatterns()
{
if (shortTimePatterns_ is null)
shortTimePatterns_ = cultureData_.shortTimes;
return shortTimePatterns_.dup;
}


    @property package const(char)[][] shortDatePatterns()
{
if (shortDatePatterns_ is null)
shortDatePatterns_ = cultureData_.shortDates;
return shortDatePatterns_.dup;
}


    @property package const(char)[][] longTimePatterns()
{
if (longTimePatterns_ is null)
longTimePatterns_ = cultureData_.longTimes;
return longTimePatterns_.dup;
}


    @property package const(char)[][] longDatePatterns()
{
if (longDatePatterns_ is null)
longDatePatterns_ = cultureData_.longDates;
return longDatePatterns_.dup;
}


    @property package const(char[])[] yearMonthPatterns()
{
if (yearMonthPatterns_ is null)
yearMonthPatterns_ = cultureData_.yearMonths;
return yearMonthPatterns_;
}


    public final const(char)[][] getAllDateTimePatterns();


    public final const(char)[][] getAllDateTimePatterns(char format);


    public final const(char)[] getAbbreviatedDayName(Calendar.DayOfWeek dayOfWeek)
{
return abbreviatedDayNames[cast(int)dayOfWeek];
}


    public final const(char)[] getDayName(Calendar.DayOfWeek dayOfWeek)
{
return dayNames[cast(int)dayOfWeek];
}


    public final const(char)[] getAbbreviatedMonthName(int month)
{
return abbreviatedMonthNames[month - 1];
}


    public final const(char)[] getMonthName(int month)
{
return monthNames[month - 1];
}


    public static DateTimeFormat getInstance(IFormatService formatService)
{
Culture culture = cast(Culture)formatService;
if (culture !is null)
{
if (culture.dateTimeFormat_ !is null)
return culture.dateTimeFormat_;
return culture.dateTimeFormat;
}
if (DateTimeFormat dateTimeFormat = cast(DateTimeFormat)formatService)
return dateTimeFormat;
if (formatService !is null)
{
if (DateTimeFormat dateTimeFormat = cast(DateTimeFormat)formatService.getFormat(typeid(DateTimeFormat)))
return dateTimeFormat;
}
return current;
}


    @property public static DateTimeFormat current()
{
return Culture.current.dateTimeFormat;
}



    public static DateTimeFormat invariantFormat()
{
if (invariantFormat_ is null)
{
invariantFormat_ = new DateTimeFormat;
invariantFormat_.calendar = new Gregorian;
invariantFormat_.isReadOnly_ = true;
}
return invariantFormat_;
}


    @property public final bool isReadOnly()
{
return isReadOnly_;
}



    @property public final Calendar calendar()
{
assert(calendar_ !is null);
return calendar_;
}



    @property public final void calendar(Calendar value)
{
checkReadOnly();
if (value !is calendar_)
{
{
for (int i = 0;
 i < optionalCalendars.length; i++)
{
{
if (optionalCalendars[i] == value.id)
{
if (calendar_ !is null)
{
shortDatePattern_ = null;
longDatePattern_ = null;
shortTimePattern_ = null;
yearMonthPattern_ = null;
monthDayPattern_ = null;
generalShortTimePattern_ = null;
generalLongTimePattern_ = null;
fullDateTimePattern_ = null;
shortDatePatterns_ = null;
longDatePatterns_ = null;
yearMonthPatterns_ = null;
abbreviatedDayNames_ = null;
abbreviatedMonthNames_ = null;
dayNames_ = null;
monthNames_ = null;
}
calendar_ = value;
initialize();
return ;
}
}
}
}
error("Not a valid calendar for the culture.");
}
}



    @property public final Calendar.DayOfWeek firstDayOfWeek()
{
return cast(Calendar.DayOfWeek)firstDayOfWeek_;
}



    @property public final void firstDayOfWeek(Calendar.DayOfWeek value)
{
checkReadOnly();
firstDayOfWeek_ = value;
}



    @property public final Calendar.WeekRule calendarWeekRule()
{
return cast(Calendar.WeekRule)calendarWeekRule_;
}



    @property public final void calendarWeekRule(Calendar.WeekRule value)
{
checkReadOnly();
calendarWeekRule_ = value;
}



    @property public final const(char)[] nativeCalendarName()
{
return cultureData_.nativeCalName;
}



    @property public final const(char)[] dateSeparator()
{
if (dateSeparator_ is null)
dateSeparator_ = cultureData_.date;
return dateSeparator_;
}



    @property public final void dateSeparator(const(char)[] value)
{
checkReadOnly();
dateSeparator_ = value;
}



    @property public final const(char)[] timeSeparator()
{
if (timeSeparator_ is null)
timeSeparator_ = cultureData_.time;
return timeSeparator_;
}



    @property public final void timeSeparator(const(char)[] value)
{
checkReadOnly();
timeSeparator_ = value;
}



    @property public final const(char)[] amDesignator()
{
assert(amDesignator_ !is null);
return amDesignator_;
}



    @property public final void amDesignator(const(char)[] value)
{
checkReadOnly();
amDesignator_ = value;
}



    @property public final const(char)[] pmDesignator()
{
assert(pmDesignator_ !is null);
return pmDesignator_;
}



    @property public final void pmDesignator(const(char)[] value)
{
checkReadOnly();
pmDesignator_ = value;
}



    @property public final const(char)[] shortDatePattern()
{
assert(shortDatePattern_ !is null);
return shortDatePattern_;
}



    @property public final void shortDatePattern(const(char)[] value)
{
checkReadOnly();
if (shortDatePatterns_ !is null)
shortDatePatterns_ = [value];
shortDatePattern_ = value;
generalLongTimePattern_ = null;
generalShortTimePattern_ = null;
}



    @property public final const(char)[] shortTimePattern()
{
if (shortTimePattern_ is null)
shortTimePattern_ = cultureData_.shortTime;
return shortTimePattern_;
}



    @property public final void shortTimePattern(const(char)[] value)
{
checkReadOnly();
shortTimePattern_ = value;
generalShortTimePattern_ = null;
}



    @property public final const(char)[] longDatePattern()
{
assert(longDatePattern_ !is null);
return longDatePattern_;
}



    @property public final void longDatePattern(const(char)[] value)
{
checkReadOnly();
if (longDatePatterns_ !is null)
longDatePatterns_ = [value];
longDatePattern_ = value;
fullDateTimePattern_ = null;
}



    @property public final const(char)[] longTimePattern()
{
assert(longTimePattern_ !is null);
return longTimePattern_;
}



    @property public final void longTimePattern(const(char)[] value)
{
checkReadOnly();
longTimePattern_ = value;
fullDateTimePattern_ = null;
}



    @property public final const(char)[] monthDayPattern()
{
if (monthDayPattern_ is null)
monthDayPattern_ = cultureData_.monthDay;
return monthDayPattern_;
}



    @property public final void monthDayPattern(const(char)[] value)
{
checkReadOnly();
monthDayPattern_ = value;
}



    @property public final const(char)[] yearMonthPattern()
{
assert(yearMonthPattern_ !is null);
return yearMonthPattern_;
}



    @property public final void yearMonthPattern(const(char)[] value)
{
checkReadOnly();
yearMonthPattern_ = value;
}



    @property public final const(char)[][] abbreviatedDayNames()
{
if (abbreviatedDayNames_ is null)
abbreviatedDayNames_ = cultureData_.abbrevDayNames;
return abbreviatedDayNames_.dup;
}



    @property public final void abbreviatedDayNames(const(char)[][] value)
{
checkReadOnly();
abbreviatedDayNames_ = value;
}



    @property public final const(char)[][] dayNames()
{
if (dayNames_ is null)
dayNames_ = cultureData_.dayNames;
return dayNames_.dup;
}



    @property public final void dayNames(const(char)[][] value)
{
checkReadOnly();
dayNames_ = value;
}



    @property public final const(char)[][] abbreviatedMonthNames()
{
if (abbreviatedMonthNames_ is null)
abbreviatedMonthNames_ = cultureData_.abbrevMonthNames;
return abbreviatedMonthNames_.dup;
}



    @property public final void abbreviatedMonthNames(const(char)[][] value)
{
checkReadOnly();
abbreviatedMonthNames_ = value;
}



    @property public final const(char)[][] monthNames()
{
if (monthNames_ is null)
monthNames_ = cultureData_.monthNames;
return monthNames_.dup;
}



    @property public final void monthNames(const(char)[][] value)
{
checkReadOnly();
monthNames_ = value;
}



    @property public final const(char)[] fullDateTimePattern()
{
if (fullDateTimePattern_ is null)
fullDateTimePattern_ = longDatePattern ~ " " ~ longTimePattern;
return fullDateTimePattern_;
}



    @property public final void fullDateTimePattern(const(char)[] value)
{
checkReadOnly();
fullDateTimePattern_ = value;
}



    @property public const final const(char)[] rfc1123Pattern()
{
return rfc1123Pattern_;
}



    @property public const final const(char)[] sortableDateTimePattern()
{
return sortableDateTimePattern_;
}



    @property public const final const(char)[] universalSortableDateTimePattern()
{
return universalSortableDateTimePattern_;
}



    @property package const(char)[] generalShortTimePattern()
{
if (generalShortTimePattern_ is null)
generalShortTimePattern_ = shortDatePattern ~ " " ~ shortTimePattern;
return generalShortTimePattern_;
}


    @property package const(char)[] generalLongTimePattern()
{
if (generalLongTimePattern_ is null)
generalLongTimePattern_ = shortDatePattern ~ " " ~ longTimePattern;
return generalLongTimePattern_;
}


    @property private const void checkReadOnly()
{
if (isReadOnly_)
error("DateTimeFormat instance is read-only.");
}



    private void initialize()
{
if (longTimePattern_ is null)
longTimePattern_ = cultureData_.longTime;
if (shortDatePattern_ is null)
shortDatePattern_ = cultureData_.shortDate;
if (longDatePattern_ is null)
longDatePattern_ = cultureData_.longDate;
if (yearMonthPattern_ is null)
yearMonthPattern_ = cultureData_.yearMonth;
if (amDesignator_ is null)
amDesignator_ = cultureData_.am;
if (pmDesignator_ is null)
pmDesignator_ = cultureData_.pm;
if (firstDayOfWeek_ is -1)
firstDayOfWeek_ = cultureData_.firstDayOfWeek;
if (calendarWeekRule_ == -1)
calendarWeekRule_ = cultureData_.firstDayOfYear;
}

    @property private const(int)[] optionalCalendars()
{
if (optionalCalendars_ is null)
optionalCalendars_ = cultureData_.optionalCalendars;
return optionalCalendars_;
}


    private const void error(immutable(char)[] msg);


}

