// D import file generated from 'tango/time/ISO8601.d'
module tango.time.ISO8601;
public import tango.time.Time;

public import tango.time.chrono.Gregorian;

import tango.core.Exception;
import tango.math.Math;
private alias Time DT;

private alias ExtendedDate FullDate;

public struct ExtendedDate
{
    DT val;
    private int year_;

    const @property int year()
out(val)
{
assert(val >= -1000000000 && val <= -1 || val >= 1 && val <= 999999999);
}
body
{
if (year_)
return year_;
auto era = Gregorian.generic.getEra(val);
if (era == Gregorian.AD_ERA)
return Gregorian.generic.getYear(val);
else
return -Gregorian.generic.getYear(val);
}

    @property private void year(int y)
{
if (DTyear(y))
{
year_ = 0;
int toAdd = y - Gregorian.generic.getYear(val);
val = Gregorian.generic.addYears(val,toAdd);
}
else
year_ = y < 0 ? y - 1 : y;
}


    private byte mask;

    const @property uint seconds()
{
return val.time.seconds + (mask >>> 0 & 1);
}

    alias seconds secs;
    alias seconds second;
    alias seconds sec;
    const @property bool endOfDay()
{
return 1 == (mask >>> 1 & 1);
}

    private void setLeap()
{
mask |= 1 << 0;
}

    private void setEndOfDay()
{
mask |= 1 << 1;
}

    debug (Tango_ISO8601)
{
    private char[] toStr()
{
return Stdout.layout.convert("{:d} and {:d}-{:d2}-{:d2} :: {:d2}:{:d2}:{:d2}.{:d3} and {:d2}, {}",year_,years(*this),months(*this),days(*this),hours(*this),mins(*this),.secs(*this),ms(*this),this.seconds,this.endOfDay);
}

}
}

public template parseDate(T)
{
size_t parseDate(T[] src, ref DT dt)
{
auto fd = FullDate(dt);
auto ret = parseDate(src,fd);
dt = fd.val;
return ret;
}
}

public template parseDate(T)
{
size_t parseDate(T[] src, ref FullDate fd, ubyte expanded = 0)
{
ubyte dummy = void;
T* p = src.ptr;
return doIso8601Date(p,src,fd,expanded,dummy);
}
}

private template doIso8601Date(T)
{
size_t doIso8601Date(ref T* p, T[] src, ref FullDate fd, ubyte expanded, out ubyte separators)
{
if (expanded > 5)
throw new IllegalArgumentException("ISO8601 :: year expanded by more than 5 digits does not fit in int");
size_t eaten()
{
return p - src.ptr;
}
size_t remaining()
{
return src.length - eaten();
}
bool done(const(T[]) s)
{
return .done(eaten(),src.length,p,s);
}
if (!parseYear(p,src.length,expanded,fd))
return 0;
auto onlyYear = eaten();
if (done("-0123W"))
return onlyYear;
if (accept(p,'-'))
{
separators = YES;
if (remaining() == 0)
return eaten() - 1;
}
if (accept(p,'W'))
{
T* p2 = p;
int i = parseInt(p,min(cast(size_t)3,remaining()));
if (i)
{
if (p - p2 == 2)
{
if (done("-"))
{
if (getMonthAndDayFromWeek(fd,i))
return eaten();
}
else
if (demand(p,'-'))
{
if (remaining() == 0)
return eaten() - 1;
if (separators == NO)
{
if (getMonthAndDayFromWeek(fd,i))
return eaten() - 1;
}
else
if (getMonthAndDayFromWeek(fd,i,*p++ - '0'))
return eaten();
}
}
else
if (p - p2 == 3)
{
if (separators == YES)
{
if (getMonthAndDayFromWeek(fd,i / 10))
return eaten() - 1;
}
else
if (getMonthAndDayFromWeek(fd,i / 10,i % 10))
return eaten();
}
}
return onlyYear;
}
T* p2 = p;
int i = parseInt(p,remaining());
if (!i)
return onlyYear;
switch (p - p2)
{
case 2:
{
if (i >= 1 && i <= 12)
addMonths(fd,i);
else
return onlyYear;
auto onlyMonth = eaten();
if (done("-") || !demand(p,'-') || separators == NO)
return onlyMonth;
int day = parseInt(p,min(cast(size_t)2,remaining()));
if (day && day <= daysPerMonth(months(fd),fd.year))
addDays(fd,day);
else
return onlyMonth;
break;
}
case 4:
{
int month = i / 100;
int day = i % 100;
if (separators == YES)
{
p -= 2;
i = month;
goto case 2;
}
if (month >= 1 && month <= 12 && day >= 1 && day <= daysPerMonth(month,fd.year))
{
addMonths(fd,month);
addDays(fd,day);
}
else
return onlyYear;
break;
}
case 3:
{
if (i > 365 + isLeapYear(fd.year))
return onlyYear;
addDays(fd,i);
break;
}
default:
{
break;
}
}
return eaten();
}
}

public template parseTime(T)
{
size_t parseTime(T[] src, ref DT dt)
{
auto fd = FullDate(dt);
auto ret = parseTime(src,fd);
dt = fd.val;
return ret;
}
}

public template parseTime(T)
{
size_t parseTime(T[] src, ref FullDate fd)
{
bool dummy = void;
T* p = src.ptr;
return doIso8601Time(p,src,fd,WHATEVER,dummy);
}
}

private enum : ubyte
{
NO = 0,
YES = 1,
WHATEVER,
}

private template doIso8601Time(T)
{
size_t doIso8601Time(ref T* p, T[] src, ref FullDate fd, ubyte separators, out bool bothValid)
{
size_t eaten()
{
return p - src.ptr;
}
size_t remaining()
{
return src.length - eaten();
}
bool done(const(T[]) s)
{
return .done(eaten(),src.length,p,s);
}
bool checkColon()
{
return .checkColon(p,separators);
}
byte getTimeZone()
{
return .getTimeZone(p,remaining(),fd,separators,&done);
}
if (separators == WHATEVER)
accept(p,'T');
int hour = void;
if (parseInt(p,min(cast(size_t)2,remaining()),hour) != 2 || hour > 24)
return 0;
if (hour == 24)
fd.setEndOfDay();
addHours(fd,hour);
auto onlyHour = eaten();
if (done("+,-.012345:"))
return onlyHour;
switch (getDecimal(p,remaining(),fd,HOUR))
{
case NOTFOUND:
{
break;
}
case FOUND:
{
auto onlyDecimal = eaten();
if (getTimeZone() == BAD)
return onlyDecimal;
return eaten();
}
case BAD:
{
return onlyHour;
}
default:
{
assert(false);
}
}
switch (getTimeZone())
{
case NOTFOUND:
{
break;
}
case FOUND:
{
return eaten();
}
case BAD:
{
return onlyHour;
}
default:
{
assert(false);
}
}
if (!checkColon())
return onlyHour;
int mins = void;
if (parseInt(p,min(cast(size_t)2,remaining()),mins) != 2 || mins > 59 || fd.endOfDay && mins != 0)
return onlyHour;
addMins(fd,mins);
auto onlyMinute = eaten();
if (done("+,-.0123456:"))
{
bothValid = true;
return onlyMinute;
}
switch (getDecimal(p,remaining(),fd,MINUTE))
{
case NOTFOUND:
{
break;
}
case FOUND:
{
auto onlyDecimal = eaten();
if (getTimeZone() == BAD)
return onlyDecimal;
bothValid = true;
return eaten();
}
case BAD:
{
return onlyMinute;
}
default:
{
assert(false);
}
}
switch (getTimeZone())
{
case NOTFOUND:
{
break;
}
case FOUND:
{
bothValid = true;
return eaten();
}
case BAD:
{
return onlyMinute;
}
default:
{
assert(false);
}
}
if (!checkColon())
return onlyMinute;
int sec = void;
if (parseInt(p,min(cast(size_t)2,remaining()),sec) != 2 || sec > 60 || fd.endOfDay && sec != 0)
return onlyMinute;
if (sec == 60)
{
if (hours(fd) != 23 && .mins(fd) != 59)
return onlyMinute;
fd.setLeap();
--sec;
}
addSecs(fd,sec);
auto onlySecond = eaten();
if (done("+,-.Z"))
{
bothValid = true;
return onlySecond;
}
switch (getDecimal(p,remaining(),fd,SECOND))
{
case NOTFOUND:
{
break;
}
case FOUND:
{
auto onlyDecimal = eaten();
if (getTimeZone() == BAD)
return onlyDecimal;
bothValid = true;
return eaten();
}
case BAD:
{
return onlySecond;
}
default:
{
assert(false);
}
}
if (getTimeZone() == BAD)
return onlySecond;
else
{
bothValid = true;
return eaten();
}
}
}

public template parseDateAndTime(T)
{
size_t parseDateAndTime(T[] src, ref DT dt)
{
FullDate fd;
auto ret = parseDateAndTime(src,fd);
dt = fd.val;
return ret;
}
}

public template parseDateAndTime(T)
{
size_t parseDateAndTime(T[] src, ref FullDate fd)
{
T* p = src.ptr;
ubyte sep;
bool bothValid = false;
if (doIso8601Date(p,src,fd,cast(ubyte)0,sep) && src.length - (p - src.ptr) >= 1 && demand(p,'T') && doIso8601Time(p,src,fd,sep,bothValid) && bothValid)
return p - src.ptr;
else
return 0;
}
}

private 
{
    template parseYear(T)
{
bool parseYear(ref T* p, size_t len, ubyte expanded, ref FullDate fd)
{
int year = void;
bool doParse()
{
T* p2 = p;
if (!parseInt(p,min(cast(size_t)(expanded + 4),len),year))
return false;
if (p - p2 - expanded == 2)
year *= 100;
else
if (p - p2 - expanded != 4)
return false;
return true;
}
if (accept(p,'-'))
{
if (!doParse() || year < 0)
return false;
year = -year;
}
else
{
accept(p,'+');
if (!doParse() || year < 0)
return false;
}
fd.year = year;
return true;
}
}
    bool getMonthAndDayFromWeek(ref FullDate fd, int week, int day = 1)
{
if (week < 1 || week > 53 || day < 1 || day > 7)
return false;
int year = fd.year;
if (week == 53)
{
int startingDay = dayOfWeek(year,1,1);
if (!(startingDay == 4 || isLeapYear(year) && startingDay == 3))
return false;
}
addDays(fd,7 * (week - 1) + day - dayOfWeek(year,1,4) + 4);
return true;
}
    bool isLeapYear(int year)
{
return year % 4 == 0 && (year % 100 != 0 || year % 400 == 0);
}
    int dayOfWeek(int year, int month, int day)
in
{
assert(month >= 1 && month <= 12);
assert(day >= 1 && day <= 31);
}
out(result)
{
assert(result >= 1 && result <= 7);
}
body
{
uint era = erafy(year);
int result = Gregorian.generic.getDayOfWeek(Gregorian.generic.toTime(year,month,day,0,0,0,0,era));
if (result == Gregorian.DayOfWeek.Sunday)
return 7;
else
return result;
}
    enum : ubyte
{
HOUR,
MINUTE,
SECOND,
}
    enum : byte
{
BAD,
FOUND,
NOTFOUND,
}
    template checkColon(T)
{
bool checkColon(ref T* p, ref ubyte separators)
{
ubyte foundSep = accept(p,':') ? YES : NO;
if (foundSep != separators)
{
if (separators == WHATEVER)
separators = foundSep;
else
return false;
}
return true;
}
}
    template getDecimal(T)
{
byte getDecimal(ref T* p, size_t len, ref FullDate fd, ubyte which)
{
if (!(accept(p,',') || accept(p,'.')))
return NOTFOUND;
T* p2 = p;
int i = void;
auto iLen = parseInt(p,len - 1,i);
if (iLen == 0 || i == 0 && iLen < 3)
return BAD;
int pow = 1;
while (--iLen)
pow *= 10;
switch (which)
{
case HOUR:
{
addMins(fd,6 * i / pow);
addSecs(fd,6 * i % pow);
break;
}
case MINUTE:
{
addSecs(fd,6 * i / pow);
addMs(fd,6000 * i / pow % 1000);
break;
}
case SECOND:
{
addMs(fd,100 * i / pow);
break;
}
default:
{
assert(false);
}
}
return FOUND;
}
}
    template getTimeZone(T)
{
byte getTimeZone(ref const(T)* p, size_t len, ref FullDate fd, ubyte separators, bool delegate(const(T[])) done)
{
bool checkColon()
{
return .checkColon(p,separators);
}
if (len == 0)
return NOTFOUND;
auto p0 = p;
if (accept(p,'Z'))
return FOUND;
int factor = -1;
if (accept(p,'-'))
factor = 1;
else
if (!accept(p,'+'))
return NOTFOUND;
int hour = void;
if (parseInt(p,min(cast(size_t)2,len - 1),hour) != 2 || hour > 12 || hour == 0 && factor == 1)
return BAD;
addHours(fd,factor * hour);
if (factor > 0 && hours(fd) == 0 && mins(fd) == 0 && secs(fd) == 0 && ms(fd) == 0)
fd.setEndOfDay();
if (done("012345:"))
return FOUND;
auto afterHours = p;
if (!checkColon())
return BAD;
int minute = void;
if (parseInt(p,min(cast(size_t)2,len - (p - p0)),minute) != 2)
{
p = afterHours;
return FOUND;
}
addMins(fd,factor * minute);
if (factor > 0 && hours(fd) == 0 && mins(fd) == 0 && secs(fd) == 0 && ms(fd) == 0)
fd.setEndOfDay();
return FOUND;
}
}
    template accept(T)
{
bool accept(ref T* p, char c)
{
if (*p == c)
{
++p;
return true;
}
return false;
}
}
    template demand(T)
{
bool demand(ref T* p, char c)
{
return *p++ == c;
}
}
    template done(T)
{
bool done(size_t eaten, size_t srcLen, const(T*) p, const(T[]) s)
{
if (eaten == srcLen)
return true;
T t = *p;
foreach (c; s)
{
if (t < c)
return true;
else
if (t == c)
break;
}
return false;
}
}
    int daysPerMonth(int month, int year)
{
uint era = erafy(year);
return Gregorian.generic.getDaysInMonth(year,month,era);
}
    uint erafy(ref int year)
{
if (year < 0)
{
year *= -1;
return Gregorian.BC_ERA;
}
else
return Gregorian.AD_ERA;
}
    template parseInt(T)
{
int parseInt(ref T* p, size_t max)
{
size_t i = 0;
int value = 0;
while (i < max && p[i] >= '0' && p[i] <= '9')
value = value * 10 + p[i++] - '0';
p += i;
return value;
}
}
    template parseInt(T)
{
size_t parseInt(ref T* p, size_t max, out int i)
{
T* p2 = p;
i = parseInt(p,max);
return p - p2;
}
}
    bool DTyear(int year)
{
return year >= -10000 && year <= 9999;
}
    void addMonths(ref FullDate d, int n)
{
d.val = Gregorian.generic.addMonths(d.val,n - 1);
}
    void addDays(ref FullDate d, int n)
{
d.val += TimeSpan.fromDays(n - 1);
}
    void addHours(ref FullDate d, int n)
{
d.val += TimeSpan.fromHours(n);
}
    void addMins(ref FullDate d, int n)
{
d.val += TimeSpan.fromMinutes(n);
}
    void addSecs(ref FullDate d, int n)
{
d.val += TimeSpan.fromSeconds(n);
}
    void addMs(ref FullDate d, int n)
{
d.val += TimeSpan.fromMillis(n);
}
    int years(const(FullDate) d)
{
return Gregorian.generic.getYear(d.val);
}
    int months(const(FullDate) d)
{
return Gregorian.generic.getMonth(d.val);
}
    int days(const(FullDate) d)
{
return Gregorian.generic.getDayOfMonth(d.val);
}
    int hours(const(FullDate) d)
{
return d.val.time.hours;
}
    int mins(const(FullDate) d)
{
return d.val.time.minutes;
}
    int secs(const(FullDate) d)
{
return d.val.time.seconds;
}
    int ms(const(FullDate) d)
{
return d.val.time.millis;
}
    debug (UnitTest)
{
    debug (Tango_ISO8601_Valgrind)
{
    import tango.stdc.stdlib;
}
    }
}
