// D import file generated from 'tango/time/WallClock.d'
module tango.time.WallClock;
public import tango.time.Time;

private import tango.time.Clock;

private import tango.sys.Common;

struct WallClock
{
    version (Win32)
{
    @property static Time now()
{
return Clock.now - localBias();
}


    @property static TimeSpan zone()
{
TIME_ZONE_INFORMATION tz = void;
auto tmp = GetTimeZoneInformation(&tz);
return TimeSpan.fromMinutes(-tz.Bias);
}


    static DateTime toDate()
{
return toDate(Clock.now);
}

    static DateTime toDate(const(Time) utc)
{
return Clock.toDate(utc - localBias());
}

    static Time fromDate(ref const(DateTime) date)
{
return Clock.fromDate(date) + localBias();
}

    private static TimeSpan localBias();


}
    version (Posix)
{
    @property static Time now()
{
tm t = void;
timeval tv = void;
gettimeofday(&tv,null);
localtime_r(&tv.tv_sec,&t);
tv.tv_sec = timegm(&t);
return Clock.convert(tv);
}


    @property static TimeSpan zone();


    static DateTime toDate()
{
return toDate(Clock.now);
}

    static DateTime toDate(const(Time) utc)
{
DateTime dt = void;
auto timeval = Clock.convert(utc);
dt.time.millis = cast(uint)timeval.tv_usec / 1000;
tm t = void;
localtime_r(&timeval.tv_sec,&t);
dt.date.year = t.tm_year + 1900;
dt.date.month = t.tm_mon + 1;
dt.date.day = t.tm_mday;
dt.date.dow = t.tm_wday;
dt.date.era = 0;
dt.time.hours = t.tm_hour;
dt.time.minutes = t.tm_min;
dt.time.seconds = t.tm_sec;
Clock.setDoy(dt);
return dt;
}

    static Time fromDate(ref const(DateTime) dt)
{
tm t = void;
t.tm_year = dt.date.year - 1900;
t.tm_mon = dt.date.month - 1;
t.tm_mday = dt.date.day;
t.tm_hour = dt.time.hours;
t.tm_min = dt.time.minutes;
t.tm_sec = dt.time.seconds;
auto seconds = mktime(&t);
return Time.epoch1970 + TimeSpan.fromSeconds(seconds) + TimeSpan.fromMillis(dt.time.millis);
}

}
    static Time toLocal(const(Time) utc)
{
auto mod = utc.ticks % TimeSpan.TicksPerMillisecond;
auto date = toDate(utc);
return Clock.fromDate(date) + TimeSpan(mod);
}

    static Time toUtc(const(Time) wall)
{
auto mod = wall.ticks % TimeSpan.TicksPerMillisecond;
auto date = Clock.toDate(wall);
return fromDate(date) + TimeSpan(mod);
}

}
version (Posix)
{
    version (darwin)
{
}
else
{
    shared static this();
}
}
