// D import file generated from 'tango/time/chrono/Calendar.d'
module tango.time.chrono.Calendar;
public import tango.time.Time;

private import tango.core.Exception;

public abstract class Calendar
{
    package enum 
{
CURRENT_ERA = 0,
}

    package enum 
{
GREGORIAN = 1,
GREGORIAN_US = 2,
JAPAN = 3,
TAIWAN = 4,
KOREA = 5,
HIJRI = 6,
THAI = 7,
HEBREW = 8,
GREGORIAN_ME_FRENCH = 9,
GREGORIAN_ARABIC = 10,
GREGORIAN_XLIT_ENGLISH = 11,
GREGORIAN_XLIT_FRENCH = 12,
}

    package enum WeekRule 
{
FirstDay,
FirstFullWeek,
FirstFourDayWeek,
}

    package enum DatePart 
{
Year,
Month,
Day,
DayOfYear,
}

    public enum DayOfWeek 
{
Sunday,
Monday,
Tuesday,
Wednesday,
Thursday,
Friday,
Saturday,
}

    const Date toDate(const(Time) time)
{
Date d;
split(time,d.year,d.month,d.day,d.doy,d.dow,d.era);
return d;
}

    const void split(const(Time) time, ref uint year, ref uint month, ref uint day, ref uint doy, ref uint dow, ref uint era)
{
year = getYear(time);
month = getMonth(time);
day = getDayOfMonth(time);
doy = getDayOfYear(time);
dow = getDayOfWeek(time);
era = getEra(time);
}

    const Time toTime(uint year, uint month, uint day, uint hour, uint minute, uint second, uint millisecond = 0)
{
return toTime(year,month,day,hour,minute,second,millisecond,CURRENT_ERA);
}

    const Time toTime(const(Date) d)
{
return toTime(d.year,d.month,d.day,0,0,0,0,d.era);
}

    const Time toTime(const(DateTime) dt)
{
return toTime(dt.date,dt.time);
}

    const Time toTime(const(Date) d, const(TimeOfDay) t)
{
return toTime(d.year,d.month,d.day,t.hours,t.minutes,t.seconds,t.millis,d.era);
}

    const abstract Time toTime(uint year, uint month, uint day, uint hour, uint minute, uint second, uint millisecond, uint era);

    const abstract DayOfWeek getDayOfWeek(const(Time) time);

    const abstract uint getDayOfMonth(const(Time) time);

    const abstract uint getDayOfYear(const(Time) time);

    const abstract uint getMonth(const(Time) time);

    const abstract uint getYear(const(Time) time);

    const abstract uint getEra(const(Time) time);

    const uint getDaysInMonth(uint year, uint month)
{
return getDaysInMonth(year,month,CURRENT_ERA);
}

    const abstract uint getDaysInMonth(uint year, uint month, uint era);

    const uint getDaysInYear(uint year)
{
return getDaysInYear(year,CURRENT_ERA);
}

    const abstract uint getDaysInYear(uint year, uint era);

    const uint getMonthsInYear(uint year)
{
return getMonthsInYear(year,CURRENT_ERA);
}

    const abstract uint getMonthsInYear(uint year, uint era);

    const uint getWeekOfYear(const(Time) time, WeekRule rule, DayOfWeek firstDayOfWeek);

    const bool isLeapYear(uint year)
{
return isLeapYear(year,CURRENT_ERA);
}

    const abstract bool isLeapYear(uint year, uint era);

    const abstract @property uint[] eras();

    const @property uint id()
{
return -1;
}

    const Time addMonths(const(Time) t, int nMonths, bool truncateDay = false);

    const Time addYears(const(Time) t, int nYears)
{
auto date = toDate(t);
auto tod = t.ticks % TimeSpan.TicksPerDay;
if (tod < 0)
tod += TimeSpan.TicksPerDay;
date.year += nYears;
return toTime(date) + TimeSpan(tod);
}

    package static long getTimeTicks(uint hour, uint minute, uint second)
{
return (TimeSpan.fromHours(hour) + TimeSpan.fromMinutes(minute) + TimeSpan.fromSeconds(second)).ticks;
}


}


