// D import file generated from 'tango/time/chrono/Hebrew.d'
module tango.time.chrono.Hebrew;
private import tango.core.Exception;

private import tango.time.chrono.Calendar;

public class Hebrew : Calendar
{
    private enum uint[14][7] MonthDays = [[0,0,0,0,0,0,0,0,0,0,0,0,0,0],[0,30,29,29,29,30,29,0,30,29,30,29,30,29],[0,30,29,30,29,30,29,0,30,29,30,29,30,29],[0,30,30,30,29,30,29,0,30,29,30,29,30,29],[0,30,29,29,29,30,30,29,30,29,30,29,30,29],[0,30,29,30,29,30,30,29,30,29,30,29,30,29],[0,30,30,30,29,30,30,29,30,29,30,29,30,29]];

    private enum uint YearOfOneAD = 3760;

    private enum uint DaysToOneAD = cast(int)(YearOfOneAD * 0xb.6a3020c49ba5e35p+5);

    private enum uint PartsPerHour = 1080;

    private enum uint PartsPerDay = 24 * PartsPerHour;

    private enum uint DaysPerMonth = 29;

    private enum uint DaysPerMonthFraction = 12 * PartsPerHour + 793;

    private enum uint PartsPerMonth = DaysPerMonth * PartsPerDay + DaysPerMonthFraction;

    private enum uint FirstNewMoon = 11 * PartsPerHour + 204;

    private uint minYear_ = YearOfOneAD + 1583;

    private uint maxYear_ = YearOfOneAD + 2240;

    public enum uint HEBREW_ERA = 1;

    public const override Time toTime(uint year, uint month, uint day, uint hour, uint minute, uint second, uint millisecond, uint era)
{
checkYear(year,era);
return getGregorianTime(year,month,day,hour,minute,second,millisecond);
}


    public const override DayOfWeek getDayOfWeek(const(Time) time)
{
return cast(DayOfWeek)cast(uint)((time.ticks / TimeSpan.TicksPerDay + 1) % 7);
}


    public const override uint getDayOfMonth(const(Time) time);


    public const override uint getDayOfYear(const(Time) time)
{
auto year = getYear(time);
auto days = getStartOfYear(year) - DaysToOneAD;
return cast(uint)(time.ticks / TimeSpan.TicksPerDay) - days + 1;
}


    public const override uint getMonth(const(Time) time);


    public const override uint getYear(const(Time) time);


    public const override uint getEra(const(Time) time)
{
return HEBREW_ERA;
}


    public const override uint getDaysInMonth(uint year, uint month, uint era)
{
checkYear(year,era);
return MonthDays[getYearType(year)][month];
}


    public const override uint getDaysInYear(uint year, uint era)
{
return getStartOfYear(year + 1) - getStartOfYear(year);
}


    public const override uint getMonthsInYear(uint year, uint era)
{
return isLeapYear(year,era) ? 13 : 12;
}


    public const override bool isLeapYear(uint year, uint era)
{
checkYear(year,era);
return (7 * year + 1) % 19 < 7;
}


    public const override uint[] eras()
{
auto tmp = [HEBREW_ERA];
return tmp.dup;
}


    public const override uint id()
{
return HEBREW;
}


    private const void checkYear(uint year, uint era);


    private const uint getYearType(uint year);


    private const uint getStartOfYear(uint year)
{
auto months = (235 * year - 234) / 19;
auto fraction = months * DaysPerMonthFraction + FirstNewMoon;
auto day = months * 29 + fraction / PartsPerDay;
fraction %= PartsPerDay;
auto dayOfWeek = day % 7;
if (dayOfWeek == 2 || dayOfWeek == 4 || dayOfWeek == 6)
{
day++;
dayOfWeek = day % 7;
}
if (dayOfWeek == 1 && fraction > 15 * PartsPerHour + 204 && !isLeapYear(year,CURRENT_ERA))
day += 2;
else
if (dayOfWeek == 0 && fraction > 21 * PartsPerHour + 589 && isLeapYear(year,CURRENT_ERA))
day++;
return day;
}


    private const Time getGregorianTime(uint year, uint month, uint day, uint hour, uint minute, uint second, uint millisecond)
{
auto yearType = getYearType(year);
auto days = getStartOfYear(year) - DaysToOneAD + day - 1;
{
for (int i = 1;
 i <= month; i++)
{
days += MonthDays[yearType][i - 1];
}
}
return Time(days * TimeSpan.TicksPerDay + getTimeTicks(hour,minute,second)) + TimeSpan.fromMillis(millisecond);
}


}

