// D import file generated from 'tango/util/Convert.d'
module tango.util.Convert;
private import tango.core.Exception;

private import tango.core.Traits;

private import tango.core.Tuple;

private import tango.math.Math;

private import tango.text.convert.Utf;

private import tango.text.convert.Float;

private import tango.text.convert.Integer;

private import Ascii = tango.text.Ascii;

version (TangoDoc)
{
    template to(D,S)
{
D to(S value);
}
    template to(D,S)
{
D to(S value, D default_);
}
}
else
{
    template to(D,S)
{
D to(S value)
{
return toImpl!(D,S)(value);
}
}
    template to(D,S,Def)
{
D to(S value, Def def = Def.init)
{
try
{
return toImpl!(D,S)(value);
}
catch(ConversionException e)
{
}
return def;
}
}
}
class ConversionException : Exception
{
    this(immutable(char)[] msg)
{
super(msg);
}
}
private 
{
    char ctfe_upper(char c)
{
if ('a' <= c && c <= 'z')
return cast(char)(c - 'a' + 'A');
else
return c;
}
    char[] ctfe_camelCase(const(char[]) s);
    template ctfe_isSpace(T)
{
bool ctfe_isSpace(T c)
{
static if(T.sizeof is 1)
{
return c <= 32 && (c is ' ' || c is '\x09' || c is '\x0d' || c is '\x0a' || c is '\x0b' || c is '\x0c');
}
else
{
return c <= 32 && (c is ' ' || c is '\x09' || c is '\x0d' || c is '\x0a' || c is '\x0b' || c is '\x0c') || (c is '\U00002028' || c is '\U00002029');
}

}
}
    template ctfe_triml(T)
{
T[] ctfe_triml(T[] source)
{
if (source.length == 0)
return null;
foreach (i, c; source)
{
if (!ctfe_isSpace(c))
return source[i..$];
}
return null;
}
}
    template ctfe_trimr(T)
{
T[] ctfe_trimr(T[] source)
{
if (source.length == 0)
return null;
foreach_reverse (i, c; source)
{
if (!ctfe_isSpace(c))
return source[0..i + 1];
}
return null;
}
}
    template ctfe_trim(T)
{
T[] ctfe_trim(T[] source)
{
return ctfe_trimr(ctfe_triml(source));
}
}
    template isPOD(T)
{
static if(is(T == struct) || is(T == union))
{
    enum isPOD = true;
}
else
{
    enum isPOD = false;
}
}
    template isObject(T)
{
static if(is(T == class) || is(T == interface))
{
    enum isObject = true;
}
else
{
    enum isObject = false;
}
}
    template isUDT(T)
{
enum isUDT = isPOD!(T) || isObject!(T);
}
    template isString(T)
{
static if(is(typeof(T[]) : const(char[])) || is(typeof(T[]) : const(wchar[])) || is(typeof(T[]) : const(dchar[])))
{
    enum isString = true;
}
else
{
    enum isString = false;
}
}
    template isMutableString(T)
{
static if(is(typeof(T[]) == char[]) || is(typeof(T[]) == wchar[]) || is(typeof(T[]) == dchar[]))
{
    enum isMutableString = true;
}
else
{
    enum isMutableString = false;
}
}
    template isImmutableString(T)
{
static if(is(typeof(T[]) == immutable(char)[]) || is(typeof(T[]) == immutable(wchar)[]) || is(typeof(T[]) == immutable(dchar)[]))
{
    enum isImmutableString = true;
}
else
{
    enum isImmutableString = false;
}
}
    template isArrayType(T)
{
enum isArrayType = isDynamicArrayType!(T) || isStaticArrayType!(T);
}
    template sintSuperType(T,U)
{
static if(is(T == long) || is(U == long))
{
    alias long sintSuperType;
}
else
{
    static if(is(T == int) || is(U == int))
{
    alias int sintSuperType;
}
else
{
    static if(is(T == short) || is(U == short))
{
    alias short sintSuperType;
}
else
{
    static if(is(T == byte) || is(U == byte))
{
    alias byte sintSuperType;
}
}
}
}
}
    template uintSuperType(T,U)
{
static if(is(T == ulong) || is(U == ulong))
{
    alias ulong uintSuperType;
}
else
{
    static if(is(T == uint) || is(U == uint))
{
    alias uint uintSuperType;
}
else
{
    static if(is(T == ushort) || is(U == ushort))
{
    alias ushort uintSuperType;
}
else
{
    static if(is(T == ubyte) || is(U == ubyte))
{
    alias ubyte uintSuperType;
}
}
}
}
}
    template uintOfSize(uint bytes)
{
static if(bytes == 1)
{
    alias ubyte uintOfSize;
}
else
{
    static if(bytes == 2)
{
    alias ushort uintOfSize;
}
else
{
    static if(bytes == 4)
{
    alias uint uintOfSize;
}
}
}
}
    template intCmp(T,U)
{
int intCmp(T lhs, U rhs)
{
static if(isSignedIntegerType!(T) && isSignedIntegerType!(U))
{
alias sintSuperType!(T,U) S;
auto l = cast(S)lhs;
auto r = cast(S)rhs;
if (l < r)
return -1;
else
if (l > r)
return 1;
else
return 0;
}
else
{
static if(isUnsignedIntegerType!(T) && isUnsignedIntegerType!(U))
{
alias uintSuperType!(T,U) S;
auto l = cast(S)lhs;
auto r = cast(S)rhs;
if (l < r)
return -1;
else
if (l > r)
return 1;
else
return 0;
}
else
{
static if(isSignedIntegerType!(T))
{
if (lhs < 0)
return -1;
else
{
static if(U.sizeof >= T.sizeof)
{
auto l = cast(U)lhs;
if (l < rhs)
return -1;
else
if (l > rhs)
return 1;
else
return 0;
}
else
{
auto l = cast(ulong)lhs;
auto r = cast(ulong)rhs;
if (l < r)
return -1;
else
if (l > r)
return 1;
else
return 0;
}

}
}
else
{
static if(isSignedIntegerType!(U))
{
if (rhs < 0)
return 1;
else
{
static if(T.sizeof >= U.sizeof)
{
auto r = cast(T)rhs;
if (lhs < r)
return -1;
else
if (lhs > r)
return 1;
else
return 0;
}
else
{
auto l = cast(ulong)lhs;
auto r = cast(ulong)rhs;
if (l < r)
return -1;
else
if (l > r)
return 1;
else
return 0;
}

}
}

}

}

}

}
}
    template unsupported(immutable(char)[] desc = "")
{
static assert(false,"Unsupported conversion: cannot convert to " ~ ctfe_trim(D.stringof) ~ " from " ~ (desc != "" ? desc ~ " " : "") ~ ctfe_trim(S.stringof) ~ ".");
}
    template unsupported_backwards(immutable(char)[] desc = "")
{
static assert(false,"Unsupported conversion: cannot convert to " ~ (desc != "" ? desc ~ " " : "") ~ ctfe_trim(D.stringof) ~ " from " ~ ctfe_trim(S.stringof) ~ ".");
}
    template TN(T : const(T[]))
{
static if(is(T == char))
{
    enum TN = "string";
}
else
{
    static if(is(T == wchar))
{
    enum TN = "wstring";
}
else
{
    static if(is(T == dchar))
{
    enum TN = "dstring";
}
else
{
    enum TN = TN!(T) ~ "_array";
}
}
}
}
    template TN(T : T*)
{
enum TN = TN!(T) ~ "_pointer";
}
    template TN(T)
{
static if(isAssocArrayType!(T))
{
    enum TN = TN!(typeof(T.keys[0])) ~ "_to_" ~ TN!(typeof(T.values[0])) ~ "_map";
}
else
{
    enum TN = ctfe_trim(T.stringof);
}
}
    template convertString_(D,C)
{
D convertString_(C[] ret)
{
static if(isImmutableString!(C))
{
static if(isMutableString!(D))
{
return cast(D)ret.dup;
}
else
{
return cast(D)ret;
}

}
else
{
static if(isImmutableString!(D))
{
return cast(D)ret.idup;
}
else
{
return cast(D)ret;
}

}

}
}
    template toString_(T,C)
{
T toString_(C[] str)
{
static if(is(T : const(char[])))
{
return convertString_!(T)(tango.text.convert.Utf.toString(str));
}
else
{
static if(is(T : const(wchar[])))
{
return convertString_!(T)(tango.text.convert.Utf.toString16(str));
}
else
{
return convertString_!(T)(tango.text.convert.Utf.toString32(str));
}

}

}
}
    template UtfNum(T)
{
enum UtfNum = is(typeof(T[0]) : const(char)) ? "8" : is(typeof(T[0]) : const(wchar)) ? "16" : "32";
}
    template StringNum(T)
{
enum StringNum = is(typeof(T[0]) : const(char)) ? "" : is(typeof(T[0]) : const(wchar)) ? "16" : "32";
}
    template firstCharOf(T)
{
dchar firstCharOf(T s, out size_t used)
{
static if(is(T : const(char[])) || is(T : const(wchar[])))
{
return tango.text.convert.Utf.decode(s,used);
}
else
{
used = 1;
return s[0];
}

}
}
    template toUDT()
{
D toDfromS()
{
static if(isString!(S))
{
static if(is(typeof(mixin("D.fromUtf" ~ UtfNum!(S) ~ "(value)")) : D))
{
return mixin("D.fromUtf" ~ UtfNum!(S) ~ "(value)");
}
else
{
static if(is(typeof(D.fromUtf8(""c)) : D))
{
return D.fromUtf8(toString_!(char[])(value));
}
else
{
static if(is(typeof(D.fromUtf16(""w)) : D))
{
return D.fromUtf16(toString_!(wchar[])(value));
}
else
{
static if(is(typeof(D.fromUtf32(""d)) : D))
{
return D.fromUtf32(toString_!(dchar[])(value));
}
else
{
static if(is(typeof(D.fromString(""c)) : D))
{
static if(is(S == char[]))
{
return D.fromString(value);
}
else
{
return D.fromString(toString_!(char[])(value));
}

}
else
{
static if(is(typeof(D.from!(S)(value)) : D))
{
return D.from!(S)(value);
}
else
{
mixin unsupported!("user-defined type");
}

}

}

}

}

}

}
else
{
static if(is(typeof(mixin("D.from_" ~ TN!(S) ~ "()")) : D))
{
return mixin("D.from_" ~ TN!(S) ~ "()");
}
else
{
static if(is(typeof(mixin("D.from" ~ ctfe_camelCase(TN!(S)) ~ "()")) : D))
{
return mixin("D.from" ~ ctfe_camelCase(TN!(S)) ~ "()");
}
else
{
static if(is(typeof(D.from!(S)(value)) : D))
{
return D.from!(S)(value);
}
else
{
mixin unsupported!("user-defined type");
}

}

}

}

}
}
    template fromUDT(immutable(char)[] fallthrough = "")
{
D toDfromS()
{
static if(isString!(D))
{
static if(is(typeof(convertString_!(D)(mixin("value.toString" ~ StringNum!(D) ~ "()"))) : D))
{
return convertString_!(D)(mixin("value.toString" ~ StringNum!(D) ~ "()"));
}
else
{
static if(is(typeof(value.toString()) : const(char[])))
{
return toString_!(D)(value.toString());
}
else
{
static if(is(typeof(value.toString16()) : const(wchar[])))
{
return toString_!(D)(value.toString16);
}
else
{
static if(is(typeof(value.toString32()) : const(dchar[])))
{
return toString_!(D)(value.toString32);
}
else
{
static if(is(typeof(value.toString()) : const(char[])))
{
static if(is(D : const(char[])))
{
return value.toString();
}
else
{
return toString_!(D)(value.toString());
}

}
else
{
static if(is(typeof(value.to!(D)()) : D))
{
return value.to!(D)();
}
else
{
static if(fallthrough != "")
{
mixin(fallthrough);
}
else
{
mixin unsupported!("user-defined type");
}

}

}

}

}

}

}

}
else
{
static if(is(typeof(mixin("value.to_" ~ TN!(D) ~ "()")) : D))
{
return mixin("value.to_" ~ TN!(D) ~ "()");
}
else
{
static if(is(typeof(mixin("value.to" ~ ctfe_camelCase(TN!(D)) ~ "()")) : D))
{
return mixin("value.to" ~ ctfe_camelCase(TN!(D)) ~ "()");
}
else
{
static if(is(typeof(value.to!(D)()) : D))
{
return value.to!(D)();
}
else
{
static if(fallthrough != "")
{
mixin(fallthrough);
}
else
{
mixin unsupported!("user-defined type");
}

}

}

}

}

}
}
    template convError()
{
void throwConvError()
{
static if(isString!(D) || !is(typeof(to!(immutable(char)[])(value)) == immutable(char)[]))
{
throw new ConversionException("Could not convert a value of type " ~ S.stringof ~ " to type " ~ D.stringof ~ ".");
}
else
{
throw new ConversionException("Could not convert `" ~ to!(immutable(char)[])(value) ~ "` of type " ~ S.stringof ~ " to type " ~ D.stringof ~ ".");
}

}
}
    template toBool(D,S)
{
D toBool(S value)
{
static assert(is(D == bool));
static if(isIntegerType!(S))
{
return !(value == 0);
}
else
{
static if(isCharType!(S))
{
switch (value)
{
case 'F':
{
}
case 'f':
{
return false;
}
case 'T':
{
}
case 't':
{
return true;
}
default:
{
mixin convError!();
throwConvError();
assert(0);
}
}
}
else
{
static if(isString!(S))
{
if (value.length == 5 || value.length == 4)
{
char[5] buf;
buf[0..value.length] = value[];
switch (Ascii.toLower(buf[0..value.length]))
{
case "false":
{
return false;
}
case "true":
{
return true;
}
default:
{
}
}
}
mixin convError!();
throwConvError();
assert(0);
}
else
{
static if(isPOD!(S) || isObject!(S))
{
mixin fromUDT!();
return toDfromS();
}
else
{
mixin unsupported!();
}

}

}

}

}
}
    template toIntegerFromInteger(D,S)
{
D toIntegerFromInteger(S value)
{
static if(cast(ulong)D.max < cast(ulong)S.max || cast(long)D.min > cast(long)S.min)
{
mixin convError!();
if (intCmp(value,D.min) < 0 || intCmp(value,D.max) > 0)
{
throwConvError();
}
}

return cast(D)value;
}
}
    template toIntegerFromReal(D,S)
{
D toIntegerFromReal(S value)
{
auto v = tango.math.Math.round(value);
if (cast(real)D.min <= v && v <= cast(real)D.max)
{
return cast(D)v;
}
else
{
mixin convError!();
throwConvError();
assert(0);
}
}
}
    template toIntegerFromString(D,S)
{
D toIntegerFromString(S value)
{
static if(is(S charT : charT[]))
{
mixin convError!();
static if(is(D == ulong))
{
S s = value;
if (s.length == 0)
throwConvError();
else
if (s[0] == '-')
throwConvError();
else
if (s[0] == '+')
s = s[1..$];
size_t len;
auto result = tango.text.convert.Integer.convert(s,10,&len);
if (len < s.length || len == 0)
throwConvError();
return result;
}
else
{
size_t len;
auto result = tango.text.convert.Integer.parse(value,10,&len);
if (len < value.length || len == 0)
throwConvError();
return toIntegerFromInteger!(D,long)(result);
}

}

}
}
    template toInteger(D,S)
{
D toInteger(S value)
{
static if(is(S == bool))
{
return value ? 1 : 0;
}
else
{
static if(isIntegerType!(S))
{
return toIntegerFromInteger!(D,S)(value);
}
else
{
static if(isCharType!(S))
{
if (value >= '0' && value <= '9')
{
return cast(D)(value - '0');
}
else
{
mixin convError!();
throwConvError();
assert(0);
}
}
else
{
static if(isRealType!(S))
{
return toIntegerFromReal!(D,S)(value);
}
else
{
static if(isString!(S))
{
return toIntegerFromString!(D,S)(value);
}
else
{
static if(isPOD!(S) || isObject!(S))
{
mixin fromUDT!();
return toDfromS();
}
else
{
mixin unsupported!();
}

}

}

}

}

}

}
}
    template toReal(D,S)
{
D toReal(S value)
{
static if(isIntegerType!(S) || isRealType!(S))
{
return cast(D)value;
}
else
{
static if(isString!(S))
{
mixin convError!();
size_t len;
auto r = tango.text.convert.Float.parse(value,&len);
if (len < value.length || len == 0)
throwConvError();
return r;
}
else
{
static if(isPOD!(S) || isObject!(S))
{
mixin fromUDT!();
return toDfromS();
}
else
{
mixin unsupported!();
}

}

}

}
}
    template toImaginary(D,S)
{
D toImaginary(S value)
{
static if(isComplexType!(S))
{
if (value.re == 0)
return value.im * cast(D)1i;
else
{
mixin convError!();
throwConvError();
assert(0);
}
}
else
{
static if(isPOD!(S) || isObject!(S))
{
mixin fromUDT!();
return toDfromS();
}
else
{
mixin unsupported!();
}

}

}
}
    template toComplex(D,S)
{
D toComplex(S value)
{
static if(isIntegerType!(S) || isRealType!(S) || isImaginaryType!(S) || isComplexType!(S))
{
return cast(D)value;
}
else
{
static if(isPOD!(S) || isObject!(S))
{
mixin fromUDT!();
return toDfromS();
}
else
{
mixin unsupported!();
}

}

}
}
    template toChar(D,S)
{
D toChar(S value)
{
static if(is(S == bool))
{
return value ? 't' : 'f';
}
else
{
static if(isIntegerType!(S))
{
if (value >= 0 && value <= 9)
return cast(D)(value + '0');
else
{
mixin convError!();
throwConvError();
assert(0);
}
}
else
{
static if(isString!(S))
{
void fail()
{
mixin convError!();
throwConvError();
}
if (value.length == 0)
{
fail();
assert(0);
}
else
{
size_t used;
dchar c = firstCharOf(value,used);
if (used < value.length)
{
fail();
assert(0);
}
if (cast(size_t)c > cast(size_t)D.max)
{
fail();
assert(0);
}
return cast(D)c;
}
}
else
{
static if(isPOD!(S) || isObject!(S))
{
mixin fromUDT!();
return toDfromS();
}
else
{
mixin unsupported!();
}

}

}

}

}
}
    template toStringFromString(D,S)
{
D toStringFromString(S value)
{
return toString_!(D)(value);
}
}
    enum immutable(char)[] CHARS = " !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~";
    template toStringFromChar(D,S)
{
D toStringFromChar(S value)
{
static if(is(D : S[]))
{
static if(is(S == char))
{
if (32 <= value && value <= 126)
{
return convertString_!(D)((&CHARS[value - 32])[0..1]);
}
}

auto r = new S[](1);
r[0] = value;
return convertString_!(D)(r);
}
else
{
S[1] temp;
temp[0] = value;
return toStringFromString!(D,S[])(temp);
}

}
}
    template toString(D,S)
{
D toString(S value)
{
static if(is(S == bool))
{
return convertString_!(D)(value ? "true" : "false");
}
else
{
static if(isCharType!(S))
{
return toStringFromChar!(D,S)(value);
}
else
{
static if(isIntegerType!(S))
{
return convertString_!(D)(mixin("tango.text.convert.Integer.toString" ~ StringNum!(D) ~ "(value)"));
}
else
{
static if(isRealType!(S))
{
return convertString_!(D)(mixin("tango.text.convert.Float.toString" ~ StringNum!(D) ~ "(value)"));
}
else
{
static if(isDynamicArrayType!(S) || isStaticArrayType!(S))
{
mixin unsupported!("array type");
}
else
{
static if(isAssocArrayType!(S))
{
mixin unsupported!("associative array type");
}
else
{
static if(isPOD!(S) || isObject!(S))
{
mixin fromUDT!();
return toDfromS();
}
else
{
mixin unsupported!();
}

}

}

}

}

}

}

}
}
    template fromString(D,S)
{
D fromString(D value)
{
static if(isDynamicArrayType!(S) || isStaticArrayType!(S))
{
mixin unsupported_backwards!("array type");
}
else
{
static if(isAssocArrayType!(S))
{
mixin unsupported_backwards!("associative array type");
}
else
{
static if(isPOD!(S) || isObject!(S))
{
mixin toUDT!();
return toDfromS();
}
else
{
mixin unsupported_backwards!();
}

}

}

}
}
    template toArrayFromArray(D,S)
{
D toArrayFromArray(S value)
{
alias BaseTypeOf!(typeof(D[0])) De;
De[] result;
result.length = value.length;
scope(failure) delete result;
foreach (i, e; value)
{
result[i] = to!(De)(e);
}
return cast(D)result;
}
}
    template toMapFromMap(D,S)
{
D toMapFromMap(S value)
{
alias typeof(D.keys[0]) Dk;
alias typeof(D.values[0]) Dv;
D result;
foreach (k, v; value)
{
result[to!(Dk)(k)] = to!(Dv)(v);
}
return result;
}
}
    template toFromUDT(D,S)
{
D toFromUDT(S value)
{
static if(is(typeof(mixin("value.to_" ~ TN!(D) ~ "()")) : D))
{
return mixin("value.to_" ~ TN!(D) ~ "()");
}
else
{
static if(is(typeof(mixin("value.to" ~ ctfe_camelCase(TN!(D)) ~ "()")) : D))
{
return mixin("value.to" ~ ctfe_camelCase(TN!(D)) ~ "()");
}
else
{
static if(is(typeof(value.to!(D)()) : D))
{
return value.to!(D)();
}
else
{
static if(is(typeof(mixin("D.from_" ~ TN!(S) ~ "(value)")) : D))
{
return mixin("D.from_" ~ TN!(S) ~ "(value)");
}
else
{
static if(is(typeof(mixin("D.from" ~ ctfe_camelCase(TN!(S)) ~ "(value)")) : D))
{
return mixin("D.from" ~ ctfe_camelCase(TN!(S)) ~ "(value)");
}
else
{
static if(is(typeof(D.from!(S)(value)) : D))
{
return D.from!(S)(value);
}
else
{
mixin unsupported!();
}

}

}

}

}

}

}
}
    template toImpl(D,S)
{
D toImpl(S value)
{
static if(is(D == S))
{
return value;
}
else
{
static if(is(S BaseType == enum))
{
return toImpl!(D,BaseType)(value);
}
else
{
static if(isArrayType!(D) && isArrayType!(S) && is(typeof(D[0]) == typeof(S[0])))
{
return value;
}
else
{
static if(is(D == bool))
{
return toBool!(D,S)(value);
}
else
{
static if(isIntegerType!(D))
{
return toInteger!(D,S)(value);
}
else
{
static if(isRealType!(D))
{
return toReal!(D,S)(value);
}
else
{
static if(isImaginaryType!(D))
{
return toImaginary!(D,S)(value);
}
else
{
static if(isComplexType!(D))
{
return toComplex!(D,S)(value);
}
else
{
static if(isCharType!(D))
{
return toChar!(D,S)(value);
}
else
{
static if(isString!(D) && isString!(S))
{
return toStringFromString!(D,S)(value);
}
else
{
static if(isString!(D))
{
return toString!(D,S)(value);
}
else
{
static if(isString!(S))
{
return fromString!(D,S)(value);
}
else
{
static if(isArrayType!(D) && isArrayType!(S))
{
return toArrayFromArray!(D,S)(value);
}
else
{
static if(isAssocArrayType!(D) && isAssocArrayType!(S))
{
return toMapFromMap!(D,S)(value);
}
else
{
static if(isUDT!(D) || isUDT!(S))
{
return toFromUDT!(D,S)(value);
}
else
{
mixin unsupported!();
}

}

}

}

}

}

}

}

}

}

}

}

}

}

}

}
}
    debug (ConvertTest)
{
    void main()
{
}
}
    debug (UnitTest)
{
    template ex(T)
{
bool ex(lazy T v)
{
bool result = false;
try
{
v();
}
catch(ConversionException _)
{
result = true;
}
return result;
}
}
    template nx(T)
{
bool nx(lazy T v)
{
bool result = true;
try
{
v();
}
catch(ConversionException _)
{
result = false;
}
return result;
}
}
    struct Foo
{
    int toInt()
{
return 42;
}
    const(char[]) toString()
{
return "string foo";
}
    int[] toIntArray()
{
return [1,2,3];
}
    Bar toBar()
{
Bar result;
return result;
}
    template to(T)
{
T to()
{
static if(is(T == bool))
{
return true;
}
else
{
static assert(false);
}

}
}
}
    struct Bar
{
    real toReal()
{
return 3.14159;
}
    ireal toIreal()
{
return 42i;
}
}
    struct Baz
{
    static Baz fromFoo(Foo foo)
{
Baz result;
return result;
}

    Bar toBar()
{
Bar result;
return result;
}
}
    }
}
