// D import file generated from 'tango/util/cipher/Salsa20.d'
module tango.util.cipher.Salsa20;
private import tango.util.cipher.Cipher;

class Salsa20 : StreamCipher
{
    protected 
{
    enum immutable(ubyte)[] sigma = cast(immutable(ubyte)[])"expand 32-byte k";
    enum immutable(ubyte)[] tau = cast(immutable(ubyte)[])"expand 16-byte k";
    uint i0;
    uint i1;
    uint[] state;
    ubyte[] keyStream;
    uint index;
    const(ubyte)[] workingKey;
    const(ubyte)[] workingIV;
}
    this()
{
state = new uint[](16);
keyStream = new ubyte[](64);
i0 = 8;
i1 = 9;
}
    this(bool encrypt, ubyte[] key, ubyte[] iv)
{
this();
init(encrypt,key,iv);
}
    void init(bool encrypt, ubyte[] key, ubyte[] iv)
{
if (key)
{
if (key.length != 16 && key.length != 32)
invalid(name() ~ ": Invalid key length. (requires 16 or 32 bytes)");
workingKey = key;
keySetup();
index = 0;
}
if (!workingKey)
invalid(name() ~ ": Key not set.");
if (!iv || iv.length != 8)
invalid(name() ~ ": 8 byte IV required.");
workingIV = iv;
ivSetup();
_encrypt = (_initialized = true);
}
    override @property const(char[]) name()
{
return "Salsa20";
}

    override ubyte returnByte(ubyte input)
{
if (!_initialized)
invalid(name() ~ ": Cipher not initialized");
if (index == 0)
{
salsa20WordToByte(state,keyStream);
state[i0]++;
if (!state[i0])
state[i1]++;
}
ubyte result = keyStream[index] ^ input;
index = index + 1 & 63;
return result;
}

    override uint update(const(void[]) input_, void[] output_)
{
if (!_initialized)
invalid(name() ~ ": Cipher not initialized");
const(ubyte[]) input = cast(const(ubyte[]))input_;
ubyte[] output = cast(ubyte[])output_;
if (input.length > output.length)
invalid(name() ~ ": Output buffer too short");
{
for (int i = 0;
 i < input.length; i++)
{
{
if (index == 0)
{
salsa20WordToByte(state,keyStream);
state[i0]++;
if (!state[i0])
state[i1]++;
}
output[i] = keyStream[index] ^ input[i];
index = index + 1 & 63;
}
}
}
return cast(uint)input.length;
}

    override void reset()
{
keySetup();
ivSetup();
index = 0;
}

    protected void keySetup()
{
uint offset;
const(ubyte)[] constants;
state[1] = ByteConverter.LittleEndian.to!(uint)(workingKey[0..4]);
state[2] = ByteConverter.LittleEndian.to!(uint)(workingKey[4..8]);
state[3] = ByteConverter.LittleEndian.to!(uint)(workingKey[8..12]);
state[4] = ByteConverter.LittleEndian.to!(uint)(workingKey[12..16]);
if (workingKey.length == 32)
{
constants = sigma;
offset = 16;
}
else
constants = tau;
state[11] = ByteConverter.LittleEndian.to!(uint)(workingKey[offset..offset + 4]);
state[12] = ByteConverter.LittleEndian.to!(uint)(workingKey[offset + 4..offset + 8]);
state[13] = ByteConverter.LittleEndian.to!(uint)(workingKey[offset + 8..offset + 12]);
state[14] = ByteConverter.LittleEndian.to!(uint)(workingKey[offset + 12..offset + 16]);
state[0] = ByteConverter.LittleEndian.to!(uint)(constants[0..4]);
state[5] = ByteConverter.LittleEndian.to!(uint)(constants[4..8]);
state[10] = ByteConverter.LittleEndian.to!(uint)(constants[8..12]);
state[15] = ByteConverter.LittleEndian.to!(uint)(constants[12..16]);
}

    protected void ivSetup()
{
state[6] = ByteConverter.LittleEndian.to!(uint)(workingIV[0..4]);
state[7] = ByteConverter.LittleEndian.to!(uint)(workingIV[4..8]);
state[8] = (state[9] = 0);
}

    protected void salsa20WordToByte(const(uint[]) input, ref ubyte[] output);

    debug (UnitTest)
{
    }
}
