// D import file generated from 'tango/util/cipher/TEA.d'
module tango.util.cipher.TEA;
private import tango.util.cipher.Cipher;

class TEA : BlockCipher
{
    private 
{
    enum uint ROUNDS = 32;
    enum uint KEY_SIZE = 16;
    enum uint BLOCK_SIZE = 8;
    enum uint DELTA = -1640531527u;
    enum uint DECRYPT_SUM = -957401312u;
    uint sk0;
    uint sk1;
    uint sk2;
    uint sk3;
    uint sum;
}
    this()
{
}
    this(bool encrypt, ubyte[] key)
{
this();
init(encrypt,key);
}
    final override void reset()
{
}

    final override @property const(char)[] name()
{
return "TEA";
}

    const final override @property uint blockSize()
{
return BLOCK_SIZE;
}

    final void init(bool encrypt, ubyte[] key)
{
_encrypt = encrypt;
if (key.length != KEY_SIZE)
invalid(name() ~ ": Invalid key length (requires 16 bytes)");
sk0 = ByteConverter.BigEndian.to!(uint)(key[0..4]);
sk1 = ByteConverter.BigEndian.to!(uint)(key[4..8]);
sk2 = ByteConverter.BigEndian.to!(uint)(key[8..12]);
sk3 = ByteConverter.BigEndian.to!(uint)(key[12..16]);
_initialized = true;
}

    final override uint update(const(void[]) input_, void[] output_)
{
if (!_initialized)
invalid(name() ~ ": Cipher not initialized");
const(ubyte[]) input = cast(const(ubyte[]))input_;
ubyte[] output = cast(ubyte[])output_;
if (input.length < BLOCK_SIZE)
invalid(name() ~ ": Input buffer too short");
if (output.length < BLOCK_SIZE)
invalid(name() ~ ": Output buffer too short");
uint v0 = ByteConverter.BigEndian.to!(uint)(input[0..4]),v1 = ByteConverter.BigEndian.to!(uint)(input[4..8]);
sum = _encrypt ? 0 : DECRYPT_SUM;
{
for (int i = 0;
 i < ROUNDS; i++)
{
{
if (_encrypt)
{
sum += DELTA;
v0 += (v1 << 4) + sk0 ^ v1 + sum ^ (v1 >> 5) + sk1;
v1 += (v0 << 4) + sk2 ^ v0 + sum ^ (v0 >> 5) + sk3;
}
else
{
v1 -= (v0 << 4) + sk2 ^ v0 + sum ^ (v0 >> 5) + sk3;
v0 -= (v1 << 4) + sk0 ^ v1 + sum ^ (v1 >> 5) + sk1;
sum -= DELTA;
}
}
}
}
ByteConverter.BigEndian.from!(uint)(v0,output[0..4]);
ByteConverter.BigEndian.from!(uint)(v1,output[4..8]);
return BLOCK_SIZE;
}

    debug (UnitTest)
{
    }
}
