// D import file generated from 'tango/util/cipher/XTEA.d'
module tango.util.cipher.XTEA;
private import tango.util.cipher.Cipher;

class XTEA : BlockCipher
{
    private 
{
    enum uint ROUNDS = 32;
    enum uint KEY_SIZE = 16;
    enum uint BLOCK_SIZE = 8;
    enum uint DELTA = -1640531527u;
    uint[] subkeys;
    uint[] sum0;
    uint[] sum1;
}
    this()
{
}
    this(bool encrypt, ubyte[] key)
{
this();
init(encrypt,key);
}
    final override void reset()
{
}

    final override @property const(char)[] name()
{
return "XTEA";
}

    const final override @property uint blockSize()
{
return BLOCK_SIZE;
}

    final void init(bool encrypt, ubyte[] key)
{
_encrypt = encrypt;
if (key.length != KEY_SIZE)
invalid(name() ~ ": Invalid key length (requires 16 bytes)");
subkeys = new uint[](4);
sum0 = new uint[](32);
sum1 = new uint[](32);
int i,j;
{
for (i = (j = 0); i < 4; i++ , j += (int).sizeof)
{
subkeys[i] = ByteConverter.BigEndian.to!(uint)(key[j..j + (int).sizeof]);
}
}
{
for (i = (j = 0); i < ROUNDS; i++)
{
{
sum0[i] = j + subkeys[j & 3];
j += DELTA;
sum1[i] = j + subkeys[j >> 11 & 3];
}
}
}
_initialized = true;
}

    final override uint update(const(void[]) input_, void[] output_)
{
if (!_initialized)
invalid(name() ~ ": Cipher not initialized");
const(ubyte[]) input = cast(const(ubyte[]))input_;
ubyte[] output = cast(ubyte[])output_;
if (input.length < BLOCK_SIZE)
invalid(name() ~ ": Input buffer too short");
if (output.length < BLOCK_SIZE)
invalid(name() ~ ": Output buffer too short");
uint v0 = ByteConverter.BigEndian.to!(uint)(input[0..4]),v1 = ByteConverter.BigEndian.to!(uint)(input[4..8]);
if (_encrypt)
{
{
for (int i = 0;
 i < ROUNDS; i++)
{
{
v0 += (v1 << 4 ^ v1 >> 5) + v1 ^ sum0[i];
v1 += (v0 << 4 ^ v0 >> 5) + v0 ^ sum1[i];
}
}
}
}
else
{
{
for (int i = ROUNDS - 1;
 i >= 0; i--)
{
{
v1 -= (v0 << 4 ^ v0 >> 5) + v0 ^ sum1[i];
v0 -= (v1 << 4 ^ v1 >> 5) + v1 ^ sum0[i];
}
}
}
}
ByteConverter.BigEndian.from!(uint)(v0,output[0..4]);
ByteConverter.BigEndian.from!(uint)(v1,output[4..8]);
return BLOCK_SIZE;
}

    debug (UnitTest)
{
    }
}
