// D import file generated from 'tango/util/container/CircularList.d'
module tango.util.container.CircularList;
private import tango.util.container.Clink;

public import tango.util.container.Container;

private import tango.util.container.model.IContainer;

template CircularList(V,alias Reap = Container.reap,alias Heap = Container.DefaultCollect)
{
class CircularList : IContainer!(V)
{
    public alias Clink!(V) Type;

    private alias Type* Ref;

    private alias Heap!(Type) Alloc;

    private size_t count;

    private Alloc heap;

    private size_t mutation;

    private Ref list;

    this()
{
this(null,0);
}
    protected this(Ref h, size_t c)
{
list = h;
count = c;
}

    ~this()
{
reset();
}
    final @property Iterator iterator()
{
Iterator i;
i.owner = this;
i.mutation = mutation;
i.cell = (i.head = list);
i.count = count;
i.index = 0;
return i;
}

    final CircularList cache(size_t chunk, size_t count = 0)
{
heap.config(chunk,count);
return this;
}

    final int opApply(scope int delegate(ref V value) dg)
{
return iterator.opApply(dg);
}

    const final @property size_t size()
{
return count;
}

    final @property CircularList dup()
{
return new CircularList!(V,Reap,Heap)(list ? list.copyList(&heap.allocate) : null,count);
}

    final bool contains(V element)
{
if (list)
return list.find(element) !is null;
return false;
}

    final V head()
{
return firstCell().value;
}

    final V tail()
{
return lastCell().value;
}

    final V get(size_t index)
{
return cellAt(index).value;
}

    final size_t first(V element, size_t startingIndex = 0)
{
if (startingIndex < 0)
startingIndex = 0;
auto p = list;
if (p is null)
return size_t.max;
{
for (size_t i = 0;
 true; ++i)
{
{
if (i >= startingIndex && element == p.value)
return i;
p = p.next;
if (p is list)
break;
}
}
}
return size_t.max;
}

    final size_t last(V element, size_t startingIndex = 0)
{
if (count is 0)
return size_t.max;
if (startingIndex >= count)
startingIndex = count - 1;
if (startingIndex < 0)
startingIndex = 0;
auto p = cellAt(startingIndex);
size_t i = startingIndex;
for (;;)
{
{
if (element == p.value)
return i;
else
if (p is list)
break;
else
{
p = p.prev;
--i;
}
}
}
return size_t.max;
}

    final CircularList subset(size_t from, size_t length)
{
Ref newlist = null;
if (length > 0)
{
checkIndex(from);
auto p = cellAt(from);
auto current = newlist = heap.allocate().set(p.value);
{
for (size_t i = 1;
 i < length; ++i)
{
{
p = p.next;
if (p is null)
length = i;
else
{
current.addNext(p.value,&heap.allocate);
current = current.next;
}
}
}
}
}
return new CircularList(newlist,length);
}

    final CircularList clear()
{
return clear(false);
}

    final CircularList reset()
{
return clear(true);
}

    final bool take(ref V v)
{
if (count)
{
v = tail();
removeTail();
return true;
}
return false;
}

    final CircularList prepend(V element)
{
if (list is null)
list = heap.allocate().set(element);
else
list = list.addPrev(element,&heap.allocate);
increment();
return this;
}

    final V head(V element)
{
auto p = firstCell();
auto v = p.value;
p.value = element;
mutate();
return v;
}

    final V removeHead()
{
auto p = firstCell();
if (p.singleton)
list = null;
else
{
auto n = p.next;
p.unlink();
list = n;
}
auto v = p.value;
decrement(p);
return v;
}

    final CircularList add(V element)
{
return append(element);
}

    final CircularList append(V element)
{
if (list is null)
prepend(element);
else
{
list.prev.addNext(element,&heap.allocate);
increment();
}
return this;
}

    final V tail(V element)
{
auto p = lastCell();
auto v = p.value;
p.value = element;
mutate();
return v;
}

    final V removeTail()
{
auto p = lastCell();
if (p is list)
list = null;
else
p.unlink();
auto v = p.value;
decrement(p);
return v;
}

    final CircularList addAt(size_t index, V element)
{
if (index is 0)
prepend(element);
else
{
cellAt(index - 1).addNext(element,&heap.allocate);
increment();
}
return this;
}

    final CircularList replaceAt(size_t index, V element)
{
cellAt(index).value = element;
mutate();
return this;
}

    final CircularList removeAt(size_t index)
{
if (index is 0)
removeHead();
else
{
auto p = cellAt(index);
p.unlink();
decrement(p);
}
return this;
}

    final size_t remove(V element, bool all)
{
auto c = count;
if (list)
{
auto p = list;
for (;;)
{
{
auto n = p.next;
if (element == p.value)
{
p.unlink();
decrement(p);
if (p is list)
{
if (p is n)
{
list = null;
break;
}
else
list = n;
}
if (!all)
break;
else
p = n;
}
else
if (n is list)
break;
else
p = n;
}
}
}
return c - count;
}

    final size_t replace(V oldElement, V newElement, bool all)
{
size_t c;
if (list)
{
auto p = list;
do
{
if (oldElement == p.value)
{
++c;
mutate();
p.value = newElement;
if (!all)
break;
}
p = p.next;
}
while (p !is list);}
return c;
}

    final size_t prepend(IContainer!(V) e)
{
Ref hd = null;
Ref current = null;
auto c = count;
foreach (element; e)
{
increment();
if (hd is null)
{
hd = heap.allocate().set(element);
current = hd;
}
else
{
current.addNext(element,&heap.allocate);
current = current.next;
}
}
if (list is null)
list = hd;
else
if (hd)
{
auto tl = list.prev;
current.next = list;
list.prev = current;
tl.next = hd;
hd.prev = tl;
list = hd;
}
return count - c;
}

    final size_t append(IContainer!(V) e)
{
auto c = count;
if (list is null)
prepend(e);
else
{
auto current = list.prev;
foreach (element; e)
{
increment();
current.addNext(element,&heap.allocate);
current = current.next;
}
}
return count - c;
}

    final size_t addAt(size_t index, IContainer!(V) e)
{
auto c = count;
if (list is null || index is 0)
prepend(e);
else
{
auto current = cellAt(index - 1);
foreach (element; e)
{
increment();
current.addNext(element,&heap.allocate);
current = current.next;
}
}
return count - c;
}

    final size_t removeRange(size_t fromIndex, size_t toIndex)
{
auto p = cellAt(fromIndex);
auto last = list.prev;
auto c = count;
{
for (size_t i = fromIndex;
 i <= toIndex; ++i)
{
{
auto n = p.next;
p.unlink();
decrement(p);
if (p is list)
{
if (p is last)
{
list = null;
break;
}
else
list = n;
}
p = n;
}
}
}
return c - count;
}

    final V[] toArray(V[] dst = null)
{
if (dst.length < count)
dst.length = count;
size_t i = 0;
foreach (v; this)
{
dst[i++] = v;
}
return dst[0..count];
}

    const final bool isEmpty()
{
return count is 0;
}

    final CircularList check()
{
assert((count is 0) is (list is null));
assert(list is null || list.size is count);
if (list)
{
size_t c = 0;
auto p = list;
do
{
assert(p.prev.next is p);
assert(p.next.prev is p);
assert(instances(p.value) > 0);
assert(contains(p.value));
p = p.next;
++c;
}
while (p !is list);assert(c is size);
}
return this;
}

    private size_t instances(V element)
{
if (list)
return list.count(element);
return 0;
}

    private void checkIndex(size_t i)
{
if (i >= count)
throw new Exception("out of range");
}

    private Ref firstCell()
{
checkIndex(0);
return list;
}

    private Ref lastCell()
{
checkIndex(0);
return list.prev;
}

    private Ref cellAt(size_t index)
{
checkIndex(index);
return list.nth(index);
}

    private CircularList clear(bool all)
{
mutate();
if (heap.collect(all) is false && count)
{
auto p = list;
do
{
auto n = p.next;
decrement(p);
p = n;
}
while (p != list);}
list = null;
count = 0;
return this;
}

    private void increment()
{
++mutation;
++count;
}

    private void decrement(Ref p)
{
Reap(p.value);
heap.collect(p);
++mutation;
--count;
}

    private void mutate()
{
++mutation;
}

    private struct Iterator
{
    bool rev;
    Ref cell;
    Ref head;
    Ref prior;
    CircularList owner;
    size_t index;
    size_t count;
    size_t mutation;
    bool valid()
{
return owner.mutation is mutation;
}
    @property bool next(ref V v)
{
auto n = next;
return n ? v = *n , true : false;
}

    @property V* next()
{
V* r;
if (index < count)
{
++index;
prior = cell;
r = &cell.value;
cell = rev ? cell.prev : cell.next;
}
else
cell = null;
return r;
}

    int opApply(scope int delegate(ref V value) dg)
{
int result;
auto c = cell;
while (index < count)
{
++index;
prior = c;
c = rev ? c.prev : c.next;
if ((result = dg(prior.value)) != 0)
break;
}
cell = null;
return result;
}
    bool remove()
{
if (prior)
{
auto next = rev ? prior.prev : prior.next;
if (prior is head)
{
if (prior is next)
owner.list = null;
else
head = (owner.list = next);
}
prior.unlink();
owner.decrement(prior);
prior = null;
--count;
++mutation;
return true;
}
return false;
}
    Iterator insert(V value)
{
if (cell is null)
prior.addNext(value,&owner.heap.allocate);
else
cell.addPrev(value,&owner.heap.allocate);
owner.increment();
++count;
++mutation;
return this;
}
    Iterator reverse()
{
rev ^= true;
next;
index = 0;
return this;
}
}

}
}
debug (UnitTest)
{
    }
debug (CircularList)
{
    import tango.io.Stdout;
    import tango.core.Thread;
    import tango.time.StopWatch;
    void main();
}
