// D import file generated from 'tango/util/container/Container.d'
module tango.util.container.Container;
private import tango.core.Memory;

private import tango.stdc.stdlib;

private import tango.stdc.string;

struct Container
{
    enum size_t defaultInitialBuckets = 31;
    enum float defaultLoadFactor = 0.75F;
    static template reap(V)
{
void reap(V v)
{
}
}

    static template reap(K,V)
{
void reap(K k, V v)
{
}
}

    static template hash(K)
{
size_t hash(K k, size_t length)
{
static if(is(K : int) || is(K : uint) || is(K : long) || is(K : ulong) || is(K : short) || is(K : ushort) || is(K : byte) || is(K : ubyte) || is(K : char) || is(K : wchar) || is(K : dchar))
{
return cast(size_t)(k % length);
}
else
{
return (typeid(K).getHash(&k) & 2147483647) % length;
}

}
}

    template ChunkGC(T)
{
struct ChunkGC
{
    static assert(T.sizeof >= (T*).sizeof,"The ChunkGC allocator can only be used for data sizes of at least " ~ (T*).sizeof.stringof[0..$ - 1] ~ " bytes!");
    private struct Cache
{
    Cache* next;
}

    private Cache* cache;

    private T[][] lists;

    private size_t chunks = 256;

    T* allocate()
{
if (cache is null)
newlist();
auto p = cache;
cache = p.next;
return cast(T*)p;
}
    T*[] allocate(size_t count)
{
auto p = (cast(T**)calloc(count,(T*).sizeof))[0..count];
GC.addRange(cast(void*)p,count * (T*).sizeof);
return p;
}
    void collect(T*[] t)
{
if (t.ptr)
{
GC.removeRange(t.ptr);
free(t.ptr);
}
}
    void collect(T* p)
{
assert(p);
auto d = cast(Cache*)p;
d.next = cache;
cache = d;
}
    bool collect(bool all = true)
{
if (all)
{
foreach (ref list; lists)
{
GC.removeRange(list.ptr);
free(list.ptr);
list = null;
}
cache = null;
lists = null;
return true;
}
return false;
}
    void config(size_t chunks, size_t allocate = 0)
{
this.chunks = chunks;
if (allocate)
{
for (size_t i = allocate / chunks + 1;
 i--;)
{
newlist();
}
}
}
    private void newlist()
{
lists.length = lists.length + 1;
auto p = (cast(T*)calloc(chunks,T.sizeof))[0..chunks];
lists[$ - 1] = p;
GC.addRange(p.ptr,T.sizeof * chunks);
auto head = cache;
foreach (ref node; p)
{
auto d = cast(Cache*)&node;
d.next = head;
head = d;
}
cache = head;
}

}
}
    template Chunk(T)
{
struct Chunk
{
    static assert(T.sizeof >= (T*).sizeof,"The Chunk allocator can only be used for data sizes of at least " ~ (T*).sizeof.stringof[0..$ - 1] ~ " bytes!");
    private struct Cache
{
    Cache* next;
}

    private Cache* cache;

    private T[][] lists;

    private size_t chunks = 256;

    T* allocate()
{
if (cache is null)
newlist();
auto p = cache;
cache = p.next;
return cast(T*)p;
}
    T*[] allocate(size_t count)
{
return (cast(T**)calloc(count,(T*).sizeof))[0..count];
}
    void collect(T*[] t)
{
if (t.ptr)
free(t.ptr);
}
    void collect(T* p)
{
assert(p);
auto d = cast(Cache*)p;
d.next = cache;
cache = d;
}
    bool collect(bool all = true)
{
if (all)
{
foreach (ref list; lists)
{
free(list.ptr);
list = null;
}
cache = null;
lists = null;
return true;
}
return false;
}
    void config(size_t chunks, int allocate = 0)
{
this.chunks = chunks;
if (allocate)
{
for (int i = allocate / chunks + 1;
 i--;)
{
newlist();
}
}
}
    private void newlist()
{
lists.length = lists.length + 1;
auto p = (cast(T*)calloc(chunks,T.sizeof))[0..chunks];
lists[$ - 1] = p;
auto head = cache;
foreach (ref node; p)
{
auto d = cast(Cache*)&node;
d.next = head;
head = d;
}
cache = head;
}

}
}
    template Collect(T)
{
struct Collect
{
    T* allocate()
{
return cast(T*)GC.calloc(T.sizeof);
}
    T*[] allocate(size_t count)
{
return new T*[](count);
}
    void collect(T* p)
{
if (p)
delete p;
}
    void collect(T*[] t)
{
if (t)
delete t;
}
    bool collect(bool all = true)
{
return false;
}
    void config(size_t chunks, int allocate = 0)
{
}
}
}
    template Malloc(T)
{
struct Malloc
{
    T* allocate()
{
return cast(T*)calloc(1,T.sizeof);
}
    T*[] allocate(size_t count)
{
return (cast(T**)calloc(count,(T*).sizeof))[0..count];
}
    void collect(T*[] t)
{
if (t.length)
free(t.ptr);
}
    void collect(T* p)
{
if (p)
free(p);
}
    bool collect(bool all = true)
{
return false;
}
    void config(size_t chunks, int allocate = 0)
{
}
}
}
    version (prior_allocator)
{
    template GCChunk(T,uint chunkSize)
{
struct GCChunk
{
    static if(T.sizeof < (void*).sizeof)
{
    static assert(false,"Error, allocator for " ~ T.stringof ~ " failed to instantiate");
}
    struct element
{
    element* next;
}
    struct chunk
{
    chunk* next;
    chunk* prev;
    element* freeList;
    uint numFree;
    T[chunkSize] elems;
    T* allocateFromFree()
{
element* x = freeList;
freeList = x.next;
x.next = null;
numFree--;
return cast(T*)x;
}
    bool deallocate(T* t)
{
memset(t,0,(T).sizeof);
element* x = cast(element*)t;
x.next = freeList;
freeList = x;
return ++numFree == chunkSize;
}
}
    chunk* used;
    chunk* fresh;
    uint nextFresh;
    T* allocate()
{
if (used !is null && used.numFree > 0)
{
T* result = used.allocateFromFree();
if (used.numFree == 0)
used = used.next;
return result;
}
if (fresh is null)
{
fresh = new chunk;
nextFresh = 0;
}
T* result = &fresh.elems[nextFresh];
if (++nextFresh == chunkSize)
{
if (used is null)
{
used = fresh;
fresh.next = fresh;
fresh.prev = fresh;
}
else
{
fresh.prev = used.prev;
fresh.next = used;
fresh.prev.next = fresh;
fresh.next.prev = fresh;
if (fresh.numFree != 0)
{
used = fresh;
}
}
fresh = null;
}
return result;
}
    T*[] allocate(uint count)
{
return new T*[](count);
}
    void collect(T* t)
{
chunk* cur = cast(chunk*)GC.addrOf(t);
if (cur !is fresh && cur.numFree == 0)
{
if (cur !is used)
{
if (used.numFree != 0)
{
cur.prev.next = cur.next;
cur.next.prev = cur.prev;
cur.prev = used.prev;
cur.next = used;
used.prev = cur;
cur.prev.next = cur;
}
used = cur;
}
}
if (cur.deallocate(t))
{
if (cur.next is cur)
{
}
else
{
if (used is cur)
{
used = used.next;
}
cur.next.prev = cur.prev;
cur.prev.next = cur.next;
delete cur;
}
}
}
    void collect(T*[] t)
{
if (t)
delete t;
}
    bool collect(bool all = true)
{
used = null;
if (fresh !is null)
{
nextFresh = 0;
fresh.freeList = null;
}
return true;
}
    void config(size_t chunks, int allocate = 0)
{
}
}
}
    template DefaultCollect(T)
{
static if((T).sizeof + (void*).sizeof * 3 + (uint).sizeof >= 4095 / 2)
{
    alias Collect!(T) DefaultCollect;
}
else
{
    alias GCChunk!(T,(4095 - (void*).sizeof * 3 - (uint).sizeof) / (T).sizeof) DefaultCollect;
}
}
}
else
{
    template DefaultCollect(T)
{
alias ChunkGC!(T) DefaultCollect;
}
}
}
