// D import file generated from 'tango/util/container/HashMap.d'
module tango.util.container.HashMap;
private import tango.util.container.Slink;

public import tango.util.container.Container;

private import tango.util.container.model.IContainer;

private import tango.core.Exception;

template HashMap(K,V,alias Hash = Container.hash,alias Reap = Container.reap,alias Heap = Container.DefaultCollect)
{
class HashMap : IContainer!(V)
{
    version (HashCache)
{
    private alias Slink!(V,K,false,true) Type;

}
else
{
    private alias Slink!(V,K) Type;

}
    private alias Type* Ref;

    private alias Heap!(Type) Alloc;

    private Ref[] table;

    private size_t count;

    private float loadFactor;

    private Alloc heap;

    private size_t mutation;

    this(float f = Container.defaultLoadFactor)
{
loadFactor = f;
}
    ~this()
{
reset;
}
    final Iterator iterator()
{
Iterator i = void;
i.mutation = mutation;
i.table = table;
i.owner = this;
i.cell = null;
i.row = 0;
return i;
}

    final int opApply(scope int delegate(ref K key, ref V value) dg)
{
return iterator.opApply(dg);
}

    final int opApply(scope int delegate(ref V value) dg)
{
return iterator.opApply(delegate (ref K k, ref V v)
{
return dg(v);
}
);
}

    const final @property size_t size()
{
return count;
}

    final bool add(K key, V element)
{
if (table is null)
resize(Container.defaultInitialBuckets);
auto hd = &table[Hash(key,table.length)];
auto node = *hd;
if (node is null)
{
*hd = heap.allocate.set(key,element,null);
increment;
}
else
{
auto p = node.findKey(key);
if (p)
{
if (element != p.value)
{
p.value = element;
mutate;
}
return false;
}
else
{
*hd = heap.allocate.set(key,element,node);
increment;
checkLoad;
}
}
return true;
}

    final bool add(K key, V element, K function(K) retain)
{
if (table is null)
resize(Container.defaultInitialBuckets);
auto hd = &table[Hash(key,table.length)];
auto node = *hd;
if (node is null)
{
*hd = heap.allocate.set(retain(key),element,null);
increment;
}
else
{
auto p = node.findKey(key);
if (p)
{
if (element != p.value)
{
p.value = element;
mutate;
}
return false;
}
else
{
*hd = heap.allocate.set(retain(key),element,node);
increment;
checkLoad;
}
}
return true;
}

    final bool get(K key, ref V element)
{
if (count)
{
auto p = table[Hash(key,table.length)];
if (p && (p = p.findKey(key)) !is null)
{
element = p.value;
return true;
}
}
return false;
}

    final V* opIn_r(K key)
{
if (count)
{
auto p = table[Hash(key,table.length)];
if (p && (p = p.findKey(key)) !is null)
return &p.value;
}
return null;
}

    final bool contains(V element)
{
return instances(element) > 0;
}

    final bool keyOf(V value, ref K key)
{
if (count)
foreach (list; table)
{
if (list)
{
auto p = list.find(value);
if (p)
{
key = p.key;
return true;
}
}
}
return false;
}

    final bool containsKey(K key)
{
if (count)
{
auto p = table[Hash(key,table.length)];
if (p && p.findKey(key))
return true;
}
return false;
}

    final bool containsPair(K key, V element)
{
if (count)
{
auto p = table[Hash(key,table.length)];
if (p && p.findPair(key,element))
return true;
}
return false;
}

    final @property HashMap dup()
{
auto clone = new HashMap!(K,V,Hash,Reap,Heap)(loadFactor);
if (count)
{
clone.buckets(buckets);
foreach (key, value; iterator)
{
clone.add(key,value);
}
}
return clone;
}

    final bool removeKey(K key)
{
V value;
return take(key,value);
}

    final bool replaceKey(K key, K replace)
{
if (count)
{
auto h = Hash(key,table.length);
auto hd = table[h];
auto trail = hd;
auto p = hd;
while (p)
{
auto n = p.next;
if (key == p.key)
{
if (p is hd)
table[h] = n;
else
trail.detachNext;
h = Hash(replace,table.length);
table[h] = p.set(replace,p.value,table[h]);
return true;
}
else
{
trail = p;
p = n;
}
}
}
return false;
}

    final bool replacePair(K key, V oldElement, V newElement)
{
if (count)
{
auto p = table[Hash(key,table.length)];
if (p)
{
auto c = p.findPair(key,oldElement);
if (c)
{
c.value = newElement;
mutate;
return true;
}
}
}
return false;
}

    final bool take(ref V element)
{
if (count)
foreach (ref list; table)
{
if (list)
{
auto p = list;
element = p.value;
list = p.next;
decrement(p);
return true;
}
}
return false;
}

    final bool take(K key, ref V value)
{
if (count)
{
auto p = &table[Hash(key,table.length)];
auto n = *p;
while ((n = *p) !is null)
if (key == n.key)
{
*p = n.next;
value = n.value;
decrement(n);
return true;
}
else
p = &n.next;
}
return false;
}

    final bool opIndexAssign(V element, K key)
{
return add(key,element);
}

    final V opIndex(K key)
{
auto p = opIn_r(key);
if (p)
return *p;
throw new NoSuchElementException("missing or invalid key");
}

    final size_t remove(IContainer!(V) e, bool all = false)
{
auto i = count;
foreach (value; e)
{
remove(value,all);
}
return i - count;
}

    final size_t remove(V element, bool all = false)
{
auto i = count;
if (i)
foreach (ref node; table)
{
auto p = node;
auto trail = node;
while (p)
{
auto n = p.next;
if (element == p.value)
{
decrement(p);
if (p is node)
{
node = n;
trail = n;
}
else
trail.next = n;
if (!all)
return i - count;
else
p = n;
}
else
{
trail = p;
p = n;
}
}
}
return i - count;
}

    final size_t replace(V oldElement, V newElement, bool all = false)
{
size_t i;
if (count && oldElement != newElement)
foreach (node; table)
{
while (node && (node = node.find(oldElement)) !is null)
{
++i;
mutate;
node.value = newElement;
if (!all)
return i;
}
}
return i;
}

    final HashMap clear()
{
return clear(false);
}

    final HashMap reset()
{
clear(true);
heap.collect(table);
table = null;
return this;
}

    final size_t buckets()
{
return table ? table.length : 0;
}

    final HashMap buckets(size_t cap)
{
if (cap < Container.defaultInitialBuckets)
cap = Container.defaultInitialBuckets;
if (cap !is buckets)
resize(cap);
return this;
}

    final HashMap buckets(size_t cap, float threshold)
{
loadFactor = threshold;
return buckets(cast(size_t)(cap / threshold) + 1);
}

    final HashMap cache(size_t chunk, size_t count = 0)
{
heap.config(chunk,count);
return this;
}

    const final float threshold()
{
return loadFactor;
}

    final void threshold(float desired)
{
assert(desired > 0);
loadFactor = desired;
if (table)
checkLoad;
}

    final V[] toArray(V[] dst = null)
{
if (dst.length < count)
dst.length = count;
size_t i = 0;
foreach (k, v; this)
{
dst[i++] = v;
}
return dst[0..count];
}

    const final bool isEmpty()
{
return count is 0;
}

    final HashMap check()
{
assert(!(table is null && count !is 0));
assert(table is null || table.length > 0);
assert(loadFactor > 0F);
if (table)
{
size_t c = 0;
{
for (size_t i = 0;
 i < table.length; ++i)
{
{
for (auto p = table[i];
 p; p = p.next)
{
{
++c;
assert(contains(p.value));
assert(containsKey(p.key));
assert(instances(p.value) >= 1);
assert(containsPair(p.key,p.value));
assert(Hash(p.key,table.length) is i);
}
}
}
}
}
assert(c is count);
}
return this;
}

    private size_t instances(V element)
{
size_t c = 0;
foreach (node; table)
{
if (node)
c += node.count(element);
}
return c;
}

    private HashMap checkLoad()
{
float fc = count;
float ft = table.length;
if (fc / ft > loadFactor)
resize(2 * cast(size_t)(fc / loadFactor) + 1);
return this;
}

    private void resize(size_t newCap)
{
auto newtab = heap.allocate(newCap);
mutate;
foreach (bucket; table)
{
while (bucket)
{
auto n = bucket.next;
version (HashCache)
{
auto h = n.cache;
}
else
{
auto h = Hash(bucket.key,newCap);
}

bucket.next = newtab[h];
newtab[h] = bucket;
bucket = n;
}
}
heap.collect(table);
table = newtab;
}

    private bool removeNode(Ref node, Ref* list)
{
auto p = list;
auto n = *p;
while ((n = *p) !is null)
if (n is node)
{
*p = n.next;
decrement(n);
return true;
}
else
p = &n.next;
return false;
}

    private final HashMap clear(bool all)
{
mutate;
if (heap.collect(all) is false)
foreach (v; table)
{
while (v)
{
auto n = v.next;
decrement(v);
v = n;
}
}
foreach (ref v; table)
{
v = null;
}
count = 0;
return this;
}


    private void increment()
{
++mutation;
++count;
}

    private void decrement(Ref p)
{
Reap(p.key,p.value);
heap.collect(p);
++mutation;
--count;
}

    private void mutate()
{
++mutation;
}

    private struct Iterator
{
    size_t row;
    Ref cell;
    Ref prior;
    Ref[] table;
    HashMap owner;
    size_t mutation;
    bool valid()
{
return owner.mutation is mutation;
}
    bool next(ref K k, ref V v)
{
auto n = next(k);
return n ? v = *n , true : false;
}
    V* next(ref K k)
{
while (cell is null)
if (row < table.length)
cell = table[row++];
else
return null;
prior = cell;
k = cell.key;
cell = cell.next;
return &prior.value;
}
    int opApply(scope int delegate(ref K key, ref V value) dg)
{
int result;
auto c = cell;
loop:
while (true)
{
while (c is null)
if (row < table.length)
c = table[row++];
else
break loop;
prior = c;
c = c.next;
if ((result = dg(prior.key,prior.value)) != 0)
break loop;
}
cell = c;
return result;
}
    bool remove()
{
if (prior)
if (owner.removeNode(prior,&table[row - 1]))
{
++mutation;
return true;
}
prior = null;
return false;
}
}

}
}
debug (HashMap)
{
    import tango.io.Stdout;
    import tango.core.Memory;
    import tango.time.StopWatch;
    void main();
}
