// D import file generated from 'tango/util/container/HashSet.d'
module tango.util.container.HashSet;
private import tango.util.container.Slink;

public import tango.util.container.Container;

private import tango.util.container.model.IContainer;

template HashSet(V,alias Hash = Container.hash,alias Reap = Container.reap,alias Heap = Container.DefaultCollect)
{
class HashSet : IContainer!(V)
{
    public alias Slink!(V) Type;

    private alias Type* Ref;

    private alias Heap!(Type) Alloc;

    private Ref[] table;

    private size_t count;

    private float loadFactor;

    private Alloc heap;

    private size_t mutation;

    this(float f = Container.defaultLoadFactor)
{
loadFactor = f;
}
    ~this()
{
reset;
}
    final Iterator iterator()
{
Iterator i = void;
i.mutation = mutation;
i.table = table;
i.owner = this;
i.cell = null;
i.row = 0;
return i;
}

    final int opApply(scope int delegate(ref V value) dg)
{
return iterator.opApply(dg);
}

    const final @property size_t size()
{
return count;
}

    final bool add(V element)
{
if (table is null)
resize(Container.defaultInitialBuckets);
auto h = Hash(element,table.length);
auto hd = table[h];
if (hd && hd.find(element))
return false;
table[h] = allocate.set(element,hd);
increment;
if (hd)
checkLoad;
return true;
}

    final bool contains(V element)
{
if (count)
{
auto p = table[Hash(element,table.length)];
if (p && p.find(element))
return true;
}
return false;
}

    final @property HashSet dup()
{
auto clone = new HashSet!(V,Hash,Reap,Heap)(loadFactor);
if (count)
{
clone.buckets(buckets);
foreach (value; iterator)
{
clone.add(value);
}
}
return clone;
}

    final size_t remove(V element, bool all)
{
return remove(element) ? 1 : 0;
}

    final bool remove(V element)
{
if (count)
{
auto h = Hash(element,table.length);
auto hd = table[h];
auto trail = hd;
auto p = hd;
while (p)
{
auto n = p.next;
if (element == p.value)
{
decrement(p);
if (p is table[h])
{
table[h] = n;
trail = n;
}
else
trail.next = n;
return true;
}
else
{
trail = p;
p = n;
}
}
}
return false;
}

    final size_t replace(V oldElement, V newElement, bool all)
{
return replace(oldElement,newElement) ? 1 : 0;
}

    final bool replace(V oldElement, V newElement)
{
if (count && oldElement != newElement)
if (contains(oldElement))
{
remove(oldElement);
add(newElement);
return true;
}
return false;
}

    final bool take(ref V element)
{
if (count)
foreach (ref list; table)
{
if (list)
{
auto p = list;
element = p.value;
list = p.next;
decrement(p);
return true;
}
}
return false;
}

    public void add(IContainer!(V) e)
{
foreach (value; e)
{
add(value);
}
}

    public size_t remove(IContainer!(V) e)
{
size_t c;
foreach (value; e)
{
if (remove(value))
++c;
}
return c;
}

    final HashSet clear()
{
return clear(false);
}

    final HashSet reset()
{
clear(true);
heap.collect(table);
table = null;
return this;
}

    final size_t buckets()
{
return table ? table.length : 0;
}

    final HashSet buckets(size_t cap)
{
if (cap < Container.defaultInitialBuckets)
cap = Container.defaultInitialBuckets;
if (cap !is buckets)
resize(cap);
return this;
}

    const final float threshold()
{
return loadFactor;
}

    final void threshold(float desired)
{
assert(desired > 0);
loadFactor = desired;
if (table)
checkLoad;
}

    final HashSet cache(size_t chunk, size_t count = 0)
{
heap.config(chunk,count);
return this;
}

    final V[] toArray(V[] dst = null)
{
if (dst.length < count)
dst.length = count;
size_t i = 0;
foreach (v; this)
{
dst[i++] = v;
}
return dst[0..count];
}

    const final bool isEmpty()
{
return count is 0;
}

    final HashSet check()
{
assert(!(table is null && count !is 0));
assert(table is null || table.length > 0);
assert(loadFactor > 0F);
if (table)
{
size_t c = 0;
{
for (size_t i = 0;
 i < table.length; ++i)
{
{
{
for (auto p = table[i];
 p; p = p.next)
{
{
++c;
assert(contains(p.value));
assert(Hash(p.value,table.length) is i);
}
}
}
}
}
}
assert(c is count);
}
return this;
}

    private Ref allocate()
{
return heap.allocate;
}

    private void checkLoad()
{
float fc = count;
float ft = table.length;
if (fc / ft > loadFactor)
resize(2 * cast(size_t)(fc / loadFactor) + 1);
}

    private void resize(size_t newCap)
{
auto newtab = heap.allocate(newCap);
mutate;
foreach (bucket; table)
{
while (bucket)
{
auto n = bucket.next;
auto h = Hash(bucket.value,newCap);
bucket.next = newtab[h];
newtab[h] = bucket;
bucket = n;
}
}
heap.collect(table);
table = newtab;
}

    private bool remove(Ref node, size_t row)
{
auto hd = table[row];
auto trail = hd;
auto p = hd;
while (p)
{
auto n = p.next;
if (p is node)
{
decrement(p);
if (p is hd)
table[row] = n;
else
trail.next = n;
return true;
}
else
{
trail = p;
p = n;
}
}
return false;
}

    private HashSet clear(bool all)
{
mutate;
if (heap.collect(all) is false)
foreach (ref v; table)
{
while (v)
{
auto n = v.next;
decrement(v);
v = n;
}
}
foreach (ref v; table)
{
v = null;
}
count = 0;
return this;
}

    private void increment()
{
++mutation;
++count;
}

    private void decrement(Ref p)
{
Reap(p.value);
heap.collect(p);
++mutation;
--count;
}

    private void mutate()
{
++mutation;
}

    private struct Iterator
{
    size_t row;
    Ref cell;
    Ref prior;
    Ref[] table;
    HashSet owner;
    size_t mutation;
    bool valid()
{
return owner.mutation is mutation;
}
    bool next(ref V v)
{
auto n = next;
return n ? v = *n , true : false;
}
    V* next()
{
while (cell is null)
if (row < table.length)
cell = table[row++];
else
return null;
prior = cell;
cell = cell.next;
return &prior.value;
}
    int opApply(scope int delegate(ref V value) dg)
{
int result;
auto c = cell;
loop:
while (true)
{
while (c is null)
if (row < table.length)
c = table[row++];
else
break loop;
prior = c;
c = c.next;
if ((result = dg(prior.value)) != 0)
break loop;
}
cell = c;
return result;
}
    bool remove()
{
if (prior)
if (owner.remove(prior,row - 1))
{
++mutation;
return true;
}
prior = null;
return false;
}
}

}
}
debug (HashSet)
{
    import tango.io.Stdout;
    import tango.core.Thread;
    import tango.time.StopWatch;
    void main();
}
