// D import file generated from 'tango/util/container/LinkedList.d'
module tango.util.container.LinkedList;
private import tango.util.container.Slink;

public import tango.util.container.Container;

private import tango.util.container.model.IContainer;

template LinkedList(V,alias Reap = Container.reap,alias Heap = Container.DefaultCollect)
{
class LinkedList : IContainer!(V)
{
    private alias Slink!(V) Type;

    private alias Type* Ref;

    private alias V* VRef;

    private alias Heap!(Type) Alloc;

    private size_t count;

    private Alloc heap;

    private size_t mutation;

    private Ref list;

    this()
{
this(null,0);
}
    protected this(Ref l, size_t c)
{
list = l;
count = c;
}

    ~this()
{
reset;
}
    final Iterator iterator()
{
Iterator i = void;
i.mutation = mutation;
i.node = list ? *(i.hook = &list) : null;
i.prior = null;
i.owner = this;
return i;
}

    final LinkedList cache(size_t chunk, size_t count = 0)
{
heap.config(chunk,count);
return this;
}

    final int opApply(scope int delegate(ref V value) dg)
{
return iterator.opApply(dg);
}

    const final @property size_t size()
{
return count;
}

    final @property LinkedList dup()
{
return new LinkedList!(V,Reap,Heap)(list ? list.copy(&heap.allocate) : null,count);
}

    final bool contains(V value)
{
if (count is 0)
return false;
return list.find(value) !is null;
}

    final V head()
{
return firstCell.value;
}

    final V tail()
{
return lastCell.value;
}

    final V get(size_t index)
{
return cellAt(index).value;
}

    final size_t first(V value, size_t startingIndex = 0)
{
if (list is null || startingIndex >= count)
return size_t.max;
if (startingIndex < 0)
startingIndex = 0;
auto p = list.nth(startingIndex);
if (p)
{
auto i = p.index(value);
if (i >= 0)
return i + startingIndex;
}
return size_t.max;
}

    final size_t last(V value, size_t startingIndex = 0)
{
if (list is null)
return size_t.max;
auto i = 0;
if (startingIndex >= count)
startingIndex = count - 1;
auto index = size_t.max;
auto p = list;
while (i <= startingIndex && p)
{
if (p.value == value)
index = i;
++i;
p = p.next;
}
return index;
}

    final LinkedList subset(size_t from, size_t length = size_t.max)
{
Ref newlist = null;
if (length > 0)
{
auto p = cellAt(from);
auto current = newlist = heap.allocate.set(p.value,null);
{
for (auto i = 1;
 i < length; ++i)
{
if ((p = p.next) is null)
length = i;
else
{
current.attach(heap.allocate.set(p.value,null));
current = current.next;
}
}
}
}
return new LinkedList(newlist,length);
}

    final LinkedList clear()
{
return clear(false);
}

    final LinkedList reset()
{
return clear(true);
}

    final bool take(ref V v)
{
if (count)
{
v = head;
removeHead;
return true;
}
return false;
}

    final LinkedList sort(Compare!(V) cmp)
{
if (list)
{
list = Ref.sort(list,cmp);
mutate;
}
return this;
}

    final LinkedList add(V value)
{
return prepend(value);
}

    final LinkedList prepend(V value)
{
list = heap.allocate.set(value,list);
increment;
return this;
}

    final size_t remove(V value, bool all = false)
{
auto c = count;
if (c)
{
auto p = list;
auto trail = p;
while (p)
{
auto n = p.next;
if (p.value == value)
{
decrement(p);
if (p is list)
{
list = n;
trail = n;
}
else
trail.next = n;
if (!all || count is 0)
break;
else
p = n;
}
else
{
trail = p;
p = n;
}
}
}
return c - count;
}

    final size_t replace(V oldElement, V newElement, bool all = false)
{
size_t c;
if (count && oldElement != newElement)
{
auto p = list.find(oldElement);
while (p)
{
++c;
mutate;
p.value = newElement;
if (!all)
break;
p = p.find(oldElement);
}
}
return c;
}

    final V head(V value)
{
auto cell = firstCell;
auto v = cell.value;
cell.value = value;
mutate;
return v;
}

    final V removeHead()
{
auto p = firstCell;
auto v = p.value;
list = p.next;
decrement(p);
return v;
}

    final LinkedList append(V value)
{
if (list is null)
prepend(value);
else
{
list.tail.next = heap.allocate.set(value,null);
increment;
}
return this;
}

    final V tail(V value)
{
auto p = lastCell;
auto v = p.value;
p.value = value;
mutate;
return v;
}

    final V removeTail()
{
if (firstCell.next is null)
return removeHead;
auto trail = list;
auto p = trail.next;
while (p.next)
{
trail = p;
p = p.next;
}
trail.next = null;
auto v = p.value;
decrement(p);
return v;
}

    final LinkedList addAt(size_t index, V value)
{
if (index is 0)
prepend(value);
else
{
cellAt(index - 1).attach(heap.allocate.set(value,null));
increment;
}
return this;
}

    final LinkedList removeAt(size_t index)
{
if (index is 0)
removeHead;
else
{
auto p = cellAt(index - 1);
auto t = p.next;
p.detachNext;
decrement(t);
}
return this;
}

    final LinkedList replaceAt(size_t index, V value)
{
cellAt(index).value = value;
mutate;
return this;
}

    final size_t prepend(IContainer!(V) e)
{
auto c = count;
splice_(e,null,list);
return count - c;
}

    final size_t append(IContainer!(V) e)
{
auto c = count;
if (list is null)
splice_(e,null,null);
else
splice_(e,list.tail,null);
return count - c;
}

    final size_t addAt(size_t index, IContainer!(V) e)
{
auto c = count;
if (index is 0)
splice_(e,null,list);
else
{
auto p = cellAt(index - 1);
splice_(e,p,p.next);
}
return count - c;
}

    final size_t removeRange(size_t fromIndex, size_t toIndex)
{
auto c = count;
if (fromIndex <= toIndex)
{
if (fromIndex is 0)
{
auto p = firstCell;
{
for (size_t i = fromIndex;
 i <= toIndex; ++i)
{
p = p.next;
}
}
list = p;
}
else
{
auto f = cellAt(fromIndex - 1);
auto p = f;
{
for (size_t i = fromIndex;
 i <= toIndex; ++i)
{
p = p.next;
}
}
f.next = p.next;
}
count -= toIndex - fromIndex + 1;
mutate;
}
return c - count;
}

    final V[] toArray(V[] dst = null)
{
if (dst.length < count)
dst.length = count;
size_t i = 0;
foreach (v; this)
{
dst[i++] = v;
}
return dst[0..count];
}

    const final bool isEmpty()
{
return count is 0;
}

    final LinkedList check()
{
assert((count is 0) is (list is null));
assert(list is null || list.count is size);
size_t c = 0;
{
for (Ref p = list;
 p; p = p.next)
{
{
assert(instances(p.value) > 0);
assert(contains(p.value));
++c;
}
}
}
assert(c is count);
return this;
}

    private size_t instances(V value)
{
if (count is 0)
return 0;
return list.count(value);
}

    private Ref firstCell()
{
checkIndex(0);
return list;
}

    private Ref lastCell()
{
checkIndex(0);
return list.tail;
}

    private Ref cellAt(size_t index)
{
checkIndex(index);
return list.nth(index);
}

    private void checkIndex(size_t index)
{
if (index >= count)
throw new Exception("out of range");
}

    private void splice_(IContainer!(V) e, Ref hd, Ref tl)
{
Ref newlist = null;
Ref current = null;
foreach (v; e)
{
increment;
auto p = heap.allocate.set(v,null);
if (newlist is null)
newlist = p;
else
current.next = p;
current = p;
}
if (current)
{
current.next = tl;
if (hd is null)
list = newlist;
else
hd.next = newlist;
}
}

    private LinkedList clear(bool all)
{
mutate;
if (heap.collect(all) is false && count)
{
auto p = list;
while (p)
{
auto n = p.next;
decrement(p);
p = n;
}
}
list = null;
count = 0;
return this;
}

    private void increment()
{
++mutation;
++count;
}

    private void decrement(Ref p)
{
Reap(p.value);
heap.collect(p);
++mutation;
--count;
}

    private void mutate()
{
++mutation;
}

    private struct Iterator
{
    Ref node;
    Ref* hook;
    Ref* prior;
    LinkedList owner;
    size_t mutation;
    bool valid()
{
return owner.mutation is mutation;
}
    bool next(ref V v)
{
auto n = next;
return n ? v = *n , true : false;
}
    V* next()
{
V* r;
if (node)
{
prior = hook;
r = &node.value;
node = *(hook = &node.next);
}
return r;
}
    void insert(V value)
{
*hook = owner.heap.allocate.set(value,*hook);
node = *hook;
next();
owner.increment;
mutation++;
}
    bool insertPrior(V value)
{
if (prior)
{
*prior = owner.heap.allocate.set(value,*prior);
prior = &(*prior).next;
owner.increment;
mutation++;
return true;
}
return false;
}
    int opApply(scope int delegate(ref V value) dg)
{
int result;
auto n = node;
while (n)
{
prior = hook;
hook = &n.next;
if ((result = dg(n.value)) != 0)
break;
n = *hook;
}
node = n;
return result;
}
    bool remove()
{
if (prior)
{
auto p = *prior;
*prior = p.next;
owner.decrement(p);
hook = prior;
prior = null;
++mutation;
return true;
}
return false;
}
}

}
}
debug (LinkedList)
{
    import tango.io.Stdout;
    import tango.core.Thread;
    import tango.time.StopWatch;
    void main();
}
