// D import file generated from 'tango/util/container/SortedMap.d'
module tango.util.container.SortedMap;
public import tango.util.container.Container;

private import tango.util.container.RedBlack;

private import tango.util.container.model.IContainer;

private import tango.core.Exception;

template SortedMap(K,V,alias Reap = Container.reap,alias Heap = Container.DefaultCollect)
{
class SortedMap : IContainer!(V)
{
    public alias RedBlack!(K,V) Type;

    private alias Type* Ref;

    private alias Heap!(Type) Alloc;

    private alias Compare!(K) Comparator;

    package Ref tree;

    private Alloc heap;

    private Comparator cmp;

    private Compare!(V) cmpElem;

    private 
{
    size_t count;
    size_t mutation;
}
    public this(Comparator c = null)
{
this(c,0);
}

    private this(Comparator c, size_t n)
{
count = n;
cmpElem = &compareElem;
cmp = c is null ? &compareKey : c;
}

    ~this()
{
reset;
}
    final Iterator iterator(bool forward = true)
{
Iterator i = void;
i.node = count ? forward ? tree.leftmost : tree.rightmost : null;
i.bump = forward ? &Iterator.fore : &Iterator.back;
i.mutation = mutation;
i.owner = this;
i.prior = null;
return i;
}

    final Iterator iterator(K key, bool forward)
{
Iterator i = iterator(forward);
i.node = count ? tree.findFirst(key,cmp,forward) : null;
return i;
}

    final SortedMap cache(size_t chunk, size_t count = 0)
{
heap.config(chunk,count);
return this;
}

    const final @property size_t size()
{
return count;
}

    final @property SortedMap dup()
{
auto clone = new SortedMap!(K,V,Reap,Heap)(cmp,count);
if (count)
clone.tree = tree.copyTree(&clone.heap.allocate);
return clone;
}

    final bool contains(V value)
{
if (count is 0)
return false;
return tree.findAttribute(value,cmpElem) !is null;
}

    final int opApply(scope int delegate(ref V value) dg)
{
return iterator.opApply(delegate (ref K k, ref V v)
{
return dg(v);
}
);
}

    final int opApply(scope int delegate(ref K key, ref V value) dg)
{
return iterator.opApply(dg);
}

    final SortedMap comparator(Comparator c)
{
if (cmp !is c)
{
cmp = c is null ? &compareKey : c;
if (count !is 0)
{
mutate;
auto t = tree.leftmost;
tree = null;
count = 0;
while (t)
{
add_(t.value,t.attribute,false);
t = t.successor;
}
}
}
return this;
}

    final bool containsKey(K key)
{
if (count is 0)
return false;
return tree.find(key,cmp) !is null;
}

    final bool containsPair(K key, V value)
{
if (count is 0)
return false;
return tree.find(key,value,cmp) !is null;
}

    final bool get(K key, ref V value)
{
if (count)
{
auto p = tree.find(key,cmp);
if (p)
{
value = p.attribute;
return true;
}
}
return false;
}

    K nearbyKey(K key, bool after)
{
if (count)
{
auto p = tree.findFirst(key,cmp,after);
if (p)
return p.value;
}
noSuchElement("no such key");
assert(0);
}
    K firstKey()
{
if (count)
return tree.leftmost.value;
noSuchElement("no such key");
assert(0);
}
    K lastKey()
{
if (count)
return tree.rightmost.value;
noSuchElement("no such key");
assert(0);
}
    final V* opIn_r(K key)
{
if (count)
{
auto p = tree.find(key,cmp);
if (p)
return &p.attribute;
}
return null;
}

    final bool keyOf(V value, ref K key)
{
if (count is 0)
return false;
auto p = tree.findAttribute(value,cmpElem);
if (p is null)
return false;
key = p.value;
return true;
}

    final SortedMap clear()
{
return clear(false);
}

    final SortedMap reset()
{
return clear(true);
}

    final size_t remove(IContainer!(V) e, bool all)
{
auto c = count;
foreach (v; e)
{
remove(v,all);
}
return c - count;
}

    final size_t remove(V value, bool all = false)
{
size_t i = count;
if (count)
{
auto p = tree.findAttribute(value,cmpElem);
while (p)
{
tree = p.remove(tree);
decrement(p);
if (!all || count is 0)
break;
p = tree.findAttribute(value,cmpElem);
}
}
return i - count;
}

    final size_t replace(V oldElement, V newElement, bool all = false)
{
size_t c;
if (count)
{
auto p = tree.findAttribute(oldElement,cmpElem);
while (p)
{
++c;
mutate;
p.attribute = newElement;
if (!all)
break;
p = tree.findAttribute(oldElement,cmpElem);
}
}
return c;
}

    final bool take(ref V v)
{
if (count)
{
auto p = tree.leftmost;
v = p.attribute;
tree = p.remove(tree);
decrement(p);
return true;
}
return false;
}

    final bool take(K key, ref V value)
{
if (count)
{
auto p = tree.find(key,cmp);
if (p)
{
value = p.attribute;
tree = p.remove(tree);
decrement(p);
return true;
}
}
return false;
}

    final bool add(K key, V value)
{
return add_(key,value,true);
}

    final bool opIndexAssign(V element, K key)
{
return add(key,element);
}

    final V opIndex(K key)
{
auto p = opIn_r(key);
if (p)
return *p;
noSuchElement("missing or invalid key");
assert(0);
}

    final bool removeKey(K key)
{
V value;
return take(key,value);
}

    final bool replacePair(K key, V oldElement, V newElement)
{
if (count)
{
auto p = tree.find(key,oldElement,cmp);
if (p)
{
p.attribute = newElement;
mutate;
return true;
}
}
return false;
}

    final V[] toArray(V[] dst = null)
{
if (dst.length < count)
dst.length = count;
size_t i = 0;
foreach (k, v; this)
{
dst[i++] = v;
}
return dst[0..count];
}

    const final bool isEmpty()
{
return count is 0;
}

    final SortedMap check()
{
assert(cmp !is null);
assert((count is 0) is (tree is null));
assert(tree is null || tree.size() is count);
if (tree)
{
tree.checkImplementation;
auto t = tree.leftmost;
K last = K.init;
while (t)
{
auto v = t.value;
assert(last is K.init || cmp(last,v) <= 0);
last = v;
t = t.successor;
}
}
return this;
}

    private void noSuchElement(immutable(char)[] msg)
{
throw new NoSuchElementException(msg);
}

    private size_t instances(V value)
{
if (count is 0)
return 0;
return tree.countAttribute(value,cmpElem);
}

    private final bool add_(K key, V value, bool checkOccurrence)
{
if (tree is null)
{
tree = heap.allocate.set(key,value);
increment;
}
else
{
auto t = tree;
for (;;)
{
{
int diff = cmp(key,t.value);
if (diff is 0 && checkOccurrence)
{
if (t.attribute != value)
{
t.attribute = value;
mutate;
}
return false;
}
else
if (diff <= 0)
{
if (t.left)
t = t.left;
else
{
tree = t.insertLeft(heap.allocate.set(key,value),tree);
increment;
break;
}
}
else
{
if (t.right)
t = t.right;
else
{
tree = t.insertRight(heap.allocate.set(key,value),tree);
increment;
break;
}
}
}
}
}
return true;
}


    private SortedMap clear(bool all)
{
mutate;
if (heap.collect(all) is false & count)
{
auto node = tree.leftmost;
while (node)
{
auto next = node.successor;
decrement(node);
node = next;
}
}
count = 0;
tree = null;
return this;
}

    private void remove(Ref node)
{
tree = node.remove(tree);
decrement(node);
}

    private void increment()
{
++mutation;
++count;
}

    private void decrement(Ref p)
{
Reap(p.value,p.attribute);
heap.collect(p);
++mutation;
--count;
}

    private void mutate()
{
++mutation;
}

    private static int compareKey(ref K fst, ref K snd)
{
if (fst is snd)
return 0;
return typeid(K).compare(&fst,&snd);
}


    private static int compareElem(ref V fst, ref V snd)
{
if (fst is snd)
return 0;
return typeid(V).compare(&fst,&snd);
}


    private struct Iterator
{
    Ref function(Ref) bump;
    Ref node;
    Ref prior;
    SortedMap owner;
    size_t mutation;
    bool valid()
{
return owner.mutation is mutation;
}
    bool next(ref K k, ref V v)
{
auto n = next(k);
return n ? v = *n , true : false;
}
    V* next(ref K k)
{
V* r;
if (node)
{
prior = node;
k = node.value;
r = &node.attribute;
node = bump(node);
}
return r;
}
    int opApply(scope int delegate(ref K key, ref V value) dg)
{
int result;
auto n = node;
while (n)
{
prior = n;
auto next = bump(n);
if ((result = dg(n.value,n.attribute)) != 0)
break;
n = next;
}
node = n;
return result;
}
    bool remove()
{
if (prior)
{
owner.remove(prior);
++mutation;
return true;
}
prior = null;
return false;
}
    Iterator reverse()
{
if (bump is &fore)
bump = &back;
else
bump = &fore;
return this;
}
    private static Ref fore(Ref p)
{
return p.successor;
}


    private static Ref back(Ref p)
{
return p.predecessor;
}


}

}
}
debug (SortedMap)
{
    import tango.io.Stdout;
    import tango.core.Thread;
    import tango.time.StopWatch;
    import tango.math.random.Kiss;
    void main();
}
