// D import file generated from 'tango/util/container/more/HashFile.d'
module tango.util.container.more.HashFile;
private import tango.io.device.FileMap;

template HashFile(K,V)
{
class HashFile
{
    struct BlockSize
{
    int capacity;
}
    private MappedFile file;

    private ubyte[] heap;

    private BlockSize block;

    private Record[K] map;

    private ulong fileSize;

    private ulong waterLine;

    public 
{
    enum BlockSize EighthK = {128 - 1};
    enum BlockSize QuarterK = {256 - 1};
    enum BlockSize HalfK = {512 - 1};
    enum BlockSize OneK = {1024 * 1 - 1};
    enum BlockSize TwoK = {1024 * 2 - 1};
    enum BlockSize FourK = {1024 * 4 - 1};
    enum BlockSize EightK = {1024 * 8 - 1};
    enum BlockSize SixteenK = {1024 * 16 - 1};
    enum BlockSize ThirtyTwoK = {1024 * 32 - 1};
    enum BlockSize SixtyFourK = {1024 * 64 - 1};
}
    this(const(char[]) path, BlockSize block, uint initialRecords = 100)
{
this.block = block;
file = new MappedFile(path);
fileSize = initialRecords * (block.capacity + 1);
heap = file.resize(fileSize);
}
    final const(char[]) path()
{
return file.path;
}

    const final ulong length()
{
return waterLine;
}

    final V get(K key, bool clear = false)
{
auto p = key in map;
if (p)
return p.read(this,clear);
return V.init;
}

    final void remove(K key)
{
map.remove(key);
}

    final void put(K key, V data, K function(K) retain = null)
{
auto r = key in map;
if (r)
r.write(this,data,block);
else
{
Record rr;
rr.write(this,data,block);
if (retain)
key = retain(key);
map[key] = rr;
}
}

    final void close()
{
if (file)
{
file.close;
file = null;
map = null;
}
}

    private struct Record
{
    private ulong offset;

    private 
{
    int used;
    int capacity = -1;
}
    V read(HashFile bucket, bool clear)
{
if (used)
{
auto ret = cast(V)bucket.heap[cast(size_t)offset..cast(size_t)(offset + used)];
if (clear)
used = 0;
return ret;
}
return V.init;
}
    void write(HashFile bucket, V data, BlockSize block)
{
this.used = data.length;
if (this.used > this.capacity)
createBucket(bucket,this.used,block);
bucket.heap[cast(size_t)offset..cast(size_t)(offset + used)] = cast(ubyte[])data;
}
    void createBucket(HashFile bucket, int bytes, BlockSize block)
{
this.offset = bucket.waterLine;
this.capacity = bytes + block.capacity & ~block.capacity;
bucket.waterLine += this.capacity;
if (bucket.waterLine > bucket.fileSize)
{
auto target = bucket.waterLine * 2;
debug (HashFile)
{
printf("growing file from %lld, %lld, to %lld\x0a",bucket.fileSize,bucket.waterLine,target);
}

bucket.heap = bucket.file.resize(bucket.fileSize = target);
}
}
}

}
}
debug (HashFile)
{
    extern (C) int printf(const(char)*,...);

    import tango.io.Path;
    import tango.io.Stdout;
    import tango.text.convert.Integer;
    void main()
{
alias HashFile!(char[],char[]) Bucket;
auto file = new Bucket("foo.map",Bucket.QuarterK,1);
char[16] tmp;
{
for (int i = 1;
 i < 1024; ++i)
{
file.put(format(tmp,i).dup,"blah");
}
}
auto s = file.get("1",true);
if (s.length)
Stdout.formatln("result '{}'",s);
s = file.get("1");
if (s.length)
Stdout.formatln("result '{}'",s);
file.close;
remove("foo.map");
}
}
