// D import file generated from 'tango/util/digest/Sha256.d'
module tango.util.digest.Sha256;
private import tango.core.ByteSwap;

public import tango.util.digest.Digest;

private import tango.util.digest.MerkleDamgard;

final class Sha256 : MerkleDamgard
{
    private uint[8] context;

    private enum uint padChar = 128;

    this()
{
}
    protected override void reset()
{
super.reset();
context[] = initial[];
}


    protected override void createDigest(ubyte[] buf);


    override uint digestSize()
{
return 32;
}

    protected override uint blockSize()
{
return 64;
}


    protected override uint addSize()
{
return 8;
}


    protected override void padMessage(ubyte[] data)
{
data[0] = padChar;
data[1..$] = 0;
}


    protected override void padLength(ubyte[] data, ulong length)
{
length <<= 3;
{
for (int j = cast(int)data.length - 1;
 j >= 0; j--)
{
data[$ - j - 1] = cast(ubyte)(length >> j * 8);
}
}
}


    protected override void transform(const(ubyte[]) input)
{
uint[64] W;
uint a,b,c,d,e,f,g,h;
uint j,t1,t2;
a = context[0];
b = context[1];
c = context[2];
d = context[3];
e = context[4];
f = context[5];
g = context[6];
h = context[7];
bigEndian32(input,W[0..16]);
{
for (j = 16; j < 64; j++)
{
{
W[j] = mix1(W[j - 2]) + W[j - 7] + mix0(W[j - 15]) + W[j - 16];
}
}
}
{
for (j = 0; j < 64; j++)
{
{
t1 = h + sum1(e) + Ch(e,f,g) + K[j] + W[j];
t2 = sum0(a) + Maj(a,b,c);
h = g;
g = f;
f = e;
e = d + t1;
d = c;
c = b;
b = a;
a = t1 + t2;
}
}
}
context[0] += a;
context[1] += b;
context[2] += c;
context[3] += d;
context[4] += e;
context[5] += f;
context[6] += g;
context[7] += h;
}


    private static uint Ch(uint x, uint y, uint z)
{
return x & y ^ ~x & z;
}


    private static uint Maj(uint x, uint y, uint z)
{
return x & y ^ x & z ^ y & z;
}


    private static uint sum0(uint x)
{
return rotateRight(x,2) ^ rotateRight(x,13) ^ rotateRight(x,22);
}


    private static uint sum1(uint x)
{
return rotateRight(x,6) ^ rotateRight(x,11) ^ rotateRight(x,25);
}


    private static uint mix0(uint x)
{
return rotateRight(x,7) ^ rotateRight(x,18) ^ shiftRight(x,3);
}


    private static uint mix1(uint x)
{
return rotateRight(x,17) ^ rotateRight(x,19) ^ shiftRight(x,10);
}


    private static uint rotateRight(uint x, uint n)
{
return x >> n | x << 32 - n;
}


    private static uint shiftRight(uint x, uint n)
{
return x >> n;
}


}

private enum uint[] K = [1116352408,1899447441,-1245643825u,-373957723u,961987163,1508970993,-1841331548u,-1424204075u,-670586216u,310598401,607225278,1426881987,1925078388,-2132889090u,-1680079193u,-1046744716u,-459576895u,-272742522u,264347078,604807628,770255983,1249150122,1555081692,1996064986,-1740746414u,-1473132947u,-1341970488u,-1084653625u,-958395405u,-710438585u,113926993,338241895,666307205,773529912,1294757372,1396182291,1695183700,1986661051,-2117940946u,-1838011259u,-1564481375u,-1474664885u,-1035236496u,-949202525u,-778901479u,-694614492u,-200395387u,275423344,430227734,506948616,659060556,883997877,958139571,1322822218,1537002063,1747873779,1955562222,2024104815,-2067236844u,-1933114872u,-1866530822u,-1538233109u,-1090935817u,-965641998u];

private enum uint[8] initial = [1779033703,-1150833019u,1013904242,-1521486534u,1359893119,-1694144372u,528734635,1541459225];

debug (UnitTest)
{
    }
