// D import file generated from 'tango/util/log/Log.d'
module tango.util.log.Log;
private import tango.sys.Common;

private import tango.time.Clock;

private import tango.core.Exception;

private import tango.io.model.IConduit;

private import tango.text.convert.Format;

private import tango.util.log.model.ILogger;

version (GNU)
{
    private import tango.core.Vararg;

    alias void* Arg;
    alias va_list ArgList;
}
else
{
    version (DigitalMars)
{
    private import tango.core.Vararg;

    alias void* Arg;
    alias va_list ArgList;
    version (X86_64)
{
    version = DigitalMarsX64;
}
}
else
{
    alias void* Arg;
    alias void* ArgList;
}
}
extern (C) private int memcmp(const(void)*, const(void)*, size_t);


version (Win32)
{
    private extern (Windows) int QueryPerformanceCounter(ulong* count);


    private extern (Windows) int QueryPerformanceFrequency(ulong* frequency);


}
alias ILogger.Level Level;
public struct Log
{
    public alias lookup getLogger;

    public alias formatln opCall;

    private static __gshared Hierarchy base;


    private static __gshared Time beginTime;


    version (Win32)
{
    private static __gshared double multiplier;


    private static __gshared ulong timerStart;


}
    private struct Pair
{
    const(char)[] name;
    Level value;
}

    private static __gshared Level[immutable(char)[]] map;


    private static __gshared Pair[] Pairs = [{"TRACE",Level.Trace},{"Trace",Level.Trace},{"trace",Level.Trace},{"INFO",Level.Info},{"Info",Level.Info},{"info",Level.Info},{"WARN",Level.Warn},{"Warn",Level.Warn},{"warn",Level.Warn},{"ERROR",Level.Error},{"Error",Level.Error},{"error",Level.Error},{"Fatal",Level.Fatal},{"FATAL",Level.Fatal},{"fatal",Level.Fatal},{"NONE",Level.None},{"None",Level.None},{"none",Level.None}];


    private immutable char[][] LevelNames = ["Trace","Info","Warn","Error","Fatal","None"];


    shared static this();
    static Level convert(const(char[]) name, Level def = Level.Trace)
{
auto p = name in map;
if (p)
return *p;
return def;
}

    @property static Time time();


    @property static Logger root()
{
return base.root;
}


    static Logger lookup(const(char[]) name)
{
return base.lookup(name);
}

    static const(char)[] convert(int level)
{
assert(level >= Level.Trace && level <= Level.None);
return LevelNames[level];
}

    static Hierarchy hierarchy()
{
return base;
}

    static void formatln(const(char[]) fmt,...);

    static void config(OutputStream stream, bool flush = true)
{
root.add(new AppendStream(stream,flush));
}

}

public class Logger : ILogger
{
    alias Level.Trace Trace;
    alias Level.Info Info;
    alias Level.Warn Warn;
    alias Level.Error Error;
    alias Level.Fatal Fatal;
    alias append opCall;
    interface Context
{
    const @property const(char)[] label();

    const bool enabled(Level setting, Level target);

}
    private 
{
    Logger next;
    Logger parent;
}
    private Hierarchy host_;

    private const(char)[] name_;

    private Level level_;

    private bool additive_;

    private Appender appender_;

    private char[] buffer_;

    private size_t buffer_size_;

    private this(Hierarchy host, const(char[]) name)
{
host_ = host;
level_ = Level.Trace;
additive_ = true;
name_ = name;
}

    final bool enabled(Level level = Level.Fatal)
{
return host_.context.enabled(level_,level);
}

    final bool trace()
{
return enabled(Level.Trace);
}

    final void trace(const(char[]) fmt,...);

    final bool info()
{
return enabled(Level.Info);
}

    final void info(const(char[]) fmt,...);

    final bool warn()
{
return enabled(Level.Warn);
}

    final void warn(const(char[]) fmt,...);

    final bool error()
{
return enabled(Level.Error);
}

    final void error(const(char[]) fmt,...);

    final bool fatal()
{
return enabled(Level.Fatal);
}

    final void fatal(const(char[]) fmt,...);

    final @property const(char)[] name()
{
size_t i = name_.length;
if (i > 0)
--i;
return name_[0..i];
}

    final Level level()
{
return level_;
}

    final Logger level(Level l)
{
return level(l,false);
}

    final Logger level(Level level, bool propagate);

    const final @property bool additive()
{
return additive_;
}

    final @property Logger additive(bool enabled)
{
additive_ = enabled;
return this;
}

    final Logger add(Appender another)
{
assert(another);
another.next = appender_;
appender_ = another;
return this;
}

    final Logger clear()
{
appender_ = null;
return this;
}

    final @property char[] buffer()
{
return buffer_;
}

    final @property Logger buffer(char[] buf)
{
buffer_ = buf;
buffer_size_ = buf.length;
return this;
}

    const final @property TimeSpan runtime()
{
return Clock.now - Log.beginTime;
}

    final Logger append(Level level, lazy const(char[]) exp)
{
if (host_.context.enabled(level_,level))
{
LogEvent event;
event.set(host_,level,exp,name.length ? name_[0..$ - 1] : "root");
append(event);
}
return this;
}

    private void append(LogEvent event);

    final char[] format(char[] buffer, const(char[]) formatStr,...);

    final Logger format(Level level, const(char[]) fmt, TypeInfo[] types, ArgList args)
{
if (types.length)
{
if (buffer_ is null)
formatWithDefaultBuffer(level,fmt,types,args);
else
formatWithProvidedBuffer(level,fmt,types,args);
}
else
append(level,fmt);
return this;
}

    private void formatWithDefaultBuffer(Level level, const(char)[] fmt, TypeInfo[] types, ArgList args)
{
char[2048] tmp = void;
formatWithBuffer(level,fmt,types,args,tmp);
}

    private void formatWithProvidedBuffer(Level level, const(char)[] fmt, TypeInfo[] types, ArgList args)
{
formatWithBuffer(level,fmt,types,args,buffer_);
buffer_.length = buffer_size_;
}

    private void formatWithBuffer(Level level, const(char)[] fmt, TypeInfo[] types, ArgList args, char[] buf)
{
append(level,Format.vprint(buf,fmt,types,args));
}

    private final bool isChildOf(const(char[]) candidate)
{
auto len = candidate.length;
if (len < name_.length)
return len is 0 || memcmp(&candidate[0],&name_[0],len) is 0;
return false;
}


    private final bool isCloserAncestor(Logger other)
{
auto name = other.name_;
if (isChildOf(name))
if (parent is null || name.length >= parent.name_.length)
return true;
return false;
}


}

private class Hierarchy : Logger.Context
{
    private Logger root_;

    private 
{
    const(char)[] name_;
    const(char)[] address_;
}
    private Logger.Context context_;

    private Logger[char[]] loggers;

    this(const(char[]) name)
{
name_ = name;
address_ = "network";
root_ = new Logger(this,"");
context_ = this;
}
    const final const(char)[] label()
{
return "";
}

    const final bool enabled(Level level, Level test)
{
return test >= level;
}

    const final @property const(char)[] name()
{
return name_;
}

    final @property void name(const(char[]) name)
{
name_ = name;
}

    const final @property const(char)[] address()
{
return address_;
}

    final @property void address(const(char[]) address)
{
address_ = address;
}

    final @property Logger.Context context()
{
return context_;
}

    final @property void context(Logger.Context context)
{
context_ = context;
}

    final @property Logger root()
{
return root_;
}

    final Logger lookup(const(char[]) label);

    final int opApply(scope int delegate(ref Logger) dg);

    private Logger inject(const(char[]) label, scope Logger delegate(const(char[]) name) dg);

    private void insert(Logger l);

    private void update(Logger changed, bool force = false);

    private void propagate(Logger logger, Logger changed, bool force = false)
{
if (logger.isCloserAncestor(changed))
{
logger.parent = changed;
if (force)
logger.level_ = changed.level();
}
}

}

package struct LogEvent
{
    private 
{
    const(char)[] msg_;
    const(char)[] name_;
}
    private Time time_;

    private Level level_;

    private Hierarchy host_;

    void set(Hierarchy host, Level level, const(char[]) msg, const(char[]) name)
{
time_ = Log.time;
level_ = level;
host_ = host;
name_ = name;
msg_ = msg;
}
    const const(char)[] toString()
{
return msg_;
}

    const @property const(char)[] name()
{
return name_;
}

    const @property Level level()
{
return level_;
}

    @property Hierarchy host()
{
return host_;
}

    const @property TimeSpan span()
{
return time_ - Log.beginTime;
}

    const @property const(Time) time()
{
return time_;
}

    @property const(Time) started()
{
return Log.beginTime;
}

    @property const(char)[] levelName()
{
return Log.LevelNames[level_];
}

    static char[] toMilli(char[] s, TimeSpan time);

}

public class Appender
{
    alias int Mask;
    private Appender next_;

    private Level level_;

    private Layout layout_;

    private static __gshared Layout generic;


    interface Layout
{
    void format(LogEvent event, scope size_t delegate(const(void)[]) dg);
}
    const abstract @property Mask mask();

    const abstract @property const(char)[] name();

    abstract void append(LogEvent event);

    this()
{
layout_ = generic;
}
    shared static this();
    final @property Level level()
{
return level_;
}

    final @property Appender level(Level l)
{
level_ = l;
return this;
}

    protected Mask register(const(char)[] tag);

    @property void layout(Layout how)
{
layout_ = how ? how : generic;
}

    @property Layout layout()
{
return layout_;
}

    @property void next(Appender appender)
{
next_ = appender;
}

    @property Appender next()
{
return next_;
}

    void close()
{
}
}

public class AppendNull : Appender
{
    private Mask mask_;

    this(Layout how = null)
{
mask_ = register(name);
layout(how);
}
    const final override @property Mask mask()
{
return mask_;
}

    const final override @property const(char)[] name()
{
return this.classinfo.name;
}

    final override void append(LogEvent event);

}

public class AppendStream : Appender
{
    private Mask mask_;

    private bool flush_;

    private OutputStream stream_;

    this(OutputStream stream, bool flush = false, Appender.Layout how = null)
{
assert(stream);
mask_ = register(name);
stream_ = stream;
flush_ = flush;
layout(how);
}
    const final override @property Mask mask()
{
return mask_;
}

    const override @property const(char)[] name()
{
return this.classinfo.name;
}

    final override void append(LogEvent event);

}

public class LayoutTimer : Appender.Layout
{
    void format(LogEvent event, scope size_t delegate(const(void)[]) dg)
{
char[20] tmp = void;
dg(event.toMilli(tmp,event.span));
dg(" ");
dg(event.levelName);
dg(" [");
dg(event.name);
dg("] ");
dg(event.host.context.label);
dg("- ");
dg(event.toString());
}
}

debug (Log)
{
    import tango.io.Console;
    void main()
{
Log.config(Cerr.stream);
auto log = Log.lookup("fu.bar");
log.level = log.Trace;
log.trace("hello {}","world");
char[100] buf;
log(log.Trace,log.format(buf,"hello {}","world"));
}
}
